/***************************************************/
/*! \class Analog
 \brief
 
 The AudioDrome Analog class, implements 
 a model of generator to limited-band for the
 common prototypes.
 
 John Lane, Dan Hoory, Ed Martinez and Patty Wang
 Modelling Analog Synthesis with DSPs
 Computer Music Journal, 21:4, pp. 23-41, Winter 1997
 
 by Alessandro Petrolati 2008.
 www.alessandro-petrolati.it
 info@alessandro-petrolati.it
 
 This library is free software; you can redistribute it and/or
 modify it under the terms of the GNU Library General Public
 License as published by the Free Software Foundation; either
 version 2 of the License, or (at your option) any later version.
 
 This library is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 Library General Public License for more details.
 
 You should have received a copy of the GNU Library General Public
 License along with this library; if not, write to the Free
 Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
/***************************************************/

#ifndef ANALOG_H
#define ANALOG_H

#include "Sinusoide.h"
#include "RmsBalance.h"
#include "BlockerDc.h"
#include "LP_Butterworth.h"
#include "HighPass.h"

//  begin namespace
namespace Aud
{	
	class Analog : public AudioDrome
		{
		public:
			//! Public constructor
			Analog();
			
			//! Public destructor
			~Analog();
			
			//! Sets the oscillator frequency
			void setFrequency(const AudFloat& hz=440); /*! Sampling Rate Change SLOT */
			
			//! Sets the oscillator amplitude (normalized range)
			void setAmplitude(const AudFloat& resc=1);
			
			//! Sets the cut-off frequency of the low-pass filter
			void setLpCutOff(const AudFloat& hz=9000);
			
			//! Sets the number of harmonics
			void setHarmsNumber(const AudFloat& hn=16);
			
			//! Sets the waveform prototype
			void setOutputWaveform(const WAVESHAPE& type=SINE);
			
			//! Sets the downsampling factor.
			/*!
			 * The Output() function returns the same value for each sample block.
			 * After, dsp generates a new audio sample.
			 * Values == 0 or 1 means no downsampling.
			 */
			virtual void setDownSampling(const ul& samples=1) /*! Sampling Rate Change SIGNAL */
			{
				_osc1.setDownSampling(samples);
				_osc2.setDownSampling(samples);
				_atoneA.setDownSampling(samples);
				_atoneB.setDownSampling(samples);
				_lpbut1A.setDownSampling(samples);
				_lpbut2A.setDownSampling(samples);
				_lpbut3A.setDownSampling(samples);
				_lpbut1B.setDownSampling(samples);
				_lpbut2B.setDownSampling(samples);
				_lpbut3B.setDownSampling(samples);
				_rms.setDownSampling(samples);
				
				
				
				
				
				
				
				_dc.setDownSampling(samples);
				
				 AudioDrome::setDownSampling(samples);
			}
			
			//! PROCESSING is the primary function
			/*!
			 * You must call the function in the main loop, 
			 * or in the callback thread.
			 * This procedure not return any value, but write 
			 * the "_output_1" variable. 
			 * This signal can be read with the Output().
			 * Similarly, the Input() sets a value input for the DSP.
			 */
			virtual void PROCESSING()
			{
				this->Processing();	
			}
			
		protected:
			virtual void srUpdate()
			{ 
				this->setFrequency(_freqLast);
			}
			
			//! Processing the audio sample
			virtual void Processing();
			
		private:
			//State variables
			AudFloat _hpCutOff, _harms, _compareSig, _freqLast;
			
			//Generating Sine
			Sinusoide _osc1;
			Sinusoide _osc2;
			
			//Branch A and B
			HighPass _atoneA, _atoneB;
			LP_Butterworth _lpbut1A, _lpbut2A, _lpbut3A, _lpbut1B, _lpbut2B, _lpbut3B;
			
			//Rms balance
			RmsBalance _rms;
			BlockerDc _dc;
			
			AudFloat (Analog::*_outPtr) ();
			AudFloat Sawtooth();
			AudFloat Triangle();
			AudFloat Square();
			AudFloat Sine();		
		};
} //	end of namespace Aud
#endif
///EOF
