/***************************************************/
/*! \class AudError
 \brief
 
 This class, model various types of error,
 When you launch a AudError exception, it is possible
 to transfer a "void pointer" and/or a generic "double" value.
 You can also, personalize the error message with additional information.
 
 by Alessandro Petrolati 2008
 www.alessandro-petrolati.it
 info@alessandro-petrolati.it
 
 This library is free software; you can redistribute it and/or
 modify it under the terms of the GNU Library General Public
 License as published by the Free Software Foundation; either
 version 2 of the License, or (at your option) any later version.
 
 This library is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 Library General Public License for more details.
 
 You should have received a copy of the GNU Library General Public
 License along with this library; if not, write to the Free
 Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
/***************************************************/

#ifndef AUDERROR_H_
#define AUDERROR_H_

#include <exception>
#include <string>

//  begin namespace
namespace Aud
{
	enum ERROR
	{
		DIVIDE_BY_ZERO = 0,
		OUT_OF_RANGE,
		INVALID_LENGTH,
		NULL_POINTER,
		FILE_OPEN,
		FILE_CLOSE,
		FILE_WRITE,
		FILE_READ,
		NEGATIVE_VALUE,
		OVER_FLOW,
		UNDER_FLOW,
		NO_SPECIFIED
	};
	
	class AudError : public std::exception
	{
	public:
		//! Public constructor
		/*!
		 * Initialize the object with the specific type of current error. 
		 * If at the time of thrown the exception is not specified 
		 * any error, default is assigned NO_SPECIFIED.
		 */
		AudError(const ERROR& e = NO_SPECIFIED) : _criticalValue(0.0), _criticalPtr(NULL)
		{
			_enumAlias = e;
			switch (_enumAlias)
			{
				case (DIVIDE_BY_ZERO):
					_errorMsg = "Divide By Zero error";
					break;
				case (OUT_OF_RANGE):
					_errorMsg = "Out Of Range error";
					break;
				case (INVALID_LENGTH):
					_errorMsg = "Invalid Length Error";
					break;
				case (FILE_OPEN):
					_errorMsg = "File Open Error";
					break;
				case (FILE_CLOSE):
					_errorMsg = "File Close Error";
					break;
				case (FILE_WRITE):
					_errorMsg = "File Write Error";
					break;
				case (FILE_READ):
					_errorMsg = "File Read Error";
					break;
				case (NEGATIVE_VALUE):
					_errorMsg = "Negative Value Error";
					break;
				case (NO_SPECIFIED):
					_errorMsg = "No Specified Error";
					break;
					
				default:
					_errorMsg = "No Specified Error";
			}
		}
		
		//! Public destructor.
		virtual ~AudError() throw() {}
		
		//! Gets the thrown error message as a c-style string.
		virtual const char* what( void ) const throw() { return _errorMsg.c_str(); }
		
		//! Gets the error message default or the specific user message
		std::string
		getMsgError()
		{
			return _errorMsg;
		}
		
		//! Gets the ERROR (int) enumeration, the exception reference error
		ERROR getTypeError()
		{
			return _enumAlias;
		}
		
		//! Sets a particular message for the error
		/*!
		 * The function is invoked at the time of thrown an exception. 
		 * The message needed to get more information on managing 
		 * the exception (from catch).
		 */
		void setMsgError(const std::string& s)
		{
			_errorMsg = s;
		}
		
		//! Sets a critical value, championship to the launch of the exception, to be transferred
		/*!
		 * Ex:
		 * Buffer divide(double n, const Buffer& obj) throw(AudError)
		 * {
		 * Buffer temp(obj);				
		 * AudFloat val;
		 *
		 * for (unsigned int i = 0; i < obj.getSize(); i++)
		 * {
		 * if( (val = obj._table[i]) == 0)
		 * {
		 * AudError er;
		 * er.setMsgError("Divide by zero, not possible!");
		 * er.setCriticalPtr((void*)obj);
		 * er.setCriticalValue(i);
		 * throw er;
		 * }
		 * temp._table[i] = (n / val);
		 * }
		 * temp.writeGuardPoint ();
		 * return temp;
		 * }
		 */
		void setCriticalValue(const double& v)
		{
			_criticalValue = v;
		}		
		
		//! Sets a void pointer, championship to the launch of the exception, to be transferred
		/*!
		 * Es:
		 * Wave result(SINE);
		 * try
		 * {
		 * result =  1 / result;
		 * }
		 * catch (AudError& e) {
		 * if ( e.getErrorType() == INVALID_LENGTH)
		 * {
		 * cout << e.getErrorMsg() << endl;
		 *
		 * //a AudioDrome pointer from void* cast
		 * Buffer* ptr = (Buffer*) e.getCriticalPtr(); 
		 *
		 * //resolve pointer to location e.getCriticalValue()) and copy a value next to zero
		 * ptr->resize( e.getCriticalValue() );
		 * ptr->genSine();
		 * }
		 * }
		 */
		void setCriticalPtr(void* x)
		{
			_criticalPtr = x;
		}		
		
		//! Gets a void pointer, championship to the launch of the exception
		/*!
		 * Ex:
		 * Buffer divide(double n, const Buffer& obj) throw(AudError)
		 * {
		 * Buffer temp(obj);				
		 * AudFloat val;
		 *
		 * for (unsigned int i = 0; i < obj.getSize(); i++)
		 * {
		 * if( (val = obj._table[i]) == 0)
		 * {
		 * AudError er;
		 * er.setMsgError("Divide by zero, not possible!");
		 * er.setCriticalPtr((void*)obj);
		 * er.setCriticalValue(i);
		 * throw er;
		 * }
		 * temp._table[i] = (n / val);
		 * }
		 * temp.writeGuardPoint ();
		 * return temp;
		 * }
		 */
		void* getCriticalPtr()
		{
			return _criticalPtr;
		}			
		
		//! Gets a critical value, championship to the launch of the exception
		/*!
		 * This function can be called at launch the exception
		 * Ex:
		 * val = ...
		 * if(val == 0)
		 * {
		 * AudError er(DIVIDE_BY_ZERO);
		 * er.setMsgError("La tabella contiene valori uguale a zero: DIVIDE_BYZERO Error");
		 * er.setCriticalValue(val);
		 * throw er;
		 * }
		 */
		double getCriticalValue()
		{
			return _criticalValue;
		}		
		
	private:
		std::string _errorMsg;
		ERROR _enumAlias;
		double _criticalValue;
		void* _criticalPtr;
	};
}
//	end of namespace Aud
#endif /*AUDERROR_H_*/
