/***************************************************/
/*! \class AudioDrome
 \brief
 
 N.B. BSD/Allman[built.in]
 
 The AudioDrome base class, contains statics methods and global declarations.
 
 AudioDrome is a "open sources" c++ classes collection.
 Each class is the abstraction of an primitive DSP (Digital Signal Processing) algorithm.
 AudioDrome is conceived with a open structure, simple and extensible interface.
 This c++ "namespace" is addressed to software developers and researchers in the digital-sound domain.
 
 by Alessandro Petrolati 2006.
 www.alessandro-petrolati.it
 info@alessandro-petrolati.it
 
 This library is free software; you can redistribute it and/or
 modify it under the terms of the GNU Library General Public
 License as published by the Free Software Foundation; either
 version 2 of the License, or (at your option) any later version.
 
 This library is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 Library General Public License for more details.
 
 You should have received a copy of the GNU Library General Public
 License along with this library; if not, write to the Free
 Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
/***************************************************/

#ifndef AUDIODROME_H_
#define AUDIODROME_H_

#include "AudError.h"
#include "math.h"
#include <iostream>
#include <vector>
#include <typeinfo>

#ifdef WIN32
#include <windows.h>
#endif

//  begin namespace
namespace Aud
{
#ifndef M_PI
#define M_PI 3.1415926535897932384626433832795
#endif
	
#ifndef M_TWO_PI
#define M_TWO_PI 0x3.243f6a8885a308d313198a2e03707344ap1 //6.28318530717958647692528676655901
#endif
	
#define DEBUG std::cout << "AudioDrome Debug Marker: Obj name: " << this->getClassName() << " " << this << std::endl;
	
#if __DOUBLE_PRECISION__
	typedef double AudFloat;
	static std::string DataType = "AudioDrome: Using 64 bit Double Precision (double) Data";
	
#else
	typedef float AudFloat;
	static std::string DataType = "AudioDrome: Using 32 bit Single Precision(float) Data (Default)";
#endif
	
	//DataType used from Buffer and DelayTaps.
	//N.B. if unsigned, the performance endures an increase of 4 - 5 %
	typedef int ul;
	
	typedef float PaFloat; /* 
	 it works alone with "float" why portaudio 
	 it does not work with "double" (64 bit) */
	
	enum CHANNEL
	{
		LEFT = 0, 
		RIGTH, 
		STEREO
	};
	
	enum INTERPOLATION
	{
		DISABLED = 1, /*!< No Interpolation */
		LINEAR, /*!< Lineare Interpolation */
		CUBIC, /*!< Cubica Interpolation */
		VARGA /*!< Cubic Interpolation, Istvan Varga optimization */
	};
	
	
	enum WAVESHAPE
	{
		SINE = 1, /*!< Sine wave */
		COSINE,  /*!< Cosine wave */
		SAWTOOTH, /*!< Tooth of saws wave */
		SQUARE, /*!< Square wave */
		TRIANGLE, /*!< triangular wave */
		PULSE, /*!< Impulse wave */
		SIGMOID, /*!< Sigmoid wave */
		DOUBLESIGMOID /*!< Doube Sigmoid wave */
	};
	
	enum WINDOWS
	{
		BARLETT = 1,
		GAUSSIAN,
		HANNING,
		HAMMING,
		BLACKMANN,
		BLACKMANNHARRIS,
		BARLETTHANNING,
		NUTTALL,
		SINC
	};
	
	enum ALGORITHM
	{
		SIMPLE = 0,
		BEST,
		NONRECURSIVE,
		GOERTZEL
	};
	
	enum POLARITY
	{
		UNIPOLAR = 0,
		BIPOLAR
	};
	
	enum STREAM
	{
		OUTPUT = 0,
		INPUT,
		FULLDUPLEX
	};
	
	enum TRIGGER
	{
		UP=0,
		DOWN,
		BOTH
	};
	
	enum BLOCKERDC
	{
		NORMAL=0,
		RBGST
	};
	
	enum MODE
	{
		PEAKING = 0,
		LOWSHELVING,
		HIGHSHELVING
	};
	
	enum SCALING
	{
		NOSCALE = 0,
		CENTRE,
		RMS
	};
	
	class AudioDrome
		{
			friend class UpSampling;
		public:
			//! Public constructor
			AudioDrome();
			
			//! Public destructor
			virtual ~AudioDrome();
			
			//! Gets "true" if the computer architecture is Big Endian or "false" if Little Endian
			/*!
			 In the case of a WORD (16 bit), the hex number 0x0123 it will come stored like:
			 Little endian             Big endian
			 +----+----+              +----+----+
			 |0x23|0x01|              |0x01|0x23|
			 +----+----+              +----+----+
			 byte:    0    1                   0    1
			 
			 In the case of a DWORD (32 bit), the hex number 0x01234567 will come stored like:
			 Little endian             Big endian              
			 +----+----+----+----+    +----+----+----+----+
			 |0x67|0x45|0x23|0x01|    |0x01|0x23|0x45|0x67|
			 +----+----+----+----+    +----+----+----+----+
			 byte:    0    1    2    3         0    1    2    3
			 
			 */
			static bool isBigEndian();
			
			//! Print the computer architecture
			static void printEndianType();
			
			//! Gets the deciBell value from raw
			static AudFloat dbRaw(const AudFloat &input);
			
			//! Gets the raw value from db
			static AudFloat rawDb(const AudFloat &input);
			
			//! Gets the increment factor to raise/lower a frequency by a given amount of semitones
			/*!
			 * The value returned by the semitone function is a factor. 
			 * You can multiply a frequency by this factor to raise/lower it by the given amount of semitones.
			 * Ex:
			 * AudFloat LA = 440;
			 * AudFloat SIb = LA * semitone(1); /Namely: 440 * 1.05946
			 * Ex:
			 * AudFloat LA4 = 440;
			 * AudFloat La3 = LA4 * semitone(-12); //Namely: 440 / 2 
			 */
			static AudFloat semitone(const AudFloat &stone);
			
			//! Gets the increment factor to raise/lower a frequency by a given amount of cents
			/*!
			 * The value returned by the cents function is a factor. 
			 * You can multiply a frequency by this factor to raise/lower it by the given amount of cents.
			 * With 100 increments of one semitone.
			 * Ex:
			 * AudFloat Hz = 440;
			 * AudFloat Res = Hz * cents(50); /Namely: 440 * 0.52973
			 * increments of half semitone, 1/4 of tone.
			 */
			static AudFloat cents(const AudFloat &cnt);
			
			//! Gets the current class name
			/*!
			 * N.B. It's not a static function.
			 */
			inline std::string getClassName() const
			{
				return typeid(*this).name();
			}
			
			//! Gets the current class name for the specified object
			static std::string getClassName(AudioDrome* obj)
			{
				return typeid(*obj).name();
			}
			
			//! Gets the total instantiated number of AudioDrome objects
			static int getInstantiatedNumObj()
			{
				return _objectList.size();
			}
			
			//! Gets the pointer of a determinate object created
			static const AudioDrome* getInstantiedObjPointer(const unsigned int& indx) throw()
			{
				if(indx < _objectList.size() && indx >= 0)
					return _objectList[indx];
				else
				{
					AudError er(OUT_OF_RANGE);
					er.setMsgError("Warning!!! Index Out of range.");
					throw er;
				}
			}
			
			//! Print the names of all AudioDrome objects istantiated
			static void printNameInstantiedObj()
			{
				std::cout << "All AudioDrome objects istantiated: " << std::endl;
				for(int i = 0; i < _objectList.size(); i++)
					std::cout << _objectList[i]->getClassName() << " mem: " << _objectList[i] << std::endl;
			}
			
			//! Sets the Absolute Sampling Rate of AudioDrome Classes
			/*! Similar to a "Pattern Design" (Observable).
			 * This function, notify to all AudioDrome Objects (Observer).
			 * These call a virtual procedure srUpdate() that it provokes 
			 * a internal value sampling rate updating.
			 */
			static void setAbsoluteSR(const AudFloat &sr=44100); /*! Sampling Rate Change SIGNAL */
			
			//! Sets the downsampling factor.
			/*!
			 * The Output() function returns the same value for each sample block.
			 * After, dsp generates a new audio sample.
			 * Values == 0 or 1 means no downsampling.
			 */
			virtual void setDownSampling(const ul& samples=1); /*! Sampling Rate Change SIGNAL */
			
			//! Returns the downsampling factor.
			/*!
			 * The Output() function returns the same value for each sample block.
			 * After, dsp generates a new audio sample.
			 * Values == 0 or 1 means no downsampling.
			 */			
			ul getDownSampling()
			{
				return _smplBlock;
			}
			
			//! Gets the absolute sampling rate
			static const AudFloat& AudioDrome::getAbsoluteSR()
			{
				return _absoluteSR;
			}
			
			//! Gets the AudioDrome object sampling rate
			const AudFloat& getLocalSR()
			{
				return _localSR;
			}
			
			//! Gets the Shannon-Nyquist frequency (same the getAbsoluteSR()/2)
			static AudFloat getAbsoluteNyquist();
			
			
			//			//! Print the names of all AudioDrome listener objects
			//			static void printNameListenersObj()
			//			{
			//				std::cout << "All AudioDrome listener objects: " << std::endl;
			//				for(int i = 0; i < _objectList.size(); i++)
			//					if(_objectList[i]->isListener())
			//						std::cout << _objectList[i]->getClassName() << " mem: " << _objectList[i] << std::endl;
			//			}
			
			//			//! When the AbsoluteSR changes, the object ignore this
			//			inline void ignoreAbsoluteSRChange()
			//			{
			//				for(int i = 0; i < _objectList.size(); i++)
			//					if (_objectList[i] == this)
			//						_objectList.erase(_objectList.begin()+i);
			//			}
			//			
			//			//! When the AbsoluteSR changes, the internal class data appropriately update their own 
			//			inline void listenAbsoluteSRChange()
			//			{	
			//				//				for(int i = 0; i < _listeners.size(); i++)
			//				//					if (_listeners[i] == this)
			//				//						return;
			//				//				_listeners.push_back(this);
			//				
			//				//				this->_connection_t.unblock();
			//				for(int i = 0; i < _slots.size(); i++)
			//					_slots[i].unblock();
			//				
			//				_isListener = true;	
			//			}
			
			//! Gets the audio sample (left channel)
			/*! 
			 * This function returns a primari value.
			 * In the main loop needs to be called PROCESSING() always. 
			 */						
			virtual AudFloat Output()
			{
				return _output_1;
			}
			
			//! Gets the audio sample (rigth channel)
			/*! 
			 * This function returns a value secondary.
			 * In the main loop needs to be called PROCESSING() always. 
			 */						
			virtual AudFloat Output_2()
			{
#if __DEBUG__
				std::cout << "Current AudioDrome Object cannot return a secondary output" << std::endl;
#endif
				return _output_2;
			}
			
			//! Sets the audio sample (left/rigth channel)
			/*! 
			 * These functions set the input values for the objects.
			 * In the main loop needs to be called PROCESSING() always. 
			 */									
			virtual void Input(const AudFloat& input_1){_input_1 = input_1;}
			virtual void Input_2(const AudFloat& input_2){_input_2 = input_2;}
			virtual void Input(AudioDrome& obj){_input_1 = obj.Output();}
			virtual void Input_2(AudioDrome& obj){_input_2 = obj.Output();}			
			
			//! Input operator<< overload
			AudioDrome& operator<<(AudioDrome& obj)
			{
#if __DEBUG__
				std::cout << "AudioDrome& AudioDrome::operator<<(const AudioDrome&)" << std::endl;
#endif
				this->Input ( obj.Output() );
				this->Input_2 ( obj.Output_2() );
				return obj;
			}
			
			//! AudFloat Operator<< overload
			AudioDrome& operator<<(const AudFloat& val)
			{
#if __DEBUG__
				std::cout << "AudioDrome& AudioDrome::operator<<(const AudFloat&)" << std::endl;
#endif
				this->_input_1 = val;
				this->_input_2 = val;				
				return *this;
			}
			
			//! Output operator>> overload
			AudioDrome& operator>>(AudioDrome& obj)
			{
#if __DEBUG__
				std::cout << "AudioDrome& AudioDrome::operator>>(const AudioDrome&)" << std::endl;
#endif
				obj.Input( this->Output() );
				obj.Input_2( this->Output_2() );
				return obj;
			}
			
			//! AudFloat Operator= overload
			AudFloat operator[](const CHANNEL& chn)
			{
#if __DEBUG__
				std::cout << "AudFloat AudioDrome::operator[](const CHANNEL& chn))" << std::endl;
#endif
				switch (chn)
				{
					case LEFT:
						return this->Output();
					case RIGTH:
						return this->Output_2();
					default:
						return this->Output();
				}
			}
			
			//! AudFloat Operator+ overload
			AudioDrome& operator+(AudioDrome& obj)
			{
#if __DEBUG__
				std::cout << "AudioDrome& AudioDrome::operator+(const AudioDrome&)" << std::endl;
#endif
				this->_output_1 += obj.Output();
				this->_output_2 += obj.Output_2();
				return *this;
			}
			
			//! AudFloat Operator- overload
			AudioDrome& operator-(AudioDrome& obj)
			{
#if __DEBUG__
				std::cout << "AudioDrome& AudioDrome::operator-(const AudioDrome&)" << std::endl;
#endif
				this->_output_1 -= obj.Output();
				this->_output_2 -= obj.Output_2();
				return *this;
			}
			
			//! AudFloat Operator* overload
			AudioDrome& operator*(AudioDrome& obj)
			{
#if __DEBUG__
				std::cout << "AudioDrome& AudioDrome::operator*(const AudioDrome&)" << std::endl;
#endif
				this->_output_1 *= obj.Output();
				this->_output_2 *= obj.Output_2();
				return *this;
			}
			
			//! AudFloat Operator/ overload
			AudioDrome& operator/(AudioDrome& obj)
			{
#if __DEBUG__
				std::cout << "AudioDrome& AudioDrome::operator/(const AudioDrome&)" << std::endl;
#endif
				this->_output_1 /= obj.Output();
				this->_output_2 /= obj.Output_2();
				return *this;
			}
			
			//! AudFloat Operator+ overload
			AudioDrome& operator+(const AudFloat& val)
			{
#if __DEBUG__
				std::cout << "AudioDrome& AudioDrome::operator+(const AudFloat&)" << std::endl;
#endif
				this->_output_1 += val;
				this->_output_2 += val;				
				return *this;
			}
			
			//! AudFloat Operator- overload
			AudioDrome& operator-(const AudFloat& val)
			{
#if __DEBUG__
				std::cout << "AudioDrome& AudioDrome::operator-(const AudFloat&)" << std::endl;
#endif
				this->_output_1 -= val;
				this->_output_2 -= val;
				return *this;
			}
			
			//! AudFloat Operator* overload
			AudioDrome& operator*(const AudFloat& val)
			{
#if __DEBUG__
				std::cout << "AudioDrome& AudioDrome::operator*(const AudFloat&)" << std::endl;
#endif
				this->_output_1 *= val;
				this->_output_2 *= val;
				return *this;
			}
			
			//! AudFloat Operator* overload
			AudioDrome& operator/(const AudFloat& val)
			{
#if __DEBUG__
				std::cout << "AudioDrome& AudioDrome::operator/(const AudFloat&)" << std::endl;
#endif
				this->_output_1 /= val;
				this->_output_2 /= val;				
				return *this;
			}
			
			//! Operator() overload
			/*!
			 * The operator(), addresses the signal toward a specific channel.
			 * See the LEFT RIGTH enumerator
			 * It addresses the sample on the input stream audio
			 * The operator(), addresses the signal toward a specific channel
			 * Ex:
			 * adc (LEFT) >> myfloat;
			 * where LEFT is a enmeration, see AudioDrome.h
			 */
			//			AudioDrome& operator() (const CHANNEL& chn, AudioDrome& obj)
			//			{
			//				switch (chn)
			//				{
			//					case LEFT:
			//						this->_input_1 = obj.Output();
			//						break;
			//					case RIGTH:
			//						this->_input_2 = obj.Output();
			//						break;
			//					case STEREO:
			//						this->_input_1 = obj.Output();
			//						this->_input_2 = obj.Output_2();
			//					default:
			//						this->_input_1 = obj.Output();
			//						return *this;
			//				}
			//			}
			
			//! PROCESSING is the primary function
			/*!
			 * You must call the function in the main loop, 
			 * or in the callback thread.
			 * This procedure not return any value, but write 
			 * the "_output_1" variable. 
			 * This signal can be read with the Output().
			 * Similarly, the Input() sets a value input for the DSP.
			 */
			virtual void PROCESSING()
			{
				if(++_smplCount < _smplBlock)
					return;
				
				_smplCount = 0.0;
				this->Processing();
			}
			
		protected:
			AudFloat _input_1, _output_1, _input_2, _output_2;			
			AudFloat _localSR;
			ul _smplBlock;
			ul _smplCount;
			
			//! Sampling rate update manage
			/*!
			 * This function is redefined in the classes that using the
			 * _localSR member. When the absolute sampling frequency changes, if is
			 * a listener registered object, this procedure calls all the member functions.
			 */			
			virtual void srUpdate()
			{ 
#if __DEBUG__
				std::cout << this->getClassName() << " Does not use the _absoluteSR." << std::endl;
				std::cout << this->getClassName() << " You must redefine the void srUpdate()" << std::endl;	
				std::cout << "if you need to notify the Absolute Sampling Rate Change." << std::endl;	
#endif
			}
			
		private: 
			virtual void Processing(){};
			static AudFloat _absoluteSR;
			static std::vector<AudioDrome*> _objectList;
		};
}
//	end of namespace Aud
#endif /*AUDIODROME_H_*/
