/***************************************************/
/*! \class AudioIO
 \brief
 
 The AudioDrome AudioIO class, is a wrapper
 for the PortAudio_v19 pa_stable_v19_20071207
 by Ross Bencina
 
 This class are not affected by _absoluteSR changing notification,
 in fact the hardware audio stream, or VST (RTAS o AUDIOUNIT) sampling rate in a host
 system, is considered like _absoluteSR value.
 
 $Id: portaudio.h 1247 2007-08-11 16:29:09Z rossb $
 PortAudio Portable Real-Time Audio Library
 PortAudio API Header File
 Latest version available at: http://www.portaudio.com/
 
 Copyright (c) 1999-2002 Ross Bencina and Phil Burk
 
 by Alessandro Petrolati 2006.
 www.alessandro-petrolati.it
 info@alessandro-petrolati.it
 
 This library is free software; you can redistribute it and/or
 modify it under the terms of the GNU Library General Public
 License as published by the Free Software Foundation; either
 version 2 of the License, or (at your option) any later version.
 
 This library is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 Library General Public License for more details.
 
 You should have received a copy of the GNU Library General Public
 License along with this library; if not, write to the Free
 Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
/***************************************************/

#ifndef AUDIOIO_H
#define AUDIOIO_H
#ifdef __WITH_PORTAUDIO__

#include "AudioDrome.h"
#include "Portaudio.h"

#ifdef WIN32
#ifndef PA_NO_ASIO
#include "pa_asio.h"
#endif
#endif

//  begin namespace
namespace Aud
{	
	class AudioIO : public AudioDrome
		{
			friend class AudioDuplex;
			
		public:
			
			//! Public constructor
			AudioIO(const int &chnls = STEREO, 
					const unsigned long &framesPerBuffer = 1024,
					const PaSampleFormat &smpFormat = paFloat32,
					const PaStreamFlags &flags = paDitherOff | paClipOff);
			
			//! Public destructor
			virtual ~AudioIO();
			
			//! Set the samples for the buffer
			/*
			 * FramesPerBuffer The number of frames passed to the stream callback
			 * function, or the preferred block granularity for a blocking read/write stream.
			 * The special value paFramesPerBufferUnspecified (0) may be used to request that
			 * the stream callback will recieve an optimal (and possibly varying) number of
			 * frames based on host requirements and the requested latency settings.
			 * Note: With some host APIs, the use of non-zero framesPerBuffer for a callback
			 * stream may introduce an additional layer of buffering which could introduce
			 * additional latency. PortAudio guarantees that the additional latency
			 * will be kept to the theoretical minimum however, it is strongly recommended
			 * that a non-zero framesPerBuffer value only be used when your algorithm
			 * requires a fixed number of frames per stream callback.
			 */
			void setFramesPerBuffer(const unsigned long &framesPerBuffer=1024);
			
			//! Set flags used to control the behavior of a stream
			/*! They are passed as parameters to openstream();
			 * 
			 * paNoFlag   
			 *
			 * Disable default clipping of out of range samples.
			 * paClipOff    
			 *
			 * Disable default dithering.
			 * paDitherOff
			 *
			 * Flag requests that where possible a full duplex stream will not discard
			 * overflowed input samples without calling the stream callback. This flag is
			 * only valid for full duplex callback streams and only when used in combination
			 * with the paFramesPerBufferUnspecified (0) framesPerBuffer parameter. Using
			 * this flag incorrectly results in a paInvalidFlag error being returned from
			 * openStream().
			 * paNeverDropInput
			 *
			 * Call the stream callback to fill initial output buffers, rather than the
			 * default behavior of priming the buffers with zeros (silence). This flag has
			 * no effect for input-only and blocking read/write streams.
			 * paPrimeOutputBuffersUsingStreamCallback
			 *
			 * A mask specifying the platform specific bits.
			 * paPlatformSpecificFlags
			 *
			 * Multiple flags may be together.
			 * ex: setStreamFlags(paClipOff | paDitherOff);
			 */
			inline void setStreamFlags(const PaStreamFlags &flags = paDitherOff | paClipOff)
			{
				bool start = false;
				bool open = false;
				
				if(this->isStreamStarted())
				{
					this->stopStream();
					start = true;
				}
				if(this->isStreamOpened())
				{
					this->closeStream();
					open = true;
				}
				
				_streamFlags = flags;
				
				if(open)
					this->openStream();
				if(start)
					this->startStream();
			}
			
			//!Set the user CallBack function and data pointer
			/*! This routine will be called by the PortAudio engine when audio is needed.
			 * It may called at interrupt level on some machines so don't do anything
			 * that could mess up the system like calling malloc() or free().
			 * N.B.!!!
			 * if the Portaudio stream is opened in the CallBack mode, 
			 * you don't call in the main loop getSinal(), putSignal() or loopConnectIO().
			 * If you don't use CallBack function but you use the BlockingI/O,
			 * you can use this.
			 *
			 * Example of out callBack function:
			 *
			 * int MyCallback(const void *input, void *output, unsigned long frameCount,
			 *                const PaStreamCallbackTimeInfo* timeInfo, 
			 *                PaStreamCallbackFlags statusFlags, void *userData )
			 * {
			 *    AudioDrome::Granulator *myObj = (AudioDrome::Granulator*) userData;
			 *
			 *    (void) input; // Prevent unused variable warning.
			 *    PaFloat *out = (PaFloat*) output;
			 *
			 *    for( unsigned int i = 0; i < frameCount; i++ )
			 *    {
			 *       muObj->PROCESSING();
			 *	     *out++ = muObj->Output();       //left 
			 *	     *out++ = myObj->Output_2();  // right 
			 *    }
			 *    return 0;
		     * }
			 
			 * Example of input output callBack function:
			 *
			 * int MyCallback(const void *input, void *output, unsigned long frameCount,
			 *                const PaStreamCallbackTimeInfo* timeInfo, 
			 *                PaStreamCallbackFlags statusFlags, void *userData )
			 * {
			 *    (void) userData; // Prevent unused variable warning. 
			 *    PaFloat *in = (PaFloat*) output;
			 *    PaFloat *out = (PaFloat*) output;
			 *
			 *    for( unsigned int i = 0; i < frameCount; i++ )
			 *    {
			 *	     *out++ = *in++;
			 *	     *out++ = *in++;
			 *    }
			 *    return 0;
		     * }			 
			 */ 
			inline void setStreamCallBack(PaStreamCallback* myCallBack = NULL, void* myData=NULL)
			{
				
				bool flag = false;
				if(this->isStreamStarted())
				{
					this->stopStream();
					flag = true;
				}
				if(this->isStreamOpened())
					this->closeStream();
				
				_streamCallback =  myCallBack;
				_data = myData;
				
				if(!this->isStreamOpened())
					this->openStream();
				if(flag)
					this->startStream();
			}
			
			//! Select a input audio device
			void setDevice(const PaDeviceIndex &devIndx=-1);
			
			//! Retrieve the index of the default input device. 
			/* 
			 * The result can be used in the inputDevice
			 * parameter to setInputParameters()
			 * The default input device index for the default host API, or paNoDevice
			 * if no default input device is available or an error was encountered.
			 */			
			inline PaDeviceIndex  getDefaultInputDevice()
			{
				return Pa_GetDefaultInputDevice();
			}
			
			//! Retrieve the index of the default output device. 
			/* 
			 * The result can be used in the outputDevice
			 * parameter to setOutputParameters()
			 * The default output device index for the default host API, or paNoDevice
			 * if no default output device is available or an error was encountered.
			 */			
			inline PaDeviceIndex getDefaultOutputDevice()
			{
				return Pa_GetDefaultOutputDevice();
			}
			
			//! Returns the frames for the buffer
			/*!
			 * Write Block I/O
			 * Simple interleaved  buffer
			 *  ---   ---   ---   --- 
			 * | L | | R | | L | | R | _samplesBuffer = _frameBuffer * _numChannels
			 *  ---   ---   ---   ---
			 *
			 * Bidimensional array block I/O
			 * PaFloat (*_buffer)[4]; 
			 *  ---   ---   ---   --- 
			 * | R | | R | | R | | R | _______
			 *  ---   ---   ---   ---	 	  \
			 *  ---   ---   ---   ---           _numBuffer
			 * | L | | L | | L | | L | _______/
			 *  ---   ---   ---   ---
			 *  ---   ---   ---   --- 
			 * | 0 | | 1 | | 2 | | 3 |--------- _frameBuffer
			 *  ---   ---   ---   ---
			 */
			inline unsigned long getFramesPerBuffer() 
			{ 
				return _framesPerBuffer; 
			}
			
			//! Returns the opened channels number
			inline int getNumberOfChannels() 
			{ 
				return _parameters.channelCount; 
			}
			
			//! Retrieve CPU usage information for the specified stream
			/*!
			 * The "CPU Load" is a fraction of total CPU time consumed by a callback stream's
			 * audio processing routines including, but not limited to the client supplied
			 * stream callback. This function does not work with blocking read/write streams.
			 *
			 * This function may be called from the stream callback function or the
			 * application.
			 * A floating point value, typically between 0.0 and 1.0, where 1.0 indicates
			 * that the stream callback is consuming the maximum number of CPU cycles possible
			 * to maintain real-time operation. A value of 0.5 would imply that PortAudio and
			 * the stream callback was consuming roughly 50% of the available CPU time. The
			 * return value may exceed 1.0. A value of 0.0 will always be returned for a
			 * blocking read/write stream, or if an error occurrs.
			 */
			inline double getStreamCpuLoad()
			{
				return Pa_GetStreamCpuLoad(_stream);
			}
			
			//! Retrieve the number of frames that can be read from the stream without waiting
			/*!
			 * Returns a non-negative value representing the maximum number of frames
			 * that can be read from the stream without blocking or busy waiting or, a
			 * Portaudio error (which are always negative) if PortAudio is not initialized or an
			 * error is encountered.
			 */
			inline signed long getStreamReadAvailable()
			{
				return Pa_GetStreamReadAvailable(_stream);
			}
			
			//! Retrieve the number of frames that can be eritten from the stream without waiting
			/*!
			 * Returns a non-negative value representing the maximum number of frames
			 * that can be written from the stream without blocking or busy waiting or, a
			 * Portaudio error (which are always negative) if PortAudio is not initialized or an
			 * error is encountered.
			 */
			inline signed long getStreamWriteAvailable()
			{
				return Pa_GetStreamWriteAvailable(_stream);
			}
			
			//! Retrieve the release number of the currently running PortAudio build
			inline int getVersion()
			{
				return Pa_GetVersion();
			}
			
			//! Retrieve a textual description of the current PortAudio build
			/*!
			 * Ex: PortAudio V19-devel (built Aug 13 2007)
			 */
			inline const char* getTextVersion()
			{
				return Pa_GetVersionText ();
			}
			
			//! Determine whether the stream is active
			/*! 
			 * A stream is active after a successful call to startStream(), until it
			 * becomes inactive either as a result of a call to stopStream() or
			 * abortStream(), or as a result of a return value other than paContinue from
			 * the stream callback. In the latter case, the stream is considered inactive
			 * after the last buffer has finished playing.
			 * return true when the stream is active (ie playing or recording
			 * audio), false when not playing or, a PaErrorCode (which are always negative)
			 * if PortAudio is not initialized or an error is encountered.			 
			 */
			inline bool isStreamStarted()
			{
				//				return  (bool) Pa_IsStreamActive( _stream ) ;
				return _isStarted;
			}
			
			//! Determine whether the stream is stopped
			/*! 
			 * A stream is active after a successful call to startStream(), until it
			 * becomes inactive either as a result of a call to stopStream() or
			 * abortStream(). In the latter case, the stream is considered inactive
			 * after the last buffer has finished playing.
			 * Return 'true' when the stream is active (ie playing or recording
			 * audio), 'false' when not playing or, a Portaudio Error.
			 * if PortAudio is not initialized or an error is encountered.
			 */
			inline bool isStreamStopped()
			{
				//				return  (bool) Pa_IsStreamStopped( _stream );
				return !_isStarted;
			}
			
			//! Determine whether the stream is opened.
			/*!
			 * Return 'true' if the stream is opened
			 * 'false' if is closed.
			 */
			inline bool isStreamOpened()
			{
				return _isOpened;
			}
			
			//! Print all "Drivers Hosts" for the avaibles devices
			static void printAllHostDevices();
			
			//! Print some information about the specified stream
			inline void printStreamInfo()
			{
				if(_stream != NULL)
				{
					std::cout << "Stream Mem.: " << _stream << std::endl;
					std::cout << "The input latency of the stream in seconds: "
					<<  Pa_GetStreamInfo(_stream)->inputLatency << std::endl;
					std::cout << "The output latency of the stream in seconds: "
					<<  Pa_GetStreamInfo(_stream)->outputLatency << std::endl;
					std::cout << "The sample rate of the stream in Hertz: "
					<<  Pa_GetStreamInfo(_stream)->sampleRate << std::endl;
				}
			}
			
			//! Print some messages with the device informations
			static inline void printDeviceInfo()
			{
				if(_isInitialized)
				{
					std::cout << "Device name: "
					<<  Pa_GetDeviceInfo(Pa_GetDefaultInputDevice())->name << std::endl;
					std::cout << "Max Input Channels: "
					<<  Pa_GetDeviceInfo(Pa_GetDefaultInputDevice())->maxInputChannels << std::endl;
					std::cout << "Max Output Channels: "
					<<  Pa_GetDeviceInfo(Pa_GetDefaultInputDevice())->maxOutputChannels << std::endl;
				}
			}
			
			//! Meneage the Portaudio print errors
			/*
			 * Translate the supplied PortAudio error code into a human 
			 * readable message.
			 */
			void portAudioError(const std::string& myError="");
			
			//! Opens a Portaudio stream for either input, output or both
			virtual void openStream() = 0; /*! Sampling Rate Change SLOT */
			
			//! Closes an PortAudio stream. 
			/*!
			 * If the audio stream is active it
			 * discards any pending buffers as if abortStream() had been called.
			 */
			void closeStream();
			
			//! Commences audio processing
			void startStream();
			
			inline void sleep(const long &msecs)
			{
				Pa_Sleep(msecs);
			}
			
			//! Terminates audio processing
			/* 
			 * It waits until all pending audio buffers have been played before it returns.
			 */
			void stopStream();
			
			//! Terminates audio processing immediately without waiting for pending buffers to complete.
			void abortStream();
			
		protected:
			PaStreamParameters _parameters;
			PaError _err;
			PaStream* _stream;
			PaStreamFlags _streamFlags;
			PaStreamCallback* _streamCallback;
			unsigned long _framesPerBuffer;
			unsigned long _samplesBuffer;
			void* _data;
			static short _isInitialized;
			bool _isOpened;
			bool _isStarted;
			PaFloat *_buffer;
			int _channel;
			
			virtual void srUpdate()
			{ 
				if(this->isStreamOpened())
					this->openStream();
			}
		};
} //	end of namespace Aud
#endif
#endif
///EOF