/***************************************************/
/*! \class AudioIn
 \brief
 
 The AudioDrome AudioIn class, is a wrapper
 for the PortAudio_v19 pa_stable_v19_20071207
 by Ross Bencina
 
 This class are not affected by _absoluteSR changing notification,
 in fact the hardware audio stream, or VST (RTAS o AUDIOUNIT) sampling rate in a host
 system, is considered like _absoluteSR value.
 
 $Id: portaudio.h 1247 2007-08-11 16:29:09Z rossb $
 PortAudio Portable Real-Time Audio Library
 PortAudio API Header File
 Latest version available at: http://www.portaudio.com/
 
 Copyright (c) 1999-2002 Ross Bencina and Phil Burk
 
 by Alessandro Petrolati 2006.
 www.alessandro-petrolati.it
 info@alessandro-petrolati.it
 
 This library is free software; you can redistribute it and/or
 modify it under the terms of the GNU Library General Public
 License as published by the Free Software Foundation; either
 version 2 of the License, or (at your option) any later version.
 
 This library is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 Library General Public License for more details.
 
 You should have received a copy of the GNU Library General Public
 License along with this library; if not, write to the Free
 Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
/***************************************************/

#ifndef AUDIOIN_H
#define AUDIOIN_H
#ifdef __WITH_PORTAUDIO__

#include "AudioIO.h"

//  begin namespace
namespace Aud
{
	class AudioIn : public AudioIO
		{
		public:
			
			//! Public constructor
			AudioIn(const PaDeviceIndex& device =  -1, 
					const int &chnls = STEREO, 
					const unsigned long &framesPerBuffer = 1024,
					const PaSampleFormat &smpFormat = paFloat32,
					const PaStreamFlags &flags = paDitherOff | paClipOff,
					const bool &openStarted=true);
			
			//! Public destructor
			virtual ~AudioIn();
			
			//! Audio Input Stream structure settings
			/*!
			 * _inputParameters.device = dev;
			 * A valid device index in the range 0 to (Pa_GetDeviceCount()-1)
			 * specifying the device to be used or the special constant
			 * paUseHostApiSpecificDeviceSpecification which indicates that the actual
			 * device(s) to use are specified in hostApiSpecificStreamInfo.
			 * This field must not be set to paNoDevice.
			 *
			 * _inputParameters.channelCount = chnls; // 2 = Stereo output
			 * The number of channels of sound to be delivered to the
			 * stream callback or accessed by Pa_ReadStream() or Pa_WriteStream().
			 * It can range from 1 to the value of maxInputChannels in the
			 * PaDeviceInfo record for the device specified by the device parameter.
			 *
			 * _inputParameters.sampleFormat = paFloat32;
			 * The sample format of the buffer provided to the stream callback,
			 * a_ReadStream() or Pa_WriteStream(). 
			 * paFloat32
			 * paInt32         
			 * paInt24         
			 * paInt16          
			 * paInt8           
			 * paUInt8          
			 * paCustomFormat  
			 *
			 * _inputparameters.suggestedLatency=0.001; 
			 * The desired latency in seconds. Where practical, implementations should
			 * configure their latency based on these parameters, otherwise they may
			 * choose the closest viable latency instead. Unless the suggested latency
			 * is greater than the absolute upper limit for the device implementations
			 * should round the suggestedLatency up to the next practial value - ie to
			 * provide an equal or higher latency than suggestedLatency wherever possibe.
			 * Actual latency values for an open stream may be retrieved using the
			 * inputLatency and outputLatency fields of the PaStreamInfo structure
			 * returned by Pa_GetStreamInfo().		
			 */
			virtual void setParameters(const PaDeviceIndex &device =  -1, 
									   const int &chnls= 2, 
									   const PaSampleFormat &smpFormat = paFloat32)
			{
				bool start = false;
				bool open = false;
				
				if( this->isStreamStarted() )
				{
					this->stopStream();
					start = true;
				}
				
				if( this->isStreamOpened() )
				{
					this->closeStream();
					open = true;
				}	

				_parameters.device = ( PaDeviceIndex) 
				(device == -1 ? this->getDefaultInputDevice() : device);
				_parameters.channelCount = chnls; /* 2 = Stereo output */
				_parameters.sampleFormat = smpFormat;
				_parameters.suggestedLatency = 
				Pa_GetDeviceInfo( _parameters.device )->defaultLowInputLatency;
				_parameters.hostApiSpecificStreamInfo = NULL;
				
				this->setFramesPerBuffer(_framesPerBuffer);
				
				if(open)
					this->openStream();
				if(start)
					this->startStream();
			}
			
			//! Opens a Portaudio stream for input
			virtual void openStream();  /*! Sampling Rate Change SLOT */
			
		protected:
			/*! Read STEREO samples from an input stream
			 /*!
			 * The function doesn't return until
			 * the entire buffer has been filled - this may involve waiting for the operating
			 * system to supply the data.
			 * A pointer to an open stream previously created with openStream().
			 *
			 * A pointer to a buffer of sample frames. The buffer contains
			 * samples in the format specified by the setOutputParameters() 
			 * Portaudio sampleFormat field.
			 *
			 * The number of frames to be written from buffer. This parameter
			 * is not constrained to a specific range, however high performance applications
			 * will want to match this parameter to the setFramesPerBuffer() parameter used
			 * when opening the stream.
			 */			 
			virtual void Processing();
			
			/*! Read MULTICHANNELS samples from an input stream
			 /*!
			 * The function doesn't return until
			 * the entire buffer has been filled - this may involve waiting for the operating
			 * system to supply the data.
			 * A pointer to an open stream previously created with openStream().
			 *
			 * A pointer to a buffer of sample frames. The buffer contains
			 * samples in the format specified by the setOutputParameters() 
			 * Portaudio sampleFormat field.
			 *
			 * The number of frames to be written from buffer. This parameter
			 * is not constrained to a specific range, however high performance applications
			 * will want to match this parameter to the setFramesPerBuffer() parameter used
			 * when opening the stream.
			 */			 			
			//			void Processing(AudFloat *frame)
			//			{
			//				for (short i = 0; i < _parameters.channelCount; i++)
			//					frame[i] = (PaFloat) _buffer[_indexBufWrite + i];
			//				
			//				_indexBufWrite+=_parameters.channelCount;
			//				
			//				if (_indexBufWrite >= _samplesBuffer)
			//				{
			//					_indexBufWrite = 0;
			//					_err = Pa_ReadStream(_stream, _buffer, _framesPerBuffer );
			//					//					if (_err != paNoError )
			//					//						this->portAudioError( "AudioIO::putSignal()" );
			//				}
			//			}	
		protected:
			ul _indexBufRead;
		};
} //	end of namespace Aud
#endif
#endif
///EOF
