/***************************************************/
/*! \class BandPass Filter
 \brief
 
 The AudioDrome BandPass class, implements a second-order band-pass filter.
 
 Csound's code, based on Robin Whittle's
 ugrw2.c and ugrw2.h of 9 September 1995
 
 by Alessandro Petrolati 2006.
 www.alessandro-petrolati.it
 info@alessandro-petrolati.it
 
 This library is free software; you can redistribute it and/or
 modify it under the terms of the GNU Library General Public
 License as published by the Free Software Foundation; either
 version 2 of the License, or (at your option) any later version.
 
 This library is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 Library General Public License for more details.
 
 You should have received a copy of the GNU Library General Public
 License along with this library; if not, write to the Free
 Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
/***************************************************/

#ifndef BANDPASS_H
#define BANDPASS_H

#include "AudioDrome.h"

//  begin namespace
namespace Aud
{
	class BandPass : public AudioDrome
		{
		public:
			//! Public constructor
			BandPass(const AudFloat &centerFreq=5000, const AudFloat &bandWidth=200,
					 const SCALING &scale=CENTRE);
			
			//! Public destructor
			virtual ~BandPass();
			
			//! Sets the center frequency of the filter
			void setCenterFrequency(const AudFloat &centerFreq); /*! Sampling Rate Change SLOT */
			
			//! Sets the bandwidth of the filter
			void setBandWidth(const AudFloat &bandWidth); /*! Sampling Rate Change SLOT */
			
			//! Set the scale factor for the resonance of the filter
			/*!
			 * 0 = No scaling of the signal. Input gain c1 = 1
			 * 1 = Make gain at centre = 1, the attenuation is normalized on the peak (default)
			 * 2 = Higher gain, so "RMS gain" = 1, the amplitude value is compensated in RMS
			 */
			void setScaleType(const SCALING &scale);
			
		protected:
			//! Processing the audio sample
			virtual void Processing();
			
			virtual void scaling(const SCALING &scale);
			
			virtual void srUpdate()
			{ 
				this->setCenterFrequency(_centerFreq);
				this->setBandWidth(_bandWidth);
			}			
			
			AudFloat _c1;
			AudFloat _c2;
			AudFloat _c3;
			AudFloat _yt1;
			AudFloat _yt2;
			AudFloat _c2sqr;
			AudFloat _c3p1;
			AudFloat _c3t4;
			AudFloat _omc3;
			AudFloat _cosf;
			AudFloat _centerFreq, _bandWidth;
			SCALING _scale;
			
		};
} //	end of namespace Aud
#endif
///EOF

