/***************************************************/
/*! \class BlockerDc
 \brief
 
 The AudioDrome BlockerDc class, implements a recoursive filter DC blocker.
 
 by Alessandro Petrolati 2006.
 www.alessandro-petrolati.it
 info@alessandro-petrolati.it
 
 This library is free software; you can redistribute it and/or
 modify it under the terms of the GNU Library General Public
 License as published by the Free Software Foundation; either
 version 2 of the License, or (at your option) any later version.
 
 This library is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 Library General Public License for more details.
 
 You should have received a copy of the GNU Library General Public
 License along with this library; if not, write to the Free
 Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
/***************************************************/

#ifndef BLOCKERDC_H
#define BLOCKERDC_H

#include "AudioDrome.h"

//  begin namespace
namespace Aud
{
	class BlockerDc : public AudioDrome
		{
		public:
			//! Public constructor
			BlockerDc();
			
			//! Public destructor
			virtual ~BlockerDc();
			
			//! Sets the gain of the filter
			/*!
			 * Typically values comprised between 0.9 and 1 are used. 
			 * With SR = 44,1 KHz (Samping Rate), is advised a 0.995 gain.
			 */
			void setGain(const AudFloat& gain=0.995);
			
			//! Set the algorithm processing.
			
			void setAlgorithm(const BLOCKERDC &mode=NORMAL)
			{
				switch (mode)
				{
					case NORMAL:
						_modePtr = &BlockerDc::normal;
						break;
					case RBGST:
						_modePtr = &BlockerDc::gst;
						break;
					default:
						_modePtr = &BlockerDc::normal;						
				}
			}
			
		protected:
			//! Processing the audio sample
			/*!
			 * The NORMAL mode equation of the filter is following:			 
			 * Y[n] = X[n] - X[n-1] + (igain * Y[n-1]);
			 *
			 * or if mode == RBGST, this use RB-GST method of Ross Bencina.
			 */
			virtual void Processing();
			
		private:
			//! Getsthe audio sample
			/*!
			 * The equation of the filter is following:			 
			 * Y[n] = X[n] - X[n-1] + (igain * Y[n-1]);
			 *
			 * This implementazione is present in Csound as 
			 * "dcblock opcode" of Perry Cook.			 
			 */
			AudFloat normal(const AudFloat& input);
			
			//! Gets the audio sample
			/*!
			 * This use RB-GST method of Ross Bencina.
			 */
			AudFloat gst(const AudFloat& input);
			
			AudFloat _sample;
			AudFloat _oldInput;
			AudFloat _oldOutput;
			AudFloat _gain;
			AudFloat _pos;
			AudFloat _speed;
			AudFloat (BlockerDc::*_modePtr) (const AudFloat &);
		};
} //	end of namespace Aud
#endif
///EOF
