/***************************************************/
/*! \class BreakLine
 \brief
 
 The AudioDrome BreakLine class, constructs of the segments 
 between two points expressed like Time-value. 
 The segments assume a specific curving through setCurve().
 
 by Alessandro Petrolati 2006.
 www.alessandro-petrolati.it
 info@alessandro-petrolati.it
 
 This library is free software; you can redistribute it and/or
 modify it under the terms of the GNU Library General Public
 License as published by the Free Software Foundation; either
 version 2 of the License, or (at your option) any later version.
 
 This library is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 Library General Public License for more details.
 
 You should have received a copy of the GNU Library General Public
 License along with this library; if not, write to the Free
 Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
/***************************************************/

#ifndef BREAKLINE_H
#define BREAKLINE_H

#include "Line.h"
#include <vector>

//  begin namespace
namespace Aud
{
	class BreakLine : virtual public Line
	{
	public:
		//! Public constructor
		BreakLine();
		
		//! Public destructor
		virtual ~BreakLine();
		
		//! Adds a segment "break-point" to the list.
		/*!
		 * The "break-point" is expressed in "Time-value".
		 * Ex:
		 * BreakLine* line = new BreakLine ();
		 * line->setStart (.3);
		 * line->addBrkPoint (.5, -.6);
		 * line->addBrkPoint (.3, .7);
		 * line->addBrkPoint (1.6, -.6);
		 *
		 * The following example produces a sequence of 3 segments, the first
		 * segment begins with value 0.3, setup through the function setStart(),
		 * after 0.5 second catches up the value of -0.6 (first "break-point"), 
		 * after 0.3 second catches up the value of 0.7 (second "break-point") 
		 * and after 1.6 second it returns to 0.6 (third party "break-point").
		 */
		void addBreakPoint(const AudFloat &time, const AudFloat &value);
		
		//! Removes the last "break-point" segment from the list
		/*!
		 * The list is a LIFO (Last in, First out)
		 * removeLastBreakPoint() it is equivalent to pop_back()
		 */
		void removeLastBreakPoint();
		
		//! Sets the start value for the first segment
		void setStartValue(const AudFloat &start);
		
		//! Erases all the break-point from the list
		void clearAllBreakPoints();
		
		//! Adds a segment "break-point" in a specific position in the list
		void insertBreakPoint(const AudFloat &time, const AudFloat &value, const int& pos) throw(AudError)
		{ 
			_iterator = _brkPoint.begin();
			if(pos > 0 && pos-1 <=_brkPoint.size()/2)
			{
				int ndx = (pos-1) * 2;
				_brkPoint.insert( _iterator+ndx, time ); 
				_brkPoint.insert( _iterator+ndx+1,value ); 
			}
			else
				throw AudError(OUT_OF_RANGE);
		}
		
		//! Gets a Boolean true, when the execution of all break-points has happened
		/*!
		 * N.B. A value (true or false) is valid only if setLoopMode(true).
		 */
		bool getTrigger(){ return _endOfBreakPoints; }
		
		//! Set the loop mode
		/*!
		 * Default value is mode=false, the envelope
		 * it executes all the break points, 
		 * to the end maintains the last value
		 * If mode=true, all break-pointses are cyclically repeated
		 */
		void enableWrapAround(const bool& mode=true) { _mode = mode; }
		
		//! Resets and reinit the envelope state
		/*!
		 * When this function is called, the envelope comes cut and 
		 * leaves again from start. It comes only cut if the call 
		 * is < of the total duration of the envelope.
		 */
		inline void trigger()
		{
			Phasor::trigger();
			_endOfBreakPoints = true;
			_index = 2;
			if(!_brkPoint.empty())
			{
				Line::setStartValue(_first);
				Line::setTime(_brkPoint.at(0));
				Line::setEndValue(_brkPoint.at(1));
			}
		}
		
	protected:
		//! Processing the audio sample
		/*!
		 * The function calls the breakPoint(),
		 * this protected function maintains 
		 * trace of the performed segments. 
		 * The return value is calculated 
		 * from the Line::Processing() function.
		 */
		virtual void Processing();
		
		void breakPoint();
		
	private:
		std::vector <AudFloat> _brkPoint;
		std::vector<AudFloat>::iterator _iterator;
		unsigned short _index;
		AudFloat _first;
		bool _endOfBreakPoints, _mode;
		void setEnd(const AudFloat &end){};
	};
} //	end of namespace Aud
#endif
///EOF
