/***************************************************/
/*! \class Buffer
 \brief
 
 The AudioDrome Buffer implements an abstraction of a buffer array. 
 The class interface is a collection of methods for data manipulation.
 
 by Alessandro Petrolati 2006.
 www.alessandro-petrolati.it
 info@alessandro-petrolati.it
 
 This library is free software; you can redistribute it and/or
 modify it under the terms of the GNU Library General Public
 License as published by the Free Software Foundation; either
 version 2 of the License, or (at your option) any later version.
 
 This library is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 Library General Public License for more details.
 
 You should have received a copy of the GNU Library General Public
 License along with this library; if not, write to the Free
 Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
/***************************************************/

#ifndef BUFFER_H
#define BUFFER_H

#include "AudioDrome.h"

//  begin namespace
namespace Aud
{
	class Buffer
		{
			//! Define overloaded + (plus) operator
			/*
			 * The operator has commutative property, as add()
			 */
			inline friend Buffer operator+(double n, const Buffer& obj)
			{
#if __DEBUG__
				std::cout << "friend Buffer operator+(double, const Buffer&)" << std::endl;
#endif						
				Buffer temp(obj);
				return temp + n;
			}
			
			//! Define overloaded - (Subtraction) operator
			/*!
			 * The operator has NOT commutative property, and then redefined
			 */
			inline friend Buffer operator-(double n, const Buffer& obj)
			{
#if __DEBUG__
				std::cout << "friend Buffer operator-(double, const Buffer&)" << std::endl;
#endif						
				Buffer temp(obj);
				for (ul i = 0; i < obj.getSize(); i++)
					temp._table[i] = (n - obj._table[i]);
				temp.writeGuardPoint ();
				return temp;
			}
			
			//! Define overloaded * (Multiplication) operator
			/*
			 * The operator has commutative property, as mul()
			 */
			inline friend Buffer operator*(double n, const Buffer& obj)
			{
#if __DEBUG__
				std::cout << "friend Buffer operator*(double, const Buffer&)" << std::endl;
#endif						
				Buffer temp(obj);
				return temp * n;
			}
			
			//! Define overloaded / (Division) operator
			/*
			 * The operator has NOT commutative property, and then redefined
			 */
			inline friend Buffer operator/(double n, const Buffer& obj)
			{
#if __DEBUG__
				std::cout << "friend Buffer operator/(double, const Buffer&)" << std::endl;
#endif		
				Buffer temp(obj);				
				AudFloat val;
				
				for (ul i = 0; i < obj.getSize(); i++)
				{
					if( (val = obj._table[i]) == 0)
						val = 0.00000000000000000000001;
					temp._table[i] = (n / val);
				}
				temp.writeGuardPoint ();
				return temp;
			}
			
		public:
			//! Public constructor
			/*!
			 * Initialize the object with an integer value for the size of the array. 
			 * Memory is allocated dynamically.
			 */
			Buffer(const ul &length=4096);
			
			//! Public copy constructor
			Buffer(const Buffer& x);
			
			//! Public destructor
			virtual ~Buffer();
			
			//! Gets the pointer to first array allocation
			/*
			 * WARNING! 
			 * Goes directly to the representation of the data (array). 
			 * Not running checks on the index. 
			 * It can be used as a left-value and rigth-value.
			 * Gets a pointer to the first lease 
			 * Buffer. We recommend the use of this 
			 * Function very carefully.				
			 */
			AudFloat* getPtr() throw(AudError);
			
			//! Gets the array length in samples
			ul getSize() const;
			
			//! Gets the maximum peak-value of array
			/*!
			 * The function is two peak values, for a positive and a negative for. 
			 * The largest is returned				 
			 */
			AudFloat getPeak() throw(AudError);
			
			//! rites the sample in the specified location.
			/*!
			 * The index is normalized (Phase 0-1).
			 */
			void writeRaw(const AudFloat &phi, const AudFloat &input) throw(AudError);
			
			//! Gets the raw (not interpolated) sample in the specified location.
			/*!
			 * The index is normalized (Phase 0-1)
			 * and gets a NON INTERPOLATED value.
			 */
			AudFloat readRaw(const AudFloat &phi) throw(AudError);
			
			//! Gets the linear interpolated (two points) sample in the specified location.
			/*!
			 * The index is normalized (Phase 0-1)
			 * and gets a LINEAR INTERPOLATED value.
			 */
			AudFloat readInterp(const AudFloat &phi) throw(AudError);
			
			//! Gets the cubic interpolated (three points) sample in the specified location.
			/*!
			 * The index is normalized (Phase 0-1)
			 * and gets a CUBIC INTERPOLATED value.
			 */
			AudFloat readInterp3(const AudFloat &phi) throw(AudError);
			
			//! Gets the Cubic-Varga interpolated (three points) sample in the specified location.
			/*!
			 * The index is normalized (Phase 0-1)
			 * and gets a CUBIC INTERPOLATED value.
			 * How getInterp3 () but the code has been optimized by 
			 * Istvan Varga (2001) 
			 * N.B. The following implementation follows the model Csound 
			 * Implemented in "table3 opcode.
			 */
			AudFloat readInterp3Varga(const AudFloat &phi) throw(AudError);
			
			//! Gets the sample interpolated between the current value and the value of b object
			/*!
			 * With the index "phi", it can access to current array 
			 * and, in the same time, in the Buffer object (b) array. 
			 * The parameter "point", gets a mixing weighed between two values
			 * N.B. For performance reasons is not included any control.
			 * The current length-array, must be identical to that of the 
			 * Input object (b).
			 * In addition, this method returns the value interpolated.				 
			 */
			AudFloat getMorphing(const AudFloat &phi, Buffer* b,
								 const AudFloat &point=0.f);
			
			//! Write the "Extend Guard Point".
			/*!
			 * When you read the interpolated values of the array, 
			 * It is necessary to expand the "_size" overall.				 
			 * This procedure, writes the first value (index = 0) in the last position (_size + 1).
			 * This procedure, is mostly used by internal generation method. 
			 * (Same class, or derived classes).
			 */
			void writeGuardPoint();
			
			//! leans the array, writes 0.f value for each allocation
			void clear();
			
			//! Copy in the current array, the values of the object r
			/*!
			 * If the size of 'current array is different from that of r, 
			 * will be scaled on the same length of r, all the values will be overwritten. 
			 * N.B. After the copy, all original values, will be lost.
			 */
			void copy(const Buffer& r) throw(AudError);
			void copy(const AudFloat& r);
			
			//! Normalize the array (0-1)
			/*!
			 * This function calls getPeak(); all the values in the array, 
			 * are rescaled in the -1 to +1 range. The default factor is: 1.a0 / getPeak().			 
			 */
			void normalize();
			
			//! Rescaled the array to the value
			void rescale(const AudFloat& resc=1.0);
			
			//! Adds a constant value "offset" to all elements of the array
			void offSet(const AudFloat& offset=0.0);
			
			//! Resize the table with a new length
			/*!
			 * Creates a temporary array to copy all the currents values, 
			 * resizes the table and restore the values. If the length is different, calls the interpolation function, this producing new values appropriate.
			 * Finally destroys the object temporarily.			 
			 */
			void resize(const ul& newSize) throw(AudError);
			
			//! Compare the current object with x
			/*!
			 * If the current object is identical to x, return TRUE
			 * else return FALSE.
			 */
			bool compare(const Buffer& x);
			bool operator== (const Buffer& x);
			
			//! Adds to the currents values, the corresponding values of the object b
			/*!
			 * This method modifies the current array, as follows: 
			 * _table [N] += b._table [n]; 
			 * N.B. If the dimensions are different, the operation is skipped.			 
			 */
			void add(const Buffer& b) throw(AudError);
			void add(const AudFloat& b);
			
			//! Subtracts to the current values, the corresponding values of the object b
			/*!
			 * This method modifies the current array, as follows: 
			 * _table [N] -= b._table [n]; 
			 * N.B. If the dimensions are different, the operation is skipped.			 
			 */
			void sub(const Buffer& b) throw(AudError);
			void sub(const AudFloat& b);
			
			//! Multiplies to the current values, the corresponding values of the object b
			/*!
			 * This method modifies the current array, as follows: 
			 * _table [N] *= b._table [n]; 
			 * N.B. If the dimensions are different, the operation is skipped.			 
			 */
			void mul(const Buffer& b) throw(AudError);
			void mul(const AudFloat& b);
			
			//! Divide to the current values, the corresponding values of the object b
			/*!
			 * This method modifies the current array, as follows: 
			 * _table [N] /= b._table [n]; 
			 * N.B. If the dimensions are different, the operation is skipped.			 
			 */
			void div(const Buffer& b) throw(AudError);
			void div(const AudFloat& b) throw(AudError);		
			
			//*! Updates the current array values, with new values calculated by interpolation with the object x
			/*!
			 * N.B. If the dimensions are different, the operation is skipped.			 
			 * The "point" value must be between 0 and 1, see Interpolation.
			 * N.B. And 'not recommended the use of this function inside loops.			 
			 */
			void morphing(const Buffer& b, const AudFloat &point=0.f) throw(AudError);
			
			/*! Overloading Operators */
			//! Assignment
			/*
			 * N.B. If is invoked by a pointer, we must resolve as follows:
			 * Ex:
			 * Wave* wave = new Wave(LENGTH);
			 * wave->genSine();
			 * Wave* result = new Wave(LENGTH);
			 * *result = *wave;
			 */
			Buffer& operator= (const Buffer& r);
			Buffer& operator= (const AudFloat& r);
			
			//! Array subscript
			/*!
			 * The index "int" must be between 0 and size-1. 
			 * Gets a sample to a specified array location.
			 * It can be used only as r-value.
			 * N.B. If is invoked by a pointer, we must resolve as follows:
			 * Ex:
			 * Wave* wave = new Wave(LENGTH);
			 * wave->genSine();
			 * Wave* result = new Wave(LENGTH);
			 * *result = *wave;
			 */
			AudFloat& operator[] (const ul &i) throw(AudError);
			
			//! Object operators
			Buffer operator+(const Buffer& b);
			Buffer operator-(const Buffer& b);
			Buffer operator*(const Buffer& b);
			Buffer operator/(const Buffer& b) throw(AudError);
			
			//! AudFloat data operators
			Buffer operator+(const AudFloat& b);
			Buffer operator-(const AudFloat& b);
			Buffer operator*(const AudFloat& b);
			Buffer operator/(const AudFloat& b) throw(AudError);
			
		protected:
			ul _size;
			AudFloat* _table;
		private:
			ul _extendGuardPoint;
			AudFloat _realLimitNorm;
		};
} //	end of namespace Aud
#endif
///EOF
