/***************************************************/
/*! \class DecodeDTMF
 \brief
 
 The AudioDrome DecodeDTMF class, implements 
 a Dual Tone Multi Frequency decoder.
 http://zone.ni.com/cms/images/devzone/tut/a/75f1be30468.gif
 
 The pitch detector is based on the DFT (Discrete Fourier Trasform), 
 coefficients of the DFT are derived using the algorithm Goertzel.
 
 coef = 2.0 * cos( (2.0 * PI * k) / (float)_goertzel_n)) ;
 Where k = (int) (0.5 + ((float)_goertzel_n * target_freq) / SAMPLING_RATE));
 More simply: coef = 2.0 * cos( (2.0 * PI * target_freq) / SAMPLING_RATE );
 
 by Alessandro Petrolati 2008.
 www.alessandro-petrolati.com
 info@alessandro-petrolati.it
 
 This library is free software; you can redistribute it and/or
 modify it under the terms of the GNU Library General Public
 License as published by the Free Software Foundation; either
 version 2 of the License, or (at your option) any later version.
 
 This library is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 Library General Public License for more details.
 
 You should have received a copy of the GNU Library General Public
 License along with this library; if not, write to the Free
 Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
/***************************************************/

#ifndef DECODEDTMF_H
#define DECODEDTMF_H

#include "AudioDrome.h"

//  begin namespace
namespace Aud
{	
	class DecodeDTMF : public AudioDrome
		{
		public:
			//! Public constructor
			DecodeDTMF(const ul& frame=2000);
			
			//! Public destructor
			virtual ~DecodeDTMF();
			
			//! Sets the frame length in samples for the DFT (Discrete Fourier Trasform)
			void setFrameLength(const ul& frame=2000);
			
			//! Sets the frame time in millisecondsfor the DFT (Discrete Fourier Trasform)
			/*!
			 * The length of the frame is the time between bins of DFT.			 
			 * Ex:
			 * AudFloat time = 100; //Millisecondi
			 * AudFloat interupt = 50;
			 * GenerateDTMF genDTMF;
			 * genDTMF.setToneTime(time);
			 * genDTMF.setToneInterrupt(interupt);
			 * genDTMF.setPauseTime(0);
			 * string st = "*1D6B023*4#";
			 * genDTMF.setStringTone(st);
			 *
			 * AudFloat sr = _localSR;
			 * AudFloat samples = sr * ((time+interupt)*0.001); //*0.001 converte in sec.
			 *
			 * DecodeDTMF decDTMF;
			 * decDTMF.setFrameLength(samples);
			 * //Namely:
			 * //decDTMF.setTimeOfFrame(time+interupt);
			 *
			 * while(true)
			 * {
			 *   genDTMF.PROCESSING();
			 *   decDTMF.Input( genDTMF.Output() );
			 *   decDTMF.PROCESSING();
			 *   decDTMF.printDigit();
			 * }
			 */			
			inline void setFrameTime(const AudFloat& ms=150)
			{
				AudFloat freqTime = _localSR * (ms*0.001);
				setFrameLength(freqTime);
			}
			
			//! Gets the current char tone
			char getDigit();
			
			//! Print the current char tone
			/*!
			 * Prints the char only when goertzel() has found a valid char-tone.
			 * Equivalent to the following code:			 
			 * if( decDTMF.getSeeDigit() )
			 *   std::cout << decDTMF.getDigit() << " ";
			 */
			void printDigit();
			
			//! Gets "true" when the Goertzel DTF ( goertzel() ) has found a valid char-tone
			inline bool getSeeDigit() { return _see_digit; };
			
		protected:
			//! Decode the input signal
			/*!
			 * The algorithm Goertzel: 
			 * The pitch detector is based on DFT (Discrete Fourier Trasform) 
			 * Coefficients of the DFT are derived using the algorithm Goertzel.			 
			 * coef = 2.0 * cos( (2.0 * PI * k) / (float)_goertzel_n)) ;
			 * Where k = (int) (0.5 + ((float)_goertzel_n * target_freq) / SAMPLING_RATE));
			 * More simply: coef = 2.0 * cos( (2.0 * PI * target_freq) / SAMPLING_RATE );
			 */
			virtual void Processing();
			
			virtual void srUpdate()
			{ 
				this->calcCoeffs();
			}
			
			
		private:
			void goertzel();
			void calcCoeffs(); /*! Sampling Rate Change SLOT */
			char***  _row_col_ascii_codes; 
			char _lastDigit;
			bool _see_digit;
			ul _goertzel_n, _sample_count, _max_bins, _i;
			double _q0;
			double* _q1;
			double* _q2;
			double* _r;
			double* _freqs;
			double* _coefs;
		};
} //	end of namespace Aud
#endif
///EOF
