/***************************************************/
/*! \class DelayLine
 \brief
 
 The AudioDrome DelayLine, implements a simple fast
 non interpolated delay line.
 
 Simple delay line:
 * ===================================================
 *  input -------[..... d.....] --------- Processing()
 * ===================================================
 
 by Alessandro Petrolati 2006.
 www.alessandro-petrolati.it
 info@alessandro-petrolati.it
 
 This library is free software; you can redistribute it and/or
 modify it under the terms of the GNU Library General Public
 License as published by the Free Software Foundation; either
 version 2 of the License, or (at your option) any later version.
 
 This library is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 Library General Public License for more details.
 
 You should have received a copy of the GNU Library General Public
 License along with this library; if not, write to the Free
 Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
/***************************************************/

#ifndef DELAYLINE_H
#define DELAYLINE_H

#include "Buffer.h"

//  begin namespace
namespace Aud
{
	class DelayLine : public AudioDrome
		{
		public:
			//! Public constructor, (delay in seconds)
			DelayLine(const AudFloat &delayMax=1);
			
			//! Public destructor
			virtual ~DelayLine();
			
			//! Sets the maximum value possible for the delay
			void setMaxDelayTime(const AudFloat &delayMax); /*! Sampling Rate Change SLOT */
			
			//! Sets the delay in seconds
			void setDelayTime(const AudFloat &delay); /*! Sampling Rate Change SLOT */
			
			//! Sets the delay in samples
			void setDelaySample(const ul &sample); /*! Sampling Rate Change SLOT */
			
			//! Gets the maximum possible delay in seconds
			AudFloat getMaxDelayTime() const; 
			
			//! Gets the delay in seconds
			AudFloat getDelayTime() const;
			
			//! Gets the delay in samples
			AudFloat getDelaySample() const;
			
			//! Cleans the buffer of Delay Line
			void clearBuffer();
			
			//! Resets the buffer pointers
			/*
			 * Clear pointers reading and writing.
			 */
			inline void resetPointers()
			{
				_read = 0L;
				_write = _read + 1L;
			}
			
			//! Sets the interpolation type
			/*!
			 * Set function from used, the following are possible methods:
			 * DISABLED = not interpolated. 
			 * LINEAR linear interpolation (2 points). 
			 * CUBIC = cubic interpolation. 
			 * VARGA = cubic optimized by Istvan Varga.
			 */
			void setInterpolation(const INTERPOLATION &type=CUBIC);
			
		protected:
			//! Processing the audio sample delayed
			/*!
			 * ===================================================
			 *  Input() -------[..... d.....] --------- Output()
			 * ===================================================
			 *
			 * Processing the value delayed, not interpolated.
			 * This feature is less complex and more performant
			 * Not writes "GuardPoint" extention on the buffer and uses 
			 * a type "int" as increased phase.
			 */
			virtual void Processing();
			
			virtual void srUpdate()
			{ 
				this->setMaxDelayTime(_maxDelayTime);
				this->setDelayTime(_delayTime);
			}
			
			Buffer* _buffer;
			AudFloat _delayTime;
			AudFloat _maxDelayTime;
			AudFloat _delaySample;
			ul _write;
			ul _read;			
		};
} //	end of namespace Aud
#endif
///EOF

