/***************************************************/
/*! \class DelayTaps
 \brief
 
 The AudioDrome DelayTaps, implements a 
 interpolated delay line with multitaps outs.
 
 Simple delay line:
 * ===================================================
 *  input -------[..... d.....] --------- Processing()
 * ===================================================
 
 Multitaps delay line:
 * ===================================================
 *                 _____________________ getSignalTap()
 *                |      _______________ getSignalTap()
 * 				  |     |   ____________ getSignalTap()
 * 		          |     |  |
 *  input -------[..... d.....] --------- Processing()
 * ===================================================
 
 by Alessandro Petrolati 2006.
 www.alessandro-petrolati.it
 info@alessandro-petrolati.it
 
 This library is free software; you can redistribute it and/or
 modify it under the terms of the GNU Library General Public
 License as published by the Free Software Foundation; either
 version 2 of the License, or (at your option) any later version.
 
 This library is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 Library General Public License for more details.
 
 You should have received a copy of the GNU Library General Public
 License along with this library; if not, write to the Free
 Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
/***************************************************/

#ifndef DELAYTAPS_H
#define DELAYTAPS_H

#include "DelayLine.h"

//  begin namespace
namespace Aud
{
	class DelayTaps : public DelayLine
		{
		public:
			//! Public constructor, (delay in seconds)
			DelayTaps(const AudFloat &delayMax=1, const INTERPOLATION& type=CUBIC);
			
			//! Public destructor
			virtual ~DelayTaps();
			
			//! Resets the buffer pointers
			/*
			 * Clear pointers reading and writing.
			 */
			inline void resetPointers()
			{
				DelayLine::resetPointers();
				_readPhase = 0.f;
			}
			
			//! Sets the interpolation type
			/*!
			 * Set function from used, the following are possible methods:
			 * DISABLED = not interpolated. 
			 * LINEAR linear interpolation (2 points). 
			 * CUBIC = cubic interpolation. 
			 * VARGA = cubic optimized by Istvan Varga.
			 */
			void setInterpolation(const INTERPOLATION &type=CUBIC);
			
			//! Processing the audio sample delayed
			/*
			 * It receives as input, the value of 
			 * Delay given in SECONDS! 
			 * You can use several functions 
			 * getSignalTap() to read at different 
			 * times in the delay line.			 
			 * Ex 1:
			 * { //main loop:
			 *   mydel.Processing( sigIn );
			 *   AudFloat outdel_1 mydel.getSignalTap(0.23);
			 *   AudFloat outdel_2 mydel.getSignalTap(0.56);
			 *   AudFloat outdel_3 mydel.getSignalTap(0.786);
			 * }   
			 * ===================================================
			 *                   _____________________ OutputTap(time1)
			 *                  |      _______________ OutputTap(time2)
			 * 				    |     |     __________ OutputTap(time3)
			 * 		            |     |    |
			 *  Input() -------[..... d.....] -------- Output()
			 * ===================================================
			 */
			AudFloat OutputTap(const AudFloat &delTapTime);
			
		protected:
			//! Processing the audio sample delayed
			/*!
			 * ===================================================
			 *  Input() -------[..... d.....] --------- Output()
			 * ===================================================
			 *
			 * Processing the value delayed, interpolated.
			 * This feature is less complex and more performant
			 * Not writes "GuardPoint" extention on the buffer and uses 
			 * a type "int" as increased phase.
			 */
			virtual void Processing();
			
			INTERPOLATION _interpType;
			AudFloat (Buffer::*_interpPtr) (const AudFloat&);
			AudFloat _readPhase;	
		};
} //	end of namespace Aud
#endif
///EOF