/***************************************************/
/*! \class Distortion
 \brief
 
 The AudioDrome Distortion class, implements a 
 DNL (Distortion Not Linear) algorithm.
 Distort an audio signal via waveshaping and optional clipping
 
 The waveshaping is implemented by an object AudioDrome::Buffer.
 
 The distortion opcode uses modified hyperbolic tangent distortion.
 Coded by Hans Mikelson November 1998 -> biquad.c e compress.c
 
 by Alessandro Petrolati 2006.
 www.alessandro-petrolati.it
 info@alessandro-petrolati.it
 
 This library is free software; you can redistribute it and/or
 modify it under the terms of the GNU Library General Public
 License as published by the Free Software Foundation; either
 version 2 of the License, or (at your option) any later version.
 
 This library is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 Library General Public License for more details.
 
 You should have received a copy of the GNU Library General Public
 License along with this library; if not, write to the Free
 Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
/***************************************************/

#ifndef DISTORTION_H_
#define DISTORTION_H_

#include "AudioDrome.h"
#include "Wave.h"

namespace Aud
{
	class Distortion : public AudioDrome
		{
		public:
			
			//! Public constructor
			Distortion(Buffer* shape=NULL);
			
			//! Public destructor
			virtual ~Distortion();
			
			//! Sets the distrotion amount (generally in the range 0-1)
			void setAmountDistort(const AudFloat &dist);
			
			//! Sets a waveshaping AudioDrome::Buffer pointer
			void setShape(Buffer* shape);
			
		protected:
			//! Processing tha audio sample
			virtual void Processing();
			
			virtual void srUpdate()
			{ 
				this->calcCoefs();
			}
			
		private:
			Buffer* _shape;
			AudFloat _prvq;
			AudFloat _prvd;
			AudFloat _c1;
			AudFloat _c2;
			AudFloat _min_rms;
			AudFloat _hp;
			AudFloat _stor;
			AudFloat _dist;
			AudFloat sig;
			AudFloat phs;
			AudFloat val;
			AudFloat q;
			AudFloat rms;
			AudFloat dist;
			AudFloat dnew;
			AudFloat dcur;
			AudFloat dinc;
			bool _flag;
			
			void calcCoefs() /*! Sampling Rate Change SLOT */
			{
				AudFloat tpidsr = 2 * M_PI / _localSR;
				
				/*  and rms coefs */
				double b = 2.0 - cos((double) (_hp * tpidsr));
				_c2 = b - sqrt(b * b - 1.0);
				_c1 = 1.0 - _c2;
			}
		};
} //	end of namespace Aud
#endif /*DISTORTION_H_*/
