/***************************************************/
/*! \class EUVerb
 \brief
 
 The AudioDrome EUVerb class, implements 
 a reverb according to the model of 
 Eugenio Giordani.
 www.eugenio-giordani.it
 L.E.M.S.
 Pesaro 
 
 by Alessandro Petrolati 2008.
 www.alessandro-petrolati.it
 info@alessandro-petrolati.it
 
 This library is free software; you can redistribute it and/or
 modify it under the terms of the GNU Library General Public
 License as published by the Free Software Foundation; either
 version 2 of the License, or (at your option) any later version.
 
 This library is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 Library General Public License for more details.
 
 You should have received a copy of the GNU Library General Public
 License along with this library; if not, write to the Free
 Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
/***************************************************/

#ifndef EUVERB_H_
#define EUVERB_H_

#include "AllPass.h"
#include "KarplusStrong.h"
#include "ParametricEQ.h"
#include "Random.h"


namespace Aud
{
	
	class EUVerb : public AudioDrome
		{
		public:
			//! Publis constructor
			/*!
			 * Sets the defaults values.
			 */
			EUVerb(const INTERPOLATION& type=LINEAR);
			
			//! Public destructor
			virtual ~EUVerb();
			
			//! Sets the filters interpolation type
			/*! 
			 * The following methods, are possible: 
			 * DISABLED = not interpolated. 
			 * LINEAR linear interpolation (two points).
			 * CUBIC = cubic interpolation. 
			 * VARGA = cubic optimized by Istvan Varga.
			 * Default is CUBIC 
			 */
			void setInperpolation(const INTERPOLATION& type=CUBIC);
			
			//! Sets the noise inject amount
			/*! 
			 * This is a low amplitude signal, it is necessary for 
			 * destabilize the IIR-filters, always keeping active.
			 */
			void setNoiseInject(const AudFloat& amount=0.00000000001);
			
			//! Sets the master amplitude
			/*! 
			 * Values between 0 and 1; Default: 0.7 
			 */
			void setMasterLevel(const AudFloat& level=0.7);
			
			//! Sets the Direct signal rescale
			/*! 
			 * Values between 0 and 1; Default: 0.81 
			 */			
			void setDirectLevel(const AudFloat& dirlevel=0.81);
			
			//! Sets the reverberated signal
			/*! 
			 * Values between 0 and 1; Default: 0.15 
			 */
			void setReverberatedLevel(const AudFloat& revlevel=0.15);
			
			//! Sets the Early Reflection amplitude
			/*! 
			 * Values between 0 and 1; Default: 0.0 
			 */
			void setEarlyReflectionLevel(const AudFloat& earlevel=0.0);
			
			//! Sets the delay time scale of comb's filters 
			/*! 
			 * Values between 0.5 and 2; Default: 1.5 
			 */
			void setCombTimeScale(const AudFloat& scale=1.5);
			
			//! Sets the delay time of each comb
			/*! 
			 * Values expressed in milliseconds.
			 */
			void setComb1Time(const AudFloat& time1=54.5501);
			void setComb2Time(const AudFloat& time2=58.3473);
			void setComb3Time(const AudFloat& time3=62.4089);
			void setComb4Time(const AudFloat& time4=66.7532);
			void setComb5Time(const AudFloat& time5=71.4001);
			void setComb6Time(const AudFloat& time6=77.3317);
			
			//! Sets the T60; reverberation time in seconds
			/*! 
			 Values between 0.1 and 5; Default: 1.2
			 */
			void setT60(const AudFloat& t60=1.2);
			
			//! Sets the cut-off frequency of low-pass comb's filters
			/*! 
			 * values between 4000 and 16000; Default: 12000 
			 */
			void setCombCutOff(const AudFloat& cuttOff=12000);
			
			//! Sets the scale delay time for the allpass filters
			/*! 
			 * Values between 0.2 and 4; Default: 1.5 
			 */
			void setAllPassTimeScale(const AudFloat& scale=1.5);
			
			//! Sets the gain of feedback for the allpass
			/*! 
			 * Values between 0.1 and 0.9; Default: 0.5 
			 */
			void setAllPassGain(const AudFloat& gain=0.5);
			
			//! Sets the scale-gain feedback for each allpass
			/* 
			 * Values between 0 and 1; Default: 1
			 */
			void setAllPass1GainScale(const AudFloat& scale1=1.0);
			void setAllPass2GainScale(const AudFloat& scale2=1.0);
			void setAllPass3GainScale(const AudFloat& scale3=1.0);
			void setAllPass4GainScale(const AudFloat& scale4=1.0);
			void setAllPass5GainScale(const AudFloat& scale5=1.0);
			void setAllPass6GainScale(const AudFloat& scale6=1.0);
			
			//! Sets the Losh Filter parameter
			/*! 
			 * fc = frequency
			 * ab = gain
			 * fc values between 20 and 1000; Default: 20 
			 * ab values between -18 and 18 dB; Default: 0 dB
			 * q value; Default = 0
			 */
			void setLowShelfEqParam(const AudFloat& fc=20, const AudFloat& ab=0, const AudFloat& q=0.707);
			
			//! Sets the Hish Filter parameter
			/*! 
			 * fc = frequency
			 * ab = gain
			 * fc values between 1000 and 12000; Default: 1000 
			 * ab values between -18 and 18 dB; Default: 0 dB
			 * q value; Default = 0
			 */
			void setHighShelfEqParam(const AudFloat& fc=1000, const AudFloat& ab=0, const AudFloat& q=0.707);
			
			//! Sets the Peacking Filter parameters
			/*! 
			 * fc = frequency
			 * ab = gain
			 * fc values between 20 and 2000; Default: 20 
			 * ab values between -18 and 18 dB; Default: 0 dB
			 * q value; Default = 0
			 */			
			void setPeakEqParam(const AudFloat& fc=20, const AudFloat& ab=0, const AudFloat& q=0.707);
			
			//! Sets the time scale for the Early Reflection multitaps delay line
			/*! 
			 * Values between 0.35 and 4; Default: 1 
			 */
			void setEarlyTimeScale(const AudFloat& scale=1.0);
			
			//! Sets the pre-delay time in milliseconds
			/* 
			 * Values between 0.1 and 199; Default: 0.1 
			 */
			void setPreDelay(const AudFloat& time=0.1);
			
			//! Sets the post-delay time in milliseconds
			/* 
			 * This delay acts on the direct signal, relaunched.
			 * Values between 0.0 and 199; Default: 0.5 
			 */
			inline void setDirectDelay(const AudFloat& time=0.5)
			{
				_directDelay.setDelayTime(time * MILLISECONDS);
			}
			
			//! Gets the audio sample (rigth channel)
			/*! 
			 * This function returns a value secondary 
			 * In the main loop needs to be called Processing() always. 
			 * This process implements an additional branch of combs-delays, 
			 * in order to create a pseudo-stereo.			 
			 */
			AudFloat Output_2();

		protected:
			//! Processing the audio sample
			/*!
			 * It implements the Eugenio Giordani EUVerb algorithm.
			 * The Input() function accepts a single input, we consider the output 
			 * as Left Channel Output(), if you want to create a pseudo-stereo, 
			 * In addition you can call Output_2();
			 */
			virtual void Processing();
						
		private:
			//AudioDrome DSP objects
			Random _sputo;
			DelayTaps _preDelay, _directDelay, _multitap;
			ParametricEQ _eq1,_eq2,_eq3;
			AllPass _ap1,_ap2,_ap3,_ap4,_ap5,_ap6;
			KarplusStrong _cmb1,_cmb2,_cmb3,_cmb4,_cmb5,_cmb6;
			
			//Variable of state
			AudFloat _input; //segnale d'ingresso
			AudFloat _outEqPeaking;
			
			//Controls parameters
			AudFloat _combDelayScale,_erScale;
			AudFloat _earlyLevel;
			AudFloat _dirLevel;
			AudFloat _revLevel;
			AudFloat _masterLevel;
			
			AudFloat _loshEqFreq;
			AudFloat _loshEqGain; 
			AudFloat _hishEqFreq;
			AudFloat _hishEqGain;
			AudFloat _peakEqFreq;
			AudFloat _peakEqGain;
			
			AudFloat _delComb1,_delComb2,_delComb3,_delComb4,_delComb5,_delComb6;
			AudFloat _delCombTap1,_delCombTap2,_delCombTap3,_delCombTap4,_delCombTap5,_delCombTap6;			
			AudFloat _delAllPass1,_delAllPass2,_delAllPass3,_delAllPass4,_delAllPass5,_delAllPass6;
			AudFloat _gainApScale1, _gainApScale2, _gainApScale3, _gainApScale4, _gainApScale5, _gainApScale6;
			AudFloat _gainAllPass;
			
			AudFloat _erTapL1,_erTapL2,_erTapL3,_erTapL4,_erTapL5,_erTapL6;
			AudFloat _erTapR1,_erTapR2,_erTapR3,_erTapR4,_erTapR5,_erTapR6;
			
			AudFloat _egL1,_egL2,_egL3,_egL4,_egL5,_egL6;
			AudFloat _egR1,_egR2,_egR3,_egR4,_egR5,_egR6;
			AudFloat MILLISECONDS,_maxdelayTime;			
		};
} //	end of namespace Aud
#endif /*EUVERB_H_*/
///EOF

