/***************************************************/
/*! \class Envelops
 \brief
 
 The AudioDrome Envelops class, implements 
 a envelope generator, based on the control 
 of the attack and decay.
 The values of attack and decay,
 are expressed as a percentage of total duration.
 therefore, the "sustain" value, is implicit:
 
 _sus = _dur - (_attack + _decay);
 
 by Alessandro Petrolati 2006.
 www.alessandro-petrolati.it
 info@alessandro-petrolati.it
 
 This library is free software; you can redistribute it and/or
 modify it under the terms of the GNU Library General Public
 License as published by the Free Software Foundation; either
 version 2 of the License, or (at your option) any later version.
 
 This library is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 Library General Public License for more details.
 
 You should have received a copy of the GNU Library General Public
 License along with this library; if not, write to the Free
 Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
/***************************************************/

#ifndef ENVELOPS_H
#define ENVELOPS_H

#include "Trapezoid.h"
#include "Window.h"

//  begin namespace
namespace Aud
{
	class Envelops : public Trapezoid
		{
		public:
			//! Public constructor
			/*!
			 * We must pass a pointer to an object "Buffer", 
			 * which contains the profile envelope.
			 * If the object Buffer pointer is NULL, 
			 * the default constructor automatically it creates 
			 * a Buffer object, containing a Window-Gaussian type.
			 * For the interpolation method, see the enumerator INTERPOLATION (DISABLED,LINEAR,CUBIC or VARGA).
			 */
			Envelops(Buffer* shape=NULL, const INTERPOLATION& type=LINEAR);
			
			//! Public destructor.
			virtual ~Envelops();
			
			//! Sets the Buffer object pointer, containing the envelope waveform
			void setShape(Buffer* shape);
			
			//! Sets the interpolation type
			/*!
			 The following methods, are possible: 
			 DISABLED = not interpolated. 
			 LINEAR linear interpolation (two points).
			 CUBIC = cubic interpolation. 
			 VARGA = cubic optimized by Istvan Varga.
			 */
			void setInterpolation(const INTERPOLATION &type=LINEAR);
			
		protected:
			//! Processing the audio sample
			/*!
			 * For each call, the internal phase is increased. 
			 * The increase depends on the length expressed. 
			 * The value of "phase", is used as an index to 
			 * access the buffer prototype Buffer.			 
			 */
			virtual void Processing();
			
		private:
			bool _flag;
			Buffer* _shape;
			AudFloat (Buffer::*_interpPtr) (const AudFloat &);
		};
} //	end of namespace Aud
#endif
///EOF
