/***************************************************/
/*! \class Flanger
 \brief
 
 The AudioDrome Flanger class, implements a flanger effect
 based on the delay line interpolated.
 This unit is useful for generating choruses and flangers. 
 
 Lehman, Scott (1996). Effects Explained. Harmony Central. Retrieved 6/04 from
 http://www.harmony-central.com/Effects/effects-explained.html
 
 Mikelson, Hans (2000). Modeling a multieffects processor in Csound. 
 In Boulanger, Richard (2000), The Csound book (pp 575-594). Cambridge, MA: MIT Press.
 Schindler, Allan. (1998). Eastman Csound tutorial.  Eastman School of Music. Retrieved 6/04 from
 http://www.esm.rochester.edu/onlinedocs/allan.cs/
 Vercoe, Barry. (1992). The public Csound reference manual, version 4.16. MIT Press.  
 Retrieved 6/04 from http://www.lakewoodsound.com/csound/hypertext/manual.htm
 
 by Alessandro Petrolati 2006.
 www.alessandro-petrolati.it
 info@alessandro-petrolati.it
 
 This library is free software; you can redistribute it and/or
 modify it under the terms of the GNU Library General Public
 License as published by the Free Software Foundation; either
 version 2 of the License, or (at your option) any later version.
 
 This library is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 Library General Public License for more details.
 
 You should have received a copy of the GNU Library General Public
 License along with this library; if not, write to the Free
 Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
/***************************************************/

#ifndef FLANGER_H
#define FLANGER_H

#include "DelayTaps.h"
#include "Oscillator.h"
#include "Wave.h"

//  begin namespace
namespace Aud
{
	class Flanger : public DelayTaps
		{
		public:
			//! Public constructor (delay in seconds)
			Flanger(const AudFloat &delayMax=1);
			
			//! Public destructor
			virtual ~Flanger();
			
			//! Sets the delay time in seconsds
			void setDelayTime(const AudFloat &delay=0.008);
			
			//! Sets the modulation width, in seconds
			void setDepthTime(const AudFloat &depth=0.003);
			
			//! Seth the gain value for te feedback
			void setFeedGain(const AudFloat &feedGain=0.5);
			
			//! Sets the modulation frequency, in Hz
			void setLfoRatio(const AudFloat &ratio=2.5);
			
			//! Sets a waveform for the LFO (Low Frequency Oscillator)
			void setModShape(const WAVESHAPE &wave=SINE);
			
			//!	Sets mixing amount (range 0-1)
			/*!
			 * Mix between the original signal and elaborated.
			 * With a value of zero, the original signal.
			 * With a value of zero signal produced.
			 * With a value of 0.5, mixing to 50%
			 */
			void setMixing(const AudFloat& mixing=0.5);
			
		protected:
			//! Processing the audio sample
			virtual void Processing();
			
		private:
			Wave* _wave;
			Oscillator* _lfo;
			AudFloat _flanging;
			AudFloat _direct;
			AudFloat _feedGain;
			AudFloat _delay;
			AudFloat _last;
		};
} //	end of namespace Aud
#endif
///EOF

