/***************************************************/
/*! \class GenerateDTMF
 \brief
 
 The AudioDrome GenerateDTMF class,
 implements a Dual Tone Multi Frequency generator.
 http://en.wikipedia.org/wiki/DTMF
 
 by Alessandro Petrolati 2008.
 www.alessandro-petrolati.com
 info@alessandro-petrolati.it
 
 This library is free software; you can redistribute it and/or
 modify it under the terms of the GNU Library General Public
 License as published by the Free Software Foundation; either
 version 2 of the License, or (at your option) any later version.
 
 This library is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 Library General Public License for more details.
 
 You should have received a copy of the GNU Library General Public
 License along with this library; if not, write to the Free
 Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
/***************************************************/

#ifndef GENERATEDTMF_H
#define GENERATEDTMF_H

#include "Sinusoide.h"
#include "Random.h"
#include "BreakLine.h"
#include <string>

//  begin namespace
namespace Aud
{
	class GenerateDTMF : public Sinusoide
		{
		public:
			//! Public constructor
			GenerateDTMF(const AudFloat ampDb=-6);
			
			//! Public destructor
			virtual ~GenerateDTMF();
			
			//! Sets the tone level in decibell
			void setToneLevel(const AudFloat& ampDb=-6);
			
			//! Sets the noise injection level in decibell
			void setNoiseLevel(const AudFloat& ampDb=-60);
			
			//! Sets the duration in milliseconds for each tone
			void setToneTime(const AudFloat& ms=100);
			
			//! Sets the silence time, in milliseconds, between a tone and the next
			void setToneInterrupt(const AudFloat& ms=40);
			
			//! Sets the pause time, in milliseconds, between tone and the next
			void setPauseTime(const AudFloat& ms=500);
			
			//! Specify a character to indicate the pause between tones
			void setPauseChar(const char& chr=',');
			
			/*! Sets a string containing the valid characters tones
			 /*!
			 * Are valid only the following characters:
			 * 12234567890ABCD*#;
			 */
			void setStringTone(const std::string& s);
			
			//! Sets the loop mode
			/*
			 * If "true", repeat the string, 
			 * otherwise if "false", run the string once.
			 */
			inline void setLoopMode(const bool& mode) 
			{ 
				_mode = mode;
			}
			
			//! Clear the string buffer
			void clearStringTone() { _stringa.clear(); }
			
			//! Restart the tone
			/*!
			 * If it is active, stop the execution and repeat
			 */
			inline void triggerTone() 
			{ 
				_retrig = true; 
				_env.enableWrapAround(true);
				_env.trigger();
				_breakCount = 0;
			}
			
		protected:
			//! Processing the audio sample
			virtual void Processing();
			
		private:
			void setFrequency(const AudFloat&);
			void setAmplitude(const AudFloat&);
			void computeToneFreqs(const char& n);
			AudFloat* _freqs;
			Sinusoide _oscColonna;
			Random _noise;
			BreakLine _env;
			AudFloat _toneTime;
			AudFloat _interrupt;
			AudFloat _pauseTime;
			char _pauseChar;
			std::string _stringa;
			ul _breakCount;
			bool _retrig, _mode;
			//! Imposta i valori per l'inviluppo
			void setEnvelope();	
		};
} //	end of namespace Aud
#endif /*GENERATEDTMF_H_*/
///EOF

