/***************************************************/
/*! \class Granulator
 \brief
 
 The AudioDrome Granulator class, implements 
 the  AGS (Asynchronous Granular Synthesis). 
 This DSP (Digital Signal Processing) manages
 all the classics parameters of the AGS
 
 by Alessandro Petrolati 2006.
 www.alessandro-petrolati.it
 info@alessandro-petrolati.it
 
 This library is free software; you can redistribute it and/or
 modify it under the terms of the GNU Library General Public
 License as published by the Free Software Foundation; either
 version 2 of the License, or (at your option) any later version.
 
 This library is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 Library General Public License for more details.
 
 You should have received a copy of the GNU Library General Public
 License along with this library; if not, write to the Free
 Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
/***************************************************/

#ifndef GRANULATOR_H
#define GRANULATOR_H

#include "Metronome.h"
#include "OscilTimout.h"
#include "Envelops.h"
#include "Wave.h"
#include "Window.h"
#include "Random.h"
#include "Interpolation.h"

//  begin namespace
namespace Aud
{
	class Granulator : public AudioDrome
		{
		public:
			//! Public constructor
			/*!
			 * We must declare a pointer to an object Wave (waveshape), 
			 * a pointer to the Window (winshape) envelope and
			 * the maximum grains number overlapping (voices). 
			 */
			Granulator(Wave* wave=NULL, Window* window=NULL, const unsigned int &maxOverlap=50);
			
			//! Public Destructor
			virtual ~Granulator();
			
			//			//! When the AbsoluteSR changes, the object ignore this
			//			virtual inline void ignoreAbsoluteSRChange()
			//			{
			//				AudioDrome::ignoreAbsoluteSRChange();
			//				_trig->ignoreAbsoluteSRChange();
			//				
			//				for (unsigned int i = 0; i < _overlap; i++)
			//				{
			//					_osc[i]->ignoreAbsoluteSRChange();
			//					_env[i]->ignoreAbsoluteSRChange(); 
			//				}				
			//			}
			//			
			//			//! When the AbsoluteSR changes, the internal class data appropriately update their own 
			//			virtual inline void listenAbsoluteSRChange()
			//			{
			//				AudioDrome::listenAbsoluteSRChange();				
			//				_trig->listenAbsoluteSRChange();
			//				
			//				for (unsigned int i = 0; i < _overlap; i++)
			//				{
			//					_osc[i]->listenAbsoluteSRChange();
			//					_env[i]->listenAbsoluteSRChange();
			//				}				
			//			}
			
			//! Sets the deaults params
			virtual void setDefaults();
			
			//! Sets the overlapping factory
			void setOverlap(const unsigned int &maxOverlap);
			
			//! Sets the wave-shape prototype for the granulator
			void setWave(Wave* wave);
			
			//! Sets the win-shape envelope for the granulator
			void setWindow(Window* window);
			
			//! Sets the amplitude (0.0 - 1.0)
			void setLevel(const AudFloat &volume);
			
			//! Sets the stereo width (0.0 - 1.0)
			void setWidth(const AudFloat &width);
			
			//! Sets the attack of the grain envelope (0.0 - 0.5)
			void setAttack(const AudFloat &attack);
			
			//! Sets the decay of the grain envelope (0.0 - 0.5)
			void setDecay(const AudFloat &decay);
			
			//! Sets the grains length in milliseconds (0.0 - inf.)
			void setLength(const AudFloat &length);
			
			//! Sets density grain per seconds (gps) (> 0.0 - inf.)
			void setDensity(const AudFloat &density);
			
			//! Sets the grains frequency in Hz
			void setFrequency(const AudFloat &frequency); /*! Sampling Rate Change SLOT */
			
			//! Sets the seed grains phase (0.0 - 1.0)
			void setPhase(const AudFloat &phase);
			
			//! Sets the randomic length, in % of setLength() (0.0 - 1.0)
			void setRndLength(const AudFloat &rndLength);
			
			//! Sets the randomic density, in % of setDensity() (0.0 - 1.0)
			void setRndDensity(const AudFloat &rndDensity);
			
			//! Sets the randomic frequency, in % of octave (0.0 - 1.0)
			void setRndFrequency(const AudFloat &rndFrequency);
			
			//! Sets the randomic phase seed (0.0 - 1.0)
			void setRndPhase(const AudFloat &rndPhase);
			
			//! Gets the audio sample (rigth channel)
			/*! 
			 * This function returns a value secondary 
			 * In the main loop needs to be called Processing() always. 
			 * This process implements a pseudo-stereo,
			 * panning width, see setWidth()
			 */			
			virtual inline AudFloat Output_2()
			{
				return _output_2;
			}
			
			virtual void setDownSampling(const ul& samples=1) /*! Sampling Rate Change SIGNAL */
			{
				AudioDrome::setDownSampling(samples);
				_trig->setDownSampling(samples);
				
				for (unsigned int i = 0; i < _overlap; i++)
				{
					_osc[i]->setDownSampling(samples);
					_env[i]->setDownSampling(samples);
				}
			}
			
			//! PROCESSING is the primary function
			/*!
			 * You must call the function in the main loop, 
			 * or in the callback thread.
			 * This procedure not return any value, but write 
			 * the "_output_1" variable. 
			 * This signal can be read with the Output().
			 * Similarly, the Input() sets a value input for the DSP.
			 */
			virtual void PROCESSING()
			{
				this->Processing();	
			}
			
		protected:
			//! Processing the audio sample
			/*!
			 * Output() function return the only left channel, 
			 * if you want to create a pseudo-stereo,
			 * you must call, in the main loop, the Output_2().
			 * See setWidth().
			 */
			virtual void Processing();
			
			unsigned int _overlap;
			unsigned int* _oscAct;
			Metronome* _trig;
			Buffer* _Wave;
			Buffer* _Win;
			OscilTimout** _osc;
			Envelops** _env;
			AudFloat* _Left;
			AudFloat* _Rigth;
			AudFloat _Volume;
			AudFloat _Width;
			AudFloat _Attack;
			AudFloat _Decay;
			AudFloat _Length;
			AudFloat _Density;
			AudFloat _Frequency;
			AudFloat _Phase;
			AudFloat _RndLength;
			AudFloat _RndDensity;
			AudFloat _RndFrequency;
			AudFloat _RndPhase;
			bool _defaultWave;
			bool _defaultWin;
		};
} //	end of namespace Aud
#endif
///EOF
