/***************************************************/
/*! \class GranulatorSF
 \brief
 
 The AudioDrome GranulatorSF class, implements 
 the  AGS (Asynchronous Granular Synthesis). 
 This DSP (Digital Signal Processing) manages
 all the classics parameters of the AGS.
 This DSP, working on audio-files, are 
 also implemented control functions: pointers 
 to sound-file and jittering.
 
 by Alessandro Petrolati 2006.
 www.alessandro-petrolati.it
 info@alessandro-petrolati.it
 
 This library is free software; you can redistribute it and/or
 modify it under the terms of the GNU Library General Public
 License as published by the Free Software Foundation; either
 version 2 of the License, or (at your option) any later version.
 
 This library is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 Library General Public License for more details.
 
 You should have received a copy of the GNU Library General Public
 License along with this library; if not, write to the Free
 Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
/***************************************************/

#ifndef GRANULATORSF_H
#define GRANULATORSF_H

#include "Granulator.h"
#include "BreakLine.h"
#include "Window.h"
#include "Sample.h"

//  begin namespace
namespace Aud
{
	class GranulatorSF : public Granulator
		{
		public:
			//! Public constructor
			/*!
			 * We must declare a pointer to an object File (file data samples),
			 * a pointer to the Window (winshape) envelope and
			 * the maximum grains number overlapping (voices). 
			 */
			GranulatorSF(Sample* sndFile=NULL, Window* window=NULL, const unsigned int &maxOverlap=50);
			
			//! Public destructor
			virtual ~GranulatorSF();
			
			//! Select the audio files to be processed
			void setSample(Sample* sndFile);
			
			//! Sets the defaults values
			void setDefaults();
			
			//! Sets the start value, for the file pointer, in seconds
			void setStart(const AudFloat &start);
			
			//! Sets the end value, for the file pointer, in seconds
			void setEnd(const AudFloat &end);
			
			//! Sets the pitch value in semitone
			/*!
			 * pitch = 0, no transpose
			 * = 12 eighth up
			 * = -6 semitone down
			 */
			void setPitch(const AudFloat &pitch);
			
			//! Sets the scanning speed
			void setSpeed(const AudFloat &speed);
			
			//! Sets the time pointer to file in milliseconds
			void setTimePointer(const AudFloat &point=0)
			{
				if(!_defaultWave)
					_scanning.setPhase((point*0.001) / static_cast<Sample*>(_Wave)->getDurationTime());
			}
			
			//! Sets the pitch random jittering 
			void setRndPitch(const AudFloat &rndPitch);
			
			//! Sets the start random jittering, in milliseconds (file time pointer)
			void setJittering(const AudFloat &jittering);
			
		protected:
			//! Processing the audio sample
			/*!
			 * Output() function return the only left channel, 
			 * if you want to create a pseudo-stereo,
			 * you must call, in the main loop, the getSignalRigth().
			 * See setWidth()			 
			 */			
			virtual void Processing();
			
		private:
			void setPhase(const AudFloat &);
			void setRndPhase(const AudFloat &);
			void setWave(Wave *);
			void setFrequency(const AudFloat &);
			void setRndFrequency(const AudFloat &);
			Phasor _scanning;
			AudFloat _rangeScan;
			AudFloat _start;
			AudFloat _end;
			AudFloat _speed;
		};
} //	end of namespace Aud
#endif
///EOF
