/***************************************************/
/*! \class Karplus-Strong
 \brief
 
 The AudioDrome Karplus-Strong class, 
 inherits from Comb, the Processing() 
 function is extended, making a low-pass
 filter in the feedback.
 
 * ===================================================
 * 				        ________________ outTap()
 * 		               |
 *  input -- + --[..... d.....] --------- Processing()
 * 	        |_____ lp ____ g ____|
 * ===================================================
 
 Alexander Strong invented the algorithm, and Kevin Karplus 
 did the first analysis of how it worked. Together they developed 
 software and hardware implementations of the algorithm, including 
 a custom VLSI chip. They named the algorithm "Digitar" synthesis, 
 as an abbreviation for "digital guitar".
 
 by Alessandro Petrolati 2006.
 www.alessandro-petrolati.it
 info@alessandro-petrolati.it
 
 Refinements to the algorithm:
 Alex Strong and Kevin Karplus realized that the Karplus-Strong algorithm 
 was physically analogous to a sampling of the transversal wave on a 
 string instrument, with the filter in the feedback loop representing 
 the total string losses over one period. Julius O. Smith III [1] and 
 others generalized the algorithm to digital waveguide synthesis, which 
 could also be used to model acoustic waves in tubes and on drum membranes.
 Alex Strong developed a superior wavetable-modification method for 
 plucked-string synthesis, but only published it as a patent.
 
 This library is free software; you can redistribute it and/or
 modify it under the terms of the GNU Library General Public
 License as published by the Free Software Foundation; either
 version 2 of the License, or (at your option) any later version.
 
 This library is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 Library General Public License for more details.
 
 You should have received a copy of the GNU Library General Public
 License along with this library; if not, write to the Free
 Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
/***************************************************/

#ifndef KARPLUSSTRONG_H_
#define KARPLUSSTRONG_H_

#include "Comb.h"
#include "LowPass.h"

namespace Aud
{
	class KarplusStrong : public Comb
		{
		public:
			//! Public constructor
			/*
			 * N.B.
			 * If interpolation is enabled, cause 
			 * a low-pass frequency response.
			 */
			KarplusStrong(const AudFloat &delayMax=1, const INTERPOLATION& type=DISABLED);
			
			//! Public destructor
			virtual ~KarplusStrong();
			
			//! Sets the cut-off of filter
			void setCutOff(const AudFloat &cutOff);
			
		protected:
			//! Processing the audio sample
			/*
			 * In the recirculation is inserted a one-pole low-pass filter.
			 * The member funcion setCutOff(), controlls the cut-off frequency.
			 * ===================================================
			 * 				           ________________ OutputTap()
			 * 		                  |
			 *  Input() -- + --[..... d.....] --------- Output()
			 * 	           |_____ lp ____ g ____|
			 * ===================================================
			 */
			virtual void Processing();
			
		private:
			LowPass filter;
		};
} //	end of namespace Aud
#endif /*KARPLUSSTRONG_H_*/
///EOF
