/***************************************************/
/*! \class Metronome
 \brief
 
 The AudioDrome Metronome class, implements 
 a trigger generator controlled in frequency.
 The resulting is "true", a sample for each 
 Period (1/freq), and "false" for all 
 remaining samples of the period.
 
 by Alessandro Petrolati 2006.
 www.alessandro-petrolati.it
 info@alessandro-petrolati.it
 
 This library is free software; you can redistribute it and/or
 modify it under the terms of the GNU Library General Public
 License as published by the Free Software Foundation; either
 version 2 of the License, or (at your option) any later version.
 
 This library is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 Library General Public License for more details.
 
 You should have received a copy of the GNU Library General Public
 License along with this library; if not, write to the Free
 Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
/***************************************************/

#ifndef METRONOME_H
#define METRONOME_H

#include "AudioDrome.h"

//  begin namespace
namespace Aud
{
	class Metronome : public AudioDrome
		{
		public:
			//! Public constructor
			/*!
			 * If "true", the first sample is "true"
			 * else is "false".
			 */
			Metronome(const bool &firstOut=true);
			
			//! Public destructor
			virtual ~Metronome();
			
			//! Sets the generation frequency of the trigger signal
			/*!
			 * The value is in Hz, a value of 10, produces a sequence of ten samples 
			 * "true" each second, with a time interval to 0.1 seconds (1/10).
			 */
			void setFrequency(const AudFloat &freq); /*! Sampling Rate Change SLOT */
			
			//! Sets the number of samples for the trigger signal generation
			/*!
			 * The value is in samples, a value of 4410, produces a sequence of ten samples 
			 * "true" each second, with a time interval to 0.1 seconds (4410/SR).
			 * Obviously if the Sampling Rate is 44100 Hz.
			 */
			void setSample(const unsigned long &sample);
			
			//! Sets the BPM for the trigger signal generation
			/*!
			 * Similar to setSample() and setFrequency(), but accepts a value expressed as Beat per Minuts.
			 */
			void setBpm(const AudFloat &bpm);
			
			//! PROCESSING is the primary function
			/*!
			 * You must call the function in the main loop, 
			 * or in the callback thread.
			 * This procedure not return any value, but write 
			 * the "_output_1" variable. 
			 * This signal can be read with the Output().
			 * Similarly, the Input() sets a value input for the DSP.
			 */
			virtual void PROCESSING()
			{
				if(++_smplCount < _smplBlock)
				{
					_output_1 = 0.0;
					return;
				}
				
				_smplCount = 0.0;
				this->Processing();
			}
			
		protected:
			//! Processing audio sample
			/*!
			 * The resulting is a "Boolean". True
			 * for a sample for each 
			 * Period (1/freq), and False for all 
			 * remaining samples of the period.
			 */
			virtual void Processing();
			
			virtual void srUpdate()
			{ 
				this->setFrequency(_freq);
			}
			
		private:
			AudFloat _incrm;
			AudFloat _accum;
			AudFloat _freq;
		};
} //	end of namespace Aud
#endif
///EOF
