/***************************************************/
/*! \class MoogLadder LowPass Filter
 \brief
 
 The AudioDrome MoogLadder class, implements
 a Moog ladder lowpass filter.
 Moogladder is an new digital implementation of the 
 Moog ladder filter based on the work of Antti Huovilainen, 
 described in the paper "Non-Linear Digital Implementation
 of the Moog Ladder Filter" (Proceedings of DaFX04, Univ of Napoli). 
 This implementation is probably a more accurate digital 
 representation of the original analogue filter.
 
 Csound's code newfils.c
 Author: Victor Lazzarini;
 January 2005
 
 by Alessandro Petrolati 2008.
 www.alessandro-petrolati.it
 info@alessandro-petrolati.it
 
 This library is free software; you can redistribute it and/or
 modify it under the terms of the GNU Library General Public
 License as published by the Free Software Foundation; either
 version 2 of the License, or (at your option) any later version.
 
 This library is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 Library General Public License for more details.
 
 You should have received a copy of the GNU Library General Public
 License along with this library; if not, write to the Free
 Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
/***************************************************/

#ifndef MOOGLADDER_H_
#define MOOGLADDER_H_

#include "AudioDrome.h"

//  begin namespace
namespace Aud
{
	class MoogLadder : public AudioDrome
		{
		public:
			
			//! Public constructor
			MoogLadder(const AudFloat& freq=1000, const AudFloat& res=707, const AudFloat& V=1.22070313);
			
			//! Public destructor
			virtual ~MoogLadder();
			
			//! Sets the filter resonance
			/*!
			 * Resonance, generally < 1, 
			 * but not limited to it. 
			 * Higher than 1 resonance values might 
			 * cause aliasing, analogue synths 
			 * generally allow resonances to be above 1.
			 */
			void setResonance(const AudFloat& res=0.707);
			
			//! Sets the cut-off frequency in hz
			void setCutOff(const AudFloat& hz=1000); /*! Sampling Rate Change SLOT */
			
			//! Sets the "transistor thermal voltage" digital value
			/*! 
			 * When the signal is a 16-bit integer, as in the case of Csound code, 
			 * The default value is 40000 (1 / 40000), If the signal is normalized, 
			 * Like the AudioDrome classes, the appropriate value is 1.2 
			 * 32768: 40000 = 1: x 
			 * 40000/32768 = 1.22070313
			 */
			void setTransistorVoltage(const AudFloat& V=1.22070313);
			
		protected:
			//! Processing the audio sample
			virtual void Processing();
			
			virtual void srUpdate()
			{ 
				this->setCutOff(_freq);
			}
			
		private:
			AudFloat _freq;
			AudFloat _res;
			AudFloat* _delay;
			AudFloat* _tanhstg;
			double* _stg;
			double  _res4, _thermal, _input, _tune, _acr;
		};
} //	end of namespace Aud
#endif
///EOF
