/***************************************************/
/*! \class OscilTimout
 \brief
 
 The AudioDrome OscilTimout class, inherits from Oscillator,
 implements an timer oscillator.
 When the execution time expires, 
 The oscillator goes off and the counter of instances is decremented
 
 by Alessandro Petrolati 2006.
 www.alessandro-petrolati.it
 info@alessandro-petrolati.it
 
 This library is free software; you can redistribute it and/or
 modify it under the terms of the GNU Library General Public
 License as published by the Free Software Foundation; either
 version 2 of the License, or (at your option) any later version.
 
 This library is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 Library General Public License for more details.
 
 You should have received a copy of the GNU Library General Public
 License along with this library; if not, write to the Free
 Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
/***************************************************/

#ifndef OSCILTIMOUT_H
#define OSCILTIMOUT_H

#include "Oscillator.h"

//  begin namespace
namespace Aud
{
	class OscilTimout : public Oscillator
		{
		public:
			//! Public constructor
			/*!
			 * If the object Buffer pointer is NULL, 
			 * the default constructor automatically it creates 
			 * a Buffer object, containing a SINE prototype.
			 * For the interpolation method, see the enumerator INTERPOLATION (DISABLED,LINEAR,CUBIC or VARGA).
			 */
			OscilTimout(Buffer* wave=NULL, const INTERPOLATION& type=LINEAR, bool initialStatus=true);
			
			//! Public destructor.
			virtual ~OscilTimout();
			
			//! Gets the oscillator state
			/*!
			 * If the oscillator is running, returns "true",
			 * otherwise returns "false"
			 * If the state is "false", means that the oscillator it could be rerun.
			 */
			bool getStatus() const;
			
			//! Gets the total number of the oscillators in use
			/*!
			 * This value depending by object number istanziated and actives.
			 * Il valore dipende dal numero di oggetti istanziati e funzionante.
			 * A static variable (_actives++) is  incresed and decresed
			 * For each activation, a static variable (_actives++), is incremented. 
			 * When the time is elapsed, the static variable is decremented.			
			 */
			static int getNumOscAtive();
			
			//! Starts the oscillator with a duration time in seconds
			/*!
			 * If getStatus() is "true" (running), the execution does not happen.
			 */
			void start();
			
			inline void setTime(const AudFloat &dur = 1.0) throw() /*! Sampling Rate Change SLOT */
			{
				if(dur < 0)
				{
					AudError er(NEGATIVE_VALUE);
					throw er;
				}
				else
				{
					_dur = dur;
					_durSamples = _dur * _localSR;
				}
			}
			
			//! Finish the current process and reinitializes the oscillator
			/*
			 * The oscillator goes off, the counter of instances is decremented 
			 * and a new istances is triggered.
			 */
			void trigger();
			
		protected:
			//! Processing the audio sample
			/*!
			 * Calls the function Oscillator::Processing(), and records the elapsed time. 
			 * When the time is over, decreases the number of oscillators actives
			 * (_istanze -) and sets the state (getStatus) "false". Namely: free.			 
			 */
			virtual void Processing();
			
			virtual void srUpdate()
			{
				Oscillator::srUpdate();
				this->setTime(_dur);
			}
			
		private:
			AudFloat _dur;
			ul _durSamples;
			ul _countSamples;
			static ul _actives;
			bool _status;			
		};
} //	end of namespace Aud
#endif
///EOF
