/***************************************************/
/*! \class Oscillator
 \brief
 
 The AudioDrome Oscillator class, implements a 
 oscillator with interpolation.
 
 by Alessandro Petrolati 2006.
 www.alessandro-petrolati.it
 info@alessandro-petrolati.it
 
 This library is free software; you can redistribute it and/or
 modify it under the terms of the GNU Library General Public
 License as published by the Free Software Foundation; either
 version 2 of the License, or (at your option) any later version.
 
 This library is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 Library General Public License for more details.
 
 You should have received a copy of the GNU Library General Public
 License along with this library; if not, write to the Free
 Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
/***************************************************/

#ifndef OSCILLATOR_H
#define OSCILLATOR_H

#include "Phasor.h"
#include "Wave.h"

//  begin namespace
namespace Aud
{
	class Oscillator : public Phasor
		{
		public:
			//! Public constructor
			/*!
			 * If the object Buffer pointer is NULL, 
			 * the default constructor automatically it creates 
			 * a Buffer object, containing a SINE prototype.
			 * For the interpolation method, see the enumerator INTERPOLATION (DISABLED,LINEAR,CUBIC or VARGA).
			 */
			Oscillator(Buffer* wave=NULL,
					   const INTERPOLATION& type=LINEAR);
			
			//! Public destructor
			virtual ~Oscillator();
			
			//! Sets the Buffer object pointer, containing the waveform
			void setWave(Buffer* wave);
			
			//! Sets the all parameters of the oscillator
			/*!
			 * The phase is expressed as a fraction of a cycle (0 to 1).
			 * Equivalent to call the following functions:
			 * setAmplitude();
			 * Phasor::setFrequency();
			 * Phasor::setPhase();
			 */
			void setParameters(const AudFloat &amp=0.5, const AudFloat &freq=440,
							   const AudFloat &phi=0);
			
			//! Sets the amplitude of signal
			void setAmplitude(const AudFloat &amp=0.5);
			
			//! Sets the interpolation type
			/*!
			 * The following methods, are possible: 
			 * DISABLED = not interpolated. 
			 * LINEAR linear interpolation (two points).
			 * CUBIC = cubic interpolation. 
			 * VARGA = cubic optimized by Istvan Varga.
			 */
			void setInterpolation(const INTERPOLATION &type=LINEAR);
			
		protected:
			//! Processing the audio sample
			/*!
			 * Each call, increases the internal phase:
			 * _incr = (AudFloat) freq / getAbsoluteSR();
			 */
			virtual void Processing();
			
		private:
			Buffer* _Wave;
			bool _flag;
			AudFloat _amp;
			AudFloat (Buffer::*_interpPtr) (const AudFloat &);
		};
} //	end of namespace Aud
#endif
///EOF
