/***************************************************/
/*! \class ParametricEQ Filters
 \brief
 
 The AudioDrome ParametricEQ class, implements
 a parametric equalizer.
 This implementation of Zoelzer's parametric equalizer filters,
 with some modifications by the author.
 
 Csound's code: biquad.c
 
 by Alessandro Petrolati 2006.
 www.alessandro-petrolati.it
 info@alessandro-petrolati.it
 
 This library is free software; you can redistribute it and/or
 modify it under the terms of the GNU Library General Public
 License as published by the Free Software Foundation; either
 version 2 of the License, or (at your option) any later version.
 
 This library is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 Library General Public License for more details.
 
 You should have received a copy of the GNU Library General Public
 License along with this library; if not, write to the Free
 Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
/***************************************************/

#ifndef PARAMETRICEQ_H_
#define PARAMETRICEQ_H_

#include "AudioDrome.h"

namespace Aud
{	
	class ParametricEQ : public AudioDrome
		{
		public:
			//! Public constructor
			ParametricEQ(const AudFloat &fc=5000,
						 const AudFloat &amount=AudioDrome::rawDb(12), const AudFloat &q=0.2, const MODE &mode=PEAKING);
			
			//! Public destructor
			virtual ~ParametricEQ();
			
			//! Sets the center frequency or corner frequency (in Hz)
			/*!
			 * It assumes the meant "Center Frequency" when the filter is
			 * in "peaking mode", and "Corner Frequency" in "Shelving mode".
			 */
			inline void setFc(const AudFloat &fc)
			{
				_fc = fc;
				this->mode(_mode);
			}
			
			//! Sets the amount of boost or cut
			/*!
			 * A value less than 1 is a cut. 
			 * A value greater than 1 is a boost. A value of 1 is a flat response.
			 */
			inline void setAmount(const AudFloat &amount)
			{
				_v = amount;
				this->mode(_mode);
			}
			
			//! Sets the Q of the filter 
			/*!
			 * sqrt(0.5) is no resonance
			 */
			inline void setQuality(const AudFloat &q)
			{
				_q = q;
				this->mode(_mode);
			}
			
			//! Sets the filter operating mode
			/*!
			 * PEAKING
			 * LOWSHELVING
			 * HIGHSHELVING
			 */
			inline void setMode(const MODE &mode)
			{
				_mode = mode;
				this->mode(_mode);
			}
			
		protected:
			//! Processing the audio sample
			virtual void Processing();
			
			virtual void srUpdate()
			{ 
				this->calcCoefs();
				this->mode(_mode);
			}
			
		private:
			MODE _mode;
			AudFloat _tpidsr;
			AudFloat _fc;
			AudFloat _v;
			AudFloat _q;
			AudFloat _xnm1;
			AudFloat _xnm2;
			AudFloat _ynm1;
			AudFloat _ynm2;
			AudFloat _a1;
			AudFloat _a2;
			AudFloat _b0;
			AudFloat _b1;
			AudFloat _b2;
			void mode(const MODE &mode); /*! Sampling Rate Change SLOT */
			void calcCoefs() /*! Sampling Rate Change SLOT */
			{
				_tpidsr = 2.0 * M_PI / _localSR;
			}
		};
} //	end of namespace Aud
#endif /*PARAMETRICEQ_H_*/
///EOF

