/***************************************************/
/*! \class Phasor
 \brief
 
 The AudioDrome Phasor class, implements
 a normalized moving phase value.
 An internal phase is successively accumulated 
 in accordance with the setFrequency() to produce
 a moving phase value, normalized in the range 0 <= phs < 1.
 When used as the index to a Buffer object, this phase 
 (multiplied by the desired function table length) 
 will cause it to behave like an oscillator.
 
 Note that phasor is a special kind of integrator, accumulating phase increments that represent frequency settings.
 
 by Alessandro Petrolati 2006.
 www.alessandro-petrolati.it
 info@alessandro-petrolati.it
 
 This library is free software; you can redistribute it and/or
 modify it under the terms of the GNU Library General Public
 License as published by the Free Software Foundation; either
 version 2 of the License, or (at your option) any later version.
 
 This library is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 Library General Public License for more details.
 
 You should have received a copy of the GNU Library General Public
 License along with this library; if not, write to the Free
 Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
/***************************************************/

#ifndef PHASOR_H
#define PHASOR_H

#include "AudioDrome.h"

//  begin namespace
namespace Aud
{
	class Phasor : public AudioDrome
		{
		public:
			//! Public constructor
			Phasor();
			
			//! Public destructor
			virtual ~Phasor();
			
			//! Sets the frequency in Hz or Cps
			/*!
			 * An internal phase is successively accumulated 
			 * in accordance with the setFrequency() to produce
			 * a moving phase value, normalized in the range 0 <= phs < 1.
			 */
			void setFrequency(const AudFloat &freq=1); /*! Sampling Rate Change SLOT */
			
			//! Sets the phase, expressed as a fraction of a cycle (0 to 1)
			/*!
			 * A value of 0.25 equals 90 ° and 180 ° to 0.5 etc ...
			 * The default value is zero.
			 */
			void setPhase(const AudFloat &phi=0);
			
			//! Resets the internal phase
			/*!
			 * Equivalent to the call setPhase(0.0).
			 */
			void trigger() { _phi = 0.0; };
			
			//! Returns a trigger signal for each ramp executed
			/*!
			 * When Processing() function generates the last sample of each ramp, 
			 * Shoot the "trigger". The function returns a sample "true",while "false" for all other.
			 */
			bool getTrigger() const;
			
		protected:
			//! Processing tha audio sample
			/*!
			 * An internal phase is successively accumulated:
			 * in accordance with the setFrequency() to produce
			 * a moving phase value, normalized in the range 0 <= phs < 1:
			 * _incr = (AudFloat) setFrequency() / getAbsoluteSR() 			 
			 */
			virtual void Processing();
			
			virtual void srUpdate()
			{ 
				this->setFrequency(_freq);
			}			
			
			bool _trigger;
			AudFloat _phi;
			AudFloat _incr;
			AudFloat _freq;
		};
} //	end of namespace Aud
#endif
///EOF
