/***************************************************/
/*! \class Random
 \brief
 
 The AudioDrome Random class, implements a linear
 random number generator.
 
 by Alessandro Petrolati 2006.
 www.alessandro-petrolati.it
 info@alessandro-petrolati.it
 
 This library is free software; you can redistribute it and/or
 modify it under the terms of the GNU Library General Public
 License as published by the Free Software Foundation; either
 version 2 of the License, or (at your option) any later version.
 
 This library is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 Library General Public License for more details.
 
 You should have received a copy of the GNU Library General Public
 License along with this library; if not, write to the Free
 Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
/***************************************************/

#ifndef RANDOM_H
#define RANDOM_H

#include "AudioDrome.h"

//  begin namespace
namespace Aud
{
	class Random : public AudioDrome
		{
		public:
			//! Public constructor
			/*!
			 * If the "seed" is "true" (default), initializes the 
			 * object with the current time, each Random-object, 
			 * can be produce an arbitrary sets of values.
			 * Otherwise the Random-generator, always produces 
			 * the same set of values for each object created.			 
			 */
			Random(const AudFloat &ampplitude=1, const POLARITY &polar=BIPOLAR, const bool &seed=true);
			
			//! Public destructor
			virtual ~Random();
			
			//! Sets the range of extraction of numbers
			/*!
			 * If the method of extraction is unipolar, 
			 * The numerical limit extraction varies from 0 to + range,
			 * else if bipolar, varies from -range to +range.
			 */
			void setAmplitude(const AudFloat &ampplitude=1);
			
			//! Sets the generator seed
			/*!
			 * If the "seed" is "true" (default), initializes the 
			 * object with the current time, each Random-object, 
			 * can be produce an arbitrary sets of values.
			 * Otherwise the Random-generator, always produces 
			 * the same set of values for each object created.			 
			 */
			void setSeed(const bool &seed=true);
			
			//! Sets the extraction method
			/*!
			 * If UNIPOLAR the "psuedo-random" number
			 * varies from 0 to +range,
			 * If BIPOLAR (default), 
			 * varies from -range to +range.
			 */
			void setPolarity(const POLARITY &polar=BIPOLAR);
			
			/*! GLOBAL STATIC FUNCTIONS */
			//! Static method, gets the audio sample between "min and max".
			/*!
			 * Generates a controlled pseudo-random number 
			 * series between min and max values.
			 * Each invocation generates a random number, the number 
			 * is evenly distributed between the minimum and maximum.
			 */
			static AudFloat rangeRand(const AudFloat &min=0, const AudFloat &max=1);
			
			//! Unipolar generator
			/*!
			 * Static method, always UNIPOLAR mode work.
			 * The same behaviour is obtained with Processing() configured as
			 * SetPolarity (UNIPOLAR). 
			 * The value absolute "psuedo-random" is
			 * UNIPOLAR (from 0 to +_amplitude)
			 */
			static AudFloat unipolar(const AudFloat &range);
			
			//! Bipolar generator
			/*!
			 * Static method always BIPOLAR mode work.
			 * The same behaviour is obtained with Processing() configured as
			 * SetPolarity (BIPOLAR). 
			 * The value absolute "psuedo-random" is
			 * BIPOLAR (from -_amplitude to +_amplitude)
			 */
			static AudFloat bipolar(const AudFloat &amplitude);
			
		protected:
			//! Processing the audio sample
			/*!
			 * This function calculates the sample, it normalizes within the range and returns the value.
			 * For every call, this function generates a random number.
			 */
			virtual void Processing();
						
			AudFloat _amplitude;

		private:
			AudFloat(*_polarFun) (const AudFloat &);
		};
} //	end of namespace Aud
#endif
///EOF
