/***************************************************/
/*! \class RmsBalance
 \brief
 
 The AudioDrome RmsBalance, adjust 
 one audio signal according to the values of another.
 Determines the root-mean-square amplitude of an audio signal
 and adjusts the amplitude audio signal according to a root-mean-square value.
 
 Csound sources ugens5.c
 by Alessandro Petrolati 2008.
 www.alessandro-petrolati.it
 info@alessandro-petrolati.it
 
 This library is free software; you can redistribute it and/or
 modify it under the terms of the GNU Library General Public
 License as published by the Free Software Foundation; either
 version 2 of the License, or (at your option) any later version.
 
 This library is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 Library General Public License for more details.
 
 You should have received a copy of the GNU Library General Public
 License along with this library; if not, write to the Free
 Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
/***************************************************/

#ifndef RMSBALANCE_H_
#define RMSBALANCE_H_

#include "AudioDrome.h"


namespace Aud
{		
	class RmsBalance : public AudioDrome
		{
		public:
			//! Public constructor
			RmsBalance(const ALGORITHM& type=BEST);
			
			//! Public destructor
			virtual ~RmsBalance();
			
			//! Sets the dsp algorithm
			/*!
			 * See ALGORITHM enumerator.
			 */
			void setAlgorithm(const ALGORITHM& type=BEST);
			
			//! Sets the compare signal
			/*!
			 * This function must be called for each sample,
			 * the same way as Processing().
			 * The rms power can be interrogated, 
			 * set, or adjusted to match that 
			 * of setCompareSignal(), a comparator signal			 
			 */
			void InputCompareSignal(const AudFloat& input)
			{
				_compareSig = input;
			}
			
			//! Sets the compare signal
			/*!
			 * This function must be called for each sample,
			 * the input is a AudioDrome object, so the Output() value
			 * is transferred to current object.
			 * The rms power can be interrogated, 
			 * set, or adjusted to match that 
			 * of setCompareSignal(), a comparator signal			 
			 */			
			void InputCompareSignal(AudioDrome& obj)
			{
				_compareSig = obj.Output();
			}
			
			//! Sets the half-power point in Hz
			/*!
			 * half-power point in Hz,
			 * of a special internal low-pass filter
			 * The default value is 10.
			 */
			void setHalfPowerPoint(const AudFloat& cutoff=10); /*! Sampling Rate Change SLOT */
			
			//! Gets the cut-off frequency of the HalfPowerPoint
			inline AudFloat getHalfPowerPoint() { return _cutOff;}
			
			//! Gets the RMS value
			inline AudFloat getRms() {return _rms;}
			
		protected:
			//! Processing the audio sample
			/*!
			 * Estimate the rms value.
			 * The sample balanced is returned by Output() function.
			 * The rms power of InputCompareSignal() can be interrogated, 
			 * set, or adjusted to match that a comparator signal
			 * Csound source code ugens5.c
			 */
			virtual void Processing();
			
			virtual void srUpdate()
			{ 
				this->setHalfPowerPoint(_cutOff);
			}
			
			
		private:
			AudFloat _cutOff, _rms, _compareSig;
			double _costh, _coef; 
			AudFloat _del[2];
			double _q, _r, _c1, _c2, _prvq, _prvr, _prva, _ar, _asig, _csig;
			AudFloat (RmsBalance::*_balancePtr) (const AudFloat&, const AudFloat&);
			
			/*! Csound Implementation of Balance
			 /*!
			 The rms power of asig can be interrogated, set, or adjusted to match that of a comparator signal
			 Csound source code ugens5.c
			 http://www.csounds.com
			 */
			AudFloat balanceBest (const AudFloat& input, const AudFloat& compareSig);
			
			//! Stima il valore rms e ritorna il campione bilanciato
			/*!
			 RMS estimation and signal balancing
			 This is the basis of the balance or envelope 
			 follower processor.
			 The RMS is estimated by rectifying the signal (taking its absolute values) and low- 
			 pass filtering it. The scaling gain to be applied will be the ratio of the comparator 
			 RMS to the input RMS. 
			 http://music.nuim.ie
			 National University of Ireland, Maynooth, Maynooth Co. Kildare, Ireland. 
			 */
			AudFloat  balanceSimple (const AudFloat& input, const AudFloat& compareSig);
		};
} //	end of namespace Aud
#endif /*RMSBALANCE_H_*/