/***************************************************/
/*! \class Sample
 \brief
 
 This class inherits from Buffer
 The left channel of audio-file, 
 are copied in the Buffer::_table. 
 
 If the file is multichannels, the samples
 are subsequently writed.
 In this moment, only raw and wav files,
 are supported, no aiff are supported.
 
 Thanks to by Dr Fred DePiero - CalPoly State University
 http://people.msoe.edu/~taylor/examples/wav.htm
 Read Header
 |
 Determine type
 |
 Converting data
 |
 write into the buffer
 
 
 Wav: Little-Endian (least significant byte first)
 
 __________________________
 | RIFF WAVE Chunk	       |
 |   groupID  = 'RIFF'     |
 |   riffType = 'WAVE'     |
 |    __________________   |
 |   | Format Chunk     |  |
 |   |	ckID = 'fmt '   |  |
 |   |__________________|  |
 |    __________________   |
 |   | Sound Data Chunk |  |
 |   |	ckID = 'data'   |  |
 |   |__________________|  |
 |_________________________|
 
 
 by Alessandro Petrolati 2006.
 www.alessandro-petrolati.it
 info@alessandro-petrolati.it
 
 This library is free software; you can redistribute it and/or
 modify it under the terms of the GNU Library General Public
 License as published by the Free Software Foundation; either
 version 2 of the License, or (at your option) any later version.
 
 This library is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 Library General Public License for more details.
 
 You should have received a copy of the GNU Library General Public
 License along with this library; if not, write to the Free
 Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
/***************************************************/

#ifndef SAMPLE_H
#define SAMPLE_H

#include "Buffer.h"
#include <string>
#include <fstream>
#include <climits>
#include <stdint.h>

//  begin namespace
namespace Aud
{
	class Sample : public Buffer
		{
		private:
			// header of _wav file 
			typedef struct
			{
				char rID[4];            // 'RIFF' 
				long int rLen;
				char wID[4];            // 'WAVE' 
				char fId[4];            // 'fmt ' 
				long int pcm_header_len;   // varies...
				short int wFormatTag;
				short int nChannels;      // 1, 2 for stereo data is (l,r) pairs 
				long int nSamplesPerSec; 
				long int nAvgBytesPerSec; 
				short int nBlockAlign;      
				short int nBitsPerSample;
			}   WAV_HDR;
			
			// header of _wav file 
			typedef struct
			{
				char dId[4];            // 'data' or 'fact' 
				long int dLen;
			}   CHUNK_HDR;
			
			
		public:
			//! Public constructor
			/*!
			 * Memory is allocated dynamically.
			 */
			Sample();
			
			//! Public conversion constructor: Buffer <- Sample
			Sample(const Buffer& x) : Buffer(x)
			{
#if __DEBUG__
				std::cout << "====> Conversion Constructor of: Buffer <- Sample " << this << std::endl;
#endif
				//this->copy(x);
			}
			
			//! Public destructor
			virtual ~Sample();
			
			//! Sets the audio-folder path
			void setPath(const std::string &path);
			
			//! Gets the audio-folder path
			std::string getPath() const { return _path; }
			
			//! Import a Wav file
			/*!
			 * Copy the samples in the Buffer AudioDrome object.
			 * N.B. the sound file will be tried in the current Path.		 
			 */
			void importWav(const std::string &filename) throw(AudError);
			
			//! Import a Raw file
			/*!
			 * Copy the samples in the Buffer AudioDrome object.
			 * N.B. the sound file will be tried in the current Path.		 
			 */
			void importRaw(const std::string &filename,
						   const AudFloat &sr=44100, const short &bit=16,
						   const short &chnl=1) throw(AudError);
			
			//! Gets the duration in seconds, of the file loaded into memory
			AudFloat getDurationTime() const 
			{ 
				return (getNumberOfSamples() / getFileSR() ) / getChannels(); 
			};
			
			//! Gets the "sampling rate", of the file loaded into memory
			AudFloat getFileSR() const { return _wav->nSamplesPerSec; }
			
			//! Gets the natural frequency of reading samples
			/*!
			 * freqNat = (file_sr / file_size) * file_channels;
			 */
			AudFloat getNaturalFrequency() const	
			{ 
				return (getFileSR() / (AudFloat) getNumberOfSamples() ) *  getChannels(); 
			}
			
			//! Gets the total numbers of samples
			/*!
			 * If the data is 32:
			 * sample = 4 byte.
			 */
			AudFloat getNumberOfSamples() const 
			{ 
				return (AudFloat) _chk->dLen / (_wav->nBitsPerSample / 8); 
			}
			
			//! Gets the bit number for every sample
			AudFloat getBitPerSamples() const 
			{ 
				return (AudFloat) _wav->nBitsPerSample; 
			}
			
			//! Gets the channels numbers of the file loaded into memory
			unsigned short getChannels() const 
			{ 
				return _wav->nChannels; 
			}
			
			//! Gets the name of the file loaded into memory
			std::string getName() const 
			{ 
				return _fileName; 
			};
			
			//! Prints the file info
			void printInfo() const;
			
			//! Clear the AudioDrome::Buffer data
			void clear();
			
		private:
			FILE* _filePtr;
			WAV_HDR* _wav;
			CHUNK_HDR* _chk;
			char _obuff[80];
			std::string _path;
			std::string _fileName;
			
			inline void openSample()
			{		
				/* open _wav file */ 
				std::string pathName;
				
				if (_fileName[0] == '/'|| _fileName[0] == '.')
					pathName = _fileName;
				else
					pathName = _path + _fileName;
				
				_filePtr = fopen(pathName.c_str(),"rb");
				if(!_filePtr)
					throw AudError(FILE_OPEN);
			}
			
			void readRiff();
			void readChunks();
			void converts();
			
			inline void closeSample()
			{
				if(_filePtr)
					fclose(_filePtr);
				_filePtr = NULL;
			}
		};
} //	end of namespace Aud
#endif
///EOF
