/***************************************************/
/*! \class Schroeder
 \brief
 
 The AudioDrome Schroeder, implements 
 a classic reverb.
 A standard reverb unit is composed 
 of four Comb filters in parallel followed
 by two Alpass units in series. 
 Loop times are set for optimal “natural room response.”
 
 Definition for reverberation time: 
 Time until the sound has decreased 
 to 0.1% of the original amplitude.
 The original signal is piped through 
 4 parallel comb filters then through 
 2 serial all-pass filters.
 Comb delay times are relatively prime to 
 one another (0.031, 0.037, 0.041, 0.043)
 The all-pass filters have delays between 
 0.005 and 0.02 seconds. The make the echo density. 
 The first all-pass turns each comb echo into a series.
 
 by Alessandro Petrolati 2006.
 www.alessandro-petrolati.it
 info@alessandro-petrolati.it
 
 This library is free software; you can redistribute it and/or
 modify it under the terms of the GNU Library General Public
 License as published by the Free Software Foundation; either
 version 2 of the License, or (at your option) any later version.
 
 This library is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 Library General Public License for more details.
 
 You should have received a copy of the GNU Library General Public
 License along with this library; if not, write to the Free
 Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
/***************************************************/

#ifndef SCHROEDER_H_
#define SCHROEDER_H_

#include "Comb.h"
#include "AllPass.h"


namespace Aud
{
	
	class Schroeder : public AudioDrome
		{
		public:
			//! Public constructor
			Schroeder(const AudFloat& t60 = 0.707, const AudFloat &maxTimeRev=10, const INTERPOLATION& type=DISABLED);
			
			//! Public destructor			
			virtual ~Schroeder();
			
			//! Sets the maximum reverberation time possible
			void setMaxT60(const AudFloat &maxTimeRev=3.0);
			
			//! Sets the gain to control amplitude, 0 <= igain <= 1
			void setAmpGain(const AudFloat& ampGain=1.0);
			
			//!	Sets the mixing between the original signal and riverberato
			/*!
			 * Percent reverberated signal, 0 <= ireverb <= 1
			 */
			void setMixing(const AudFloat& mixing=.5);
			
			//! Sets the reverberation time in econds
			void setT60(const AudFloat& t60=0.707);
			
		protected:
			//!	 Processing the audio sample
			virtual void Processing();
			
		private:
			Comb* _cmb1;
			Comb* _cmb2;
			Comb* _cmb3;
			Comb* _cmb4;
			AllPass* _allPass1;
			AllPass* _allPass2;
			AudFloat _gain;
			AudFloat _reverb;
			AudFloat _acoustic;
			AudFloat _norm;
		};
} //	end of namespace Aud
#endif /*SCHROEDER_H_*/
