/***************************************************/
/*! \class Sinusoide
 \brief
 
 The AudioDrome Sinusoide class,
 implements a simple sinusoidal 
 generator based on the two methods:
 No recursive and Goertzel.
 
 by Alessandro Petrolati 2008.
 www.alessandro-petrolati.com
 info@alessandro-petrolati.it
 
 This library is free software; you can redistribute it and/or
 modify it under the terms of the GNU Library General Public
 License as published by the Free Software Foundation; either
 version 2 of the License, or (at your option) any later version.
 
 This library is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 Library General Public License for more details.
 
 You should have received a copy of the GNU Library General Public
 License along with this library; if not, write to the Free
 Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
/***************************************************/

#ifndef SINUSOIDE_H_
#define SINUSOIDE_H_

#include "AudioDrome.h"


namespace Aud
{	
	
	class Sinusoide : public AudioDrome
		{
		public:
			//! Public constructor
			Sinusoide(const AudFloat& amp=1.0, const AudFloat& freq=440,  const AudFloat& phi=0.0, const ALGORITHM& type=NONRECURSIVE);
			
			//! Public destructor
			virtual ~Sinusoide();
			
			//! Sets the generating algorithm
			void setAlgorithm(const ALGORITHM& type=NONRECURSIVE);
			
			//! Sets the signal amplitude, normalized
			inline void setAmplitude(const AudFloat &amp=1.0) {_amp = amp;}
			
			//! Sets the frequency in Hz
			void setFrequency(const AudFloat& freq=440); /*! Sampling Rate Change SLOT */
			
			//! Sets the phase, expressed as a fraction of a cycle (0 to 1)
			/*!
			 * A value of 0.25 equals 90 ° and 180 ° to 0.5 etc ...
			 * The default value is zero.
			 */
			inline void setPhase(const AudFloat &phi)
			{
				_phase = (fabs (phi) > 1.0 ? 1.0 : fabs(phi));
			}
			
			//! Resets the internal phase
			inline void resetPhase() {_phase = 0.0; };			
			
		protected:
			//! Processing the audio sample
			/*!
			 * The signal is generated by one of two methods:
			 * GOERTZEL or NONRECURSIVE.
			 */
			virtual void Processing();
			
			virtual void srUpdate()
			{
				this->setFrequency(_freq);
			}			
			
			AudFloat _amp, _freq, _phase, _phaseInc, _coeff;
			AudFloat _q0;
			AudFloat _q1[2],_q2[2];
			AudFloat (Sinusoide::*_oscPtr) ();
			//! Simple Sine Oscillator
			/*! Non Recursive method */
			AudFloat nonRecursive ();			
			//! Goertzel's Sine Oscillator
			/*! Recursive method */
			AudFloat Goertzel();
		};
} //	end of namespace Aud
#endif /*SINUSOIDE_H_*/
///EOF
