/***************************************************/
/*! \class Timer
 \brief
 
 The AudioDrome Timer class, keeps track of time.
 
 by Alessandro Petrolati 2006.
 www.alessandro-petrolati.it
 info@alessandro-petrolati.it
 
 This library is free software; you can redistribute it and/or
 modify it under the terms of the GNU Library General Public
 License as published by the Free Software Foundation; either
 version 2 of the License, or (at your option) any later version.
 
 This library is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 Library General Public License for more details.
 
 You should have received a copy of the GNU Library General Public
 License along with this library; if not, write to the Free
 Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
/***************************************************/

#ifndef TIMER_H
#define TIMER_H

#include "AudioDrome.h"
#include <iomanip>
#include <ctime>

#if defined(_WIN32)
#include <time.h>
#else
#include <sys/time.h>
#include <sys/times.h>
#endif

#ifdef _SC_CLK_TCK
static const long CLK_X_TCK = sysconf(_SC_CLK_TCK);
#else
#define CLK_X_TCK CLOCKS_PER_SEC
#endif

#if defined(_WIN32)

inline unsigned long clocktick()
{
	return clock();
}
#else
inline unsigned long clocktick()
{
	static struct tms buf;
	return (unsigned long) times(&buf);
}
#endif

typedef unsigned int uint;

//  begin namespace
namespace Aud
{
	class Timer : public AudioDrome
		{
			friend std::ostream& operator<<(std::ostream& os, Timer& t);
			friend void* threadPrint(void*);
		public:
			//! Public constructor
			Timer();
			
			//! Public destructor
			virtual ~Timer();
			
			//! Initialize the timer counter and print the value
			/*!
			 * This procedure creates a thread, 
			 * Prints the values every setSleepPrintTime() time intervals.
			 * The print format is as follows: 
			 * day: hours: minuts: seconds: milliseconds: microseconds: accumulatedTime
			 */
			void printTimerStart() throw(AudError);
			
			//! Stop the printing process
			/*!
			 * This procedure clears 
			 * the thread of StartTimer().			 
			 */
			void printTimerStop()
			{
				if(_isRunning)
				{
					_isRunning = false;
					pthread_cancel(_thread);
				}
			}
			
			//! Sets the time for the intervals printing
			/*!
			 * The resolution reaches the order of Microseconds. 
			 * But the time, for simplicity, is expressed in MilliSeconds. 
			 * Default: 1000 Ms. ie print a string-value every second.
			 */
			void setPrintTimeSleep(const uint& ms=1000)
			{
				_printWhenMs = ms;
			}			
			
			//! Prints the time values
			/*!
			 * The print format is as follows: 
			 * day: hours: minuts: seconds: milliseconds: microseconds
			 * This function NOT CREATES A SPECIFIC THREAD, can be used in the current thread.
			 */
			inline void printTime()
			{
				this->elapsedTime();
				
				std::cout << _day << " : " << _hours 
				<< " : " << _minuts<< " : " << _seconds << " : " 
				<< std::setprecision (8) << _us << std::endl;
				
				std::cout.flush();
			}
			
			//! Gets the days passed from the resetTimer()
			inline uint getDay() { this->elapsedTime(); return _day; }
			
			//! Gets the hours passed from the resetTimer()
			inline uint getHours() { this->elapsedTime(); return _hours; }
			
			//! Gets the minuts passed from the resetTimer()
			inline uint getMinuts() { this->elapsedTime(); return _minuts; }
			
			//! Gets the seconds passed from the resetTimer()
			inline uint getSeconds() { this->elapsedTime(); return _seconds; }
			
			//! Gets the microseconds passed from the resetTimer()
			inline uint getMicroSeconds() { this->elapsedTime(); return _us; }
			
			//! Gets the accumulated time passed from the resetTimer()
			/*!
			 * The value of the accumulated time is in seconds.
			 */
			double getAccumulatedTime(){ this->elapsedTime(); return _accumTime; }
			
			//! resets the counter parameters
			void resetTimer();
			
			//! Sets a breakpoint at the beginning of the scanning timee
			/*!
			 * You can set up a message to be printed. 
			 * N.B. When invoked this feature, is not possible re-invoked it. 
			 * Only after reading the time, with readTimeAfterMarker (), is possible retrieve it.			 
			 */
			void putMarkerTime(const std::string& msg="")
			{
				if (msg.size() > 0) 
					std::cout << msg << std::endl;
				_startClock = clocktick();
			}
			
			//! Gets the elapsed time, by putMarkerTime()
			/*!
			 * N.B. The invocation of this function must be 
			 * After putMarkerTime (). Otherwise returns a null value.
			 */
			double readTimeAfterMarker()
			{
				return (double)(clocktick() - _startClock) / CLK_X_TCK;
			}
			
		private:
			time_t _startTime;
			unsigned long _startClock;
			struct timeval _tv;
			struct timezone _tz;
			pthread_t _thread;
			bool _isRunning;
			uint _day, _hours, _minuts, _seconds, _us, _printWhenMs;
			double _accumTime;
			
			//! Gets the elapsed time, by startTimer()
			void elapsedTime();
		};
	
	inline std::ostream& operator<<(std::ostream& os, Timer& t)
	{
		t.printTime();		
		return os;
	}
} //	end of namespace Aud
#endif
///EOF
