/***************************************************/
/*! \class Trapezoid
 \brief
 
 The AudioDrome Trapezoid class, implements 
 a trapezoidal envelope generator, based on the control 
 of the attack and decay.
 The values of attack and decay,
 are expressed as a percentage of total duration.
 therefore, the "sustain" value, is implicit:
 
 _sus = _dur - (_attack + _decay);
 
 by Alessandro Petrolati 2006.
 www.alessandro-petrolati.it
 info@alessandro-petrolati.it
 
 This library is free software; you can redistribute it and/or
 modify it under the terms of the GNU Library General Public
 License as published by the Free Software Foundation; either
 version 2 of the License, or (at your option) any later version.
 
 This library is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 Library General Public License for more details.
 
 You should have received a copy of the GNU Library General Public
 License along with this library; if not, write to the Free
 Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
/***************************************************/

#ifndef TRAPEZOID_H_
#define TRAPEZOID_H_

#include "Phasor.h"

//  begin namespace
namespace Aud
{
	class Trapezoid : public Phasor
		{
		public:
			//! Public constructor
			Trapezoid();
			
			//! Public destructor
			virtual ~Trapezoid();
			
			//! Sets the Duration, attack and decay parameters
			/*!
			 * The duration is expressed in seconds, the attack and decay 
			 * they are expressed in percent (%) of the duration. 
			 * For the attack and decay, Are possibles values between 
			 * 0 and 0.5, (0.5 = 50% of the duration) 
			 * Set a 0.5 value for the attack and for the decay, 
			 * to obtain a triangular prototype. 
			 * Values smaller than 0.5 (50%), produces a trapezoide. 
			 * With values close to 0, the prototype degenerates into a rectangular.			 
			 *
			 * N.B. This function INITIALIZE the envelope, so similar proceeds to this->setDuration();
			 * This function is equivalent to the call of the following: 
			 * setDuration();
			 * setAttack();
			 * setDecay();
			 * Sets the parameters ASD (Attack, Sustain and Decay).
			 * After the trapezoid generation, the last value is returned.
			 */
			void setParameters(const AudFloat &dur, const AudFloat &attack,
							   const AudFloat &decay);
			
			//! Sets the duration time in seconds of the envelope
			void setDuration(const AudFloat &dur);
			
			//! Sets the attack percentage of the envelope (0.0 - 0.5)
			void setAttack(const AudFloat &attack);
			
			//! Sets the decay percentage of the envelope (0.0 - 0.5)
			void setDecay(const AudFloat &decay);
			
			//! Reset the envelope again based on the setup parameters
			void trigger();
			
		protected:
			//! Processing the audio sample
			virtual void Processing();
			
			AudFloat _attack;
			AudFloat _decay;
			AudFloat _sus;
			bool _trig;
		};
} //	end of namespace Aud

#endif /*TRAPEZOID_H_*/


