/***************************************************/
/*! \class Trigger
 \brief
 
 The AudioDrome Trigger class, informs 
 when a signal crosses a threshold.
 
 Questa classe implementa un generatore
 di trigger. Il trigger viene emesso quando
 il segnale in ingresso attraversa la soglia
 di valore indicata in threshold..
 
 by Alessandro Petrolati 2006.
 www.alessandro-petrolati.it
 info@alessandro-petrolati.it
 
 This library is free software; you can redistribute it and/or
 modify it under the terms of the GNU Library General Public
 License as published by the Free Software Foundation; either
 version 2 of the License, or (at your option) any later version.
 
 This library is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 Library General Public License for more details.
 
 You should have received a copy of the GNU Library General Public
 License along with this library; if not, write to the Free
 Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
/***************************************************/

#ifndef TRIGGER_H
#define TRIGGER_H

#include "AudioDrome.h"

//  begin namespace
namespace Aud
{	
	class Trigger : public AudioDrome
		{
		public:
			//! Public constructor
			Trigger(const AudFloat &threshold=0.5, const TRIGGER &mode=UP);
			
			//! Public destructor
			virtual ~Trigger();
			
			//! Set the algorithm processing.
			/*!
			 * Normally trigger outputs "false":
			 * only each time input signal crosses 
			 * kthreshold trigger outputs a "true". 
			 * There are three modes of using ktrig:
			 *
			 * UP outputs "true"  when current value 
			 * of input signal is higher than "threshold", 
			 * while old value of input signal
			 * was equal to or lower than kthreshold.
			 *
			 * DOWN outputs "true" when current value of 
			 * input signal is lower than kthreshold while old 
			 * value of input signal was equal or higher than kthreshold.
			 *
			 * BOTH outputs "true" in both the two previous cases.
			 */
			void setMode(const TRIGGER &mode=UP);
			
			//! Sets the trigger threshold
			void setThresHold(const AudFloat &threshold=0.5);
			
		protected:
			//! Processing the audio sample
			/*!
			 * When the value of the Input() signal crosses the threshold 
			 * Output(), returns AudFloat 1.0, otherwise = 0.0.
			 * See the setThresHold() method.			 
			 */
			virtual void Processing();
			
		private:
			AudFloat _threshold;
			AudFloat _old_sig;
			AudFloat (Trigger::*_modePtr) (const AudFloat &);
			AudFloat Up (const AudFloat &input);
			AudFloat Down (const AudFloat &input);
			AudFloat Both (const AudFloat &input);
		};
} //	end of namespace Aud
#endif
///EOF
