/***************************************************/
/*! \class UpSampling
 \brief
 
 The AudioDrome UpSampling class, implements a 
 linear interpolation, converts a control 
 signal to an audio signal.
 Given an input value,  returns 
 setNumberSamples() samples interpolated.
 
 by Alessandro Petrolati 2006.
 www.alessandro-petrolati.it
 info@alessandro-petrolati.it
 
 This library is free software; you can redistribute it and/or
 modify it under the terms of the GNU Library General Public
 License as published by the Free Software Foundation; either
 version 2 of the License, or (at your option) any later version.
 
 This library is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 Library General Public License for more details.
 
 You should have received a copy of the GNU Library General Public
 License along with this library; if not, write to the Free
 Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
/***************************************************/

#ifndef UPSAMPLING_H
#define UPSAMPLING_H

#include "Interpolation.h"

//  begin namespace
namespace Aud
{
	class UpSampling : public Interpolation
		{
		public:
			//! Public constructor
			UpSampling(const ul& nsamp=0);
			
			//! Public constructor
			UpSampling(AudioDrome& obj=NULL);
			
			//! Public destructor
			virtual ~UpSampling();
			
			//! Sets the numbers point of interpolation
			/*!
			 * Returns a block of setUpSampling() interpolated  
			 * samples, each input signal.
			 */
			void setUpSampling(const ul& nsamp=0);
			
			//! Sets the numbers point of interpolation
			/*!
			 * Returns a block of setUpSampling() interpolated  
			 * samples, each input signal of implicit AudioDrome object.
			 */
			void setUpSampling(AudioDrome& obj=NULL)
			{
				this->setUpSampling( obj._smplBlock );
			}
			
		protected:
			//! Processing the upsampling audio sample
			/*!
			 * This function performs interpolations on a number of points.
			 * If the input signal is downsampling, this function may 
			 * be called with a higher sampling, the number of points must be equal to the downsampling.
			 * Converts a control signal to an audio signal using linear interpolation.
			 * Given an input value,  returns setNumberSamples() samples interpolated.
			 * N.B. INPUT SIGNAL CHANGES EVERY TIMES setNumberPoint();
			 * An AudioDrome downsampling is correctly interpolated when:
			 * setNumberPoints( downsamplingFactor );
			 * Ex:
			 * AudFloat sample;
			 * AudFloat downsamplingFactor = 4410;
			 * UpSampling us;
			 * us.setNumberPoints( _localSR / downsamplingFactor;
			 * Metronome mt;
			 * mt.setFrequency( downsamplingFactor );
			 * Sinusoide sin;
			 * sin.setFrequency(440);
			 *
			 * while {
			 * mt.PROCESSING();
			 * if ( mt.Output() )
			 * {
			 *    sin.PROCESSING(); 
			 *    sample = sin.Output();
			 * }
			 * us.PROCESSING();
			 * us.Input( sample );
			 * sample = us.Output();
			 * ...
			 */
			virtual void Processing();
			
		private:
			ul _counter;
			ul _nsamp;
		};
} //	end of namespace Aud
#endif
///EOF
