/***************************************************/
/*! \class Analog
 \brief
 
 The AudioDrome Analog class, implements 
 a model of generator to limited-band for the
 common prototypes.
 
 John Lane, Dan Hoory, Ed Martinez and Patty Wang
 Modelling Analog Synthesis with DSPs
 Computer Music Journal, 21:4, pp. 23-41, Winter 1997
 
 by Alessandro Petrolati 2008.
 www.alessandro-petrolati.it
 info@alessandro-petrolati.it
 
 This library is free software; you can redistribute it and/or
 modify it under the terms of the GNU Library General Public
 License as published by the Free Software Foundation; either
 version 2 of the License, or (at your option) any later version.
 
 This library is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 Library General Public License for more details.
 
 You should have received a copy of the GNU Library General Public
 License along with this library; if not, write to the Free
 Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
/***************************************************/

#include "Analog.h"

using namespace Aud; 

namespace Aud
{
	Analog::Analog()
	{	
#if __DEBUG__
		std::cout << "Analog Constructor: " << this <<std::endl;
#endif
		_compareSig = 0;
		this->setHarmsNumber(16);
		this->setLpCutOff(9000);
		this->setFrequency(440);
		this->setOutputWaveform(SINE);
	}
	
	Analog::~Analog()
	{
#if __DEBUG__
		std::cout << "Analog Destructor: " << this <<std::endl;
#endif	
	}
	
	void Analog::setFrequency(const AudFloat& hz) /*! Sampling Rate Change SLOT */
	{
		_freqLast = hz;
		_osc1.setFrequency(_freqLast / 2.0);
		_osc2.setFrequency(_freqLast);
		AudFloat hpCutOff = _harms * _freqLast;
		_hpCutOff = (hpCutOff >= (_localSR * 0.45) ? (_localSR * 0.45) : hpCutOff);
		_atoneA.setCutOff(hpCutOff);
		_atoneB.setCutOff(hpCutOff);				
	}
	
	void Analog::setAmplitude(const AudFloat& resc)
	{
		_osc1.setAmplitude(resc);
		_osc2.setAmplitude(resc);
	}
	
	void Analog::setLpCutOff(const AudFloat& hz)
	{
		_lpbut1A.setCutOff(hz);
		_lpbut2A.setCutOff(hz);
		_lpbut3A.setCutOff(hz);	
		//branch B
		_lpbut1B.setCutOff(hz);
		_lpbut2B.setCutOff(hz);
		_lpbut3B.setCutOff(hz);	
	}
	
	void Analog::setHarmsNumber(const AudFloat& hn)
	{
		_harms = hn;
	}
	
	void Analog::setOutputWaveform(const WAVESHAPE& type)
	{
		switch (type)
		{
			case (SAWTOOTH):
				_outPtr = &Analog::Sawtooth;
				break;
			case (TRIANGLE):
				_outPtr = &Analog::Triangle;
				break;
			case (SQUARE):
				_outPtr = &Analog::Square;
				break;
			case (SINE):
				_outPtr = &Analog::Sine;
				break;
				
			default:
				_outPtr = &Analog::Sine;
		}
	}
	
	void Analog::Processing()
	{ 
		//Output
		_dc.Input( (this->*_outPtr)() );
		_dc.PROCESSING();
		_rms.Input( _dc.Output() );
		_rms.InputCompareSignal(_compareSig/2.0);
		_rms.PROCESSING();
		_output_1 = _rms.Output();
	}
	
	AudFloat Analog::Sawtooth()
	{
		//branch A
		_osc1.PROCESSING();
		AudFloat absRamoA = 2.0 * (fabs(_compareSig = _osc1.Output()) - 0.5);
		_lpbut1A.Input(absRamoA);
		_lpbut1A.PROCESSING();
		_lpbut2A.Input( _lpbut1A.Output() );
		_lpbut2A.PROCESSING();
		_lpbut3A.Input( _lpbut2A.Output() );
		_lpbut3A.PROCESSING();
		_atoneA.Input( _lpbut3A.Output() );
		_atoneA.PROCESSING();
		return _atoneA.Output();
	}
	
	AudFloat Analog::Triangle()
	{
		//branch A
		_osc1.PROCESSING();
		AudFloat absRamoA = 2.0 * (fabs(_osc1.Output()) - 0.5);
		//branch B
		_osc2.PROCESSING();
		AudFloat absRamoB = 2.0 * (fabs( _compareSig = _osc2.Output() ) - 0.5);
		AudFloat nodoSomma = absRamoA + (absRamoB * -(1.0 / 5.0));
		
		
		_lpbut1B.Input(nodoSomma);
		_lpbut1B.PROCESSING();
		_lpbut2B.Input( _lpbut1B.Output() );
		_lpbut2B.PROCESSING();
		_lpbut3B.Input( _lpbut2B.Output() );
		_lpbut3B.PROCESSING();
		
		return _lpbut3B.Output();
	}
	
	AudFloat Analog::Square()
	{				
		_atoneB.Input( this->Triangle() );
		_atoneB.PROCESSING();
		return _atoneB.Output();				
	}
	
	AudFloat Analog::Sine()
	{
		_osc2.PROCESSING();
		return _compareSig = _osc2.Output();
	}
} //	end of namespace Aud
///EOF
