/***************************************************/
/*! \class AudioDuplex
 \brief
 
 The AudioDrome AudioDuplex class, is a wrapper
 for the PortAudio_v19 pa_stable_v19_20071207
 by Ross Bencina
 
 This class are not affected by _absoluteSR changing notification,
 in fact the hardware audio stream, or VST (RTAS o AUDIOUNIT) sampling rate in a host
 system, is considered like _absoluteSR value.
 
 $Id: portaudio.h 1247 2007-08-11 16:29:09Z rossb $
 PortAudio Portable Real-Time Audio Library
 PortAudio API Header File
 Latest version available at: http://www.portaudio.com/
 
 Copyright (c) 1999-2002 Ross Bencina and Phil Burk
 
 by Alessandro Petrolati 2006.
 www.alessandro-petrolati.it
 info@alessandro-petrolati.it
 
 This library is free software; you can redistribute it and/or
 modify it under the terms of the GNU Library General Public
 License as published by the Free Software Foundation; either
 version 2 of the License, or (at your option) any later version.
 
 This library is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 Library General Public License for more details.
 
 You should have received a copy of the GNU Library General Public
 License along with this library; if not, write to the Free
 Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
/***************************************************/

#ifdef __WITH_PORTAUDIO__
#include "AudioDuplex.h"

using namespace Aud; 

//  begin namespace
namespace Aud
{
	AudioDuplex::AudioDuplex(const PaDeviceIndex &deviceInput, 
							 const PaDeviceIndex &deviceOutput,
							 const int &chnls,
							 const unsigned long &framesPerBuffer,
							 const PaSampleFormat &smpFormat,
							 const PaStreamFlags &flags,
							 const bool &openStarted)
	: AudioIO(chnls, framesPerBuffer, smpFormat, flags), _indexBufRead(0), _indexBufWrite(0)
	{
#if __DEBUG__
		std::cout << "AudioDuplex Constructor: "<< this<<std::endl;
#endif
		_bufRead = new PaFloat[_samplesBuffer];
		for(ul i = 0; i < _samplesBuffer; i++)
			_bufRead[i] = 0.0;
		
		
		this->setParameters(deviceInput, deviceOutput, chnls, smpFormat);
		
		if(openStarted)
		{
			this->openStream();
			this->startStream();
		}
	}
	
	AudioDuplex::~AudioDuplex()
	{
#if __DEBUG__
		std::cout << "AudioDuplex Destructor: "<< this<<std::endl;
#endif
	}
	
	void AudioDuplex::setParameters(const PaDeviceIndex &deviceInput, 
									const PaDeviceIndex &deviceOutput, 
									const int &chnls, 
									const PaSampleFormat &smpFormat)
	{		
		bool start = false;
		bool open = false;
		
		if( this->isStreamStarted() )
		{
			this->stopStream();
			start = true;
		}
		if( this->isStreamOpened() )
		{
			this->closeStream();
			open = true;
		}
		
		/* Input Parameters*/
		_parametersInput.device = ( PaDeviceIndex) 
		(deviceInput == -1 ? this->getDefaultInputDevice() : deviceInput); 
		_parametersInput.channelCount = chnls; /* 2 = Stereo output */
		_parametersInput.sampleFormat = smpFormat;
		_parametersInput.suggestedLatency = 
		Pa_GetDeviceInfo(_parametersInput.device)->defaultLowOutputLatency;
		_parametersInput.hostApiSpecificStreamInfo = NULL;	
		
		/* Output Parameters*/
		_parameters.device = ( PaDeviceIndex) 
		(deviceOutput == -1 ? this->getDefaultOutputDevice() : deviceOutput); 
		_parameters.channelCount = chnls; /* 2 = Stereo output */
		_parameters.sampleFormat = smpFormat;
		_parameters.suggestedLatency = 
		Pa_GetDeviceInfo(_parameters.device)->defaultLowOutputLatency;
		_parameters.hostApiSpecificStreamInfo = NULL;
		
		this->setFramesPerBuffer(_framesPerBuffer);
		
		if(open)
			this->openStream();
		if(start)
			this->startStream();
	}
	
	
	void AudioDuplex::openStream()  /*! Sampling Rate Change SLOT */
	{
		bool start = false;	
		
		if( this->isStreamStarted() )
		{
			this->stopStream();
			start = true;
		}
		if( this->isStreamOpened() )
			this->closeStream();
		
		/* Open an audio I/O stream. */
		_err = Pa_OpenStream( &_stream,
							 &_parametersInput,
							 &_parameters,
							 AudioDrome::getAbsoluteSR(),
							 _framesPerBuffer,
							 _streamFlags,
							 _streamCallback,
							 _data );					
		
		if( _err != paNoError )
			this->portAudioError( "AudioDuplex::openStream()" );
		_isOpened = true;
		
		if(start)
			this->startStream();
	}
	
	void AudioDuplex::setFramesPerBuffer(const unsigned long &framesPerBuffer)
	{
		
		if(framesPerBuffer != _framesPerBuffer)
		{
			bool start = false;	
			bool open = false;
			
			if( this->isStreamStarted() )
			{
				this->stopStream();
				start = true;
			}
			if( this->isStreamOpened() )
			{
				this->closeStream();
				open = true;
			}
			_framesPerBuffer = framesPerBuffer;
			_samplesBuffer = _framesPerBuffer * _parametersInput.channelCount;
			
			if(_bufRead != NULL)
			{
				delete _bufRead;
				_bufRead = NULL;
			}
			
			if(_bufRead == NULL)
			{
				_bufRead = new PaFloat[_samplesBuffer];
				for(ul i = 0; i < _samplesBuffer; i++)
					_bufRead[i] = 0.0;
			}
			
			AudioIO::setFramesPerBuffer(_framesPerBuffer);
			
			if(open)
				this->openStream();
			if(start)
				this->startStream();
		}
	}
	
	void AudioDuplex::Processing()
	{
		_output_1 = (PaFloat) _bufRead[_indexBufRead];
		_output_2 =	(PaFloat) _bufRead[++_indexBufRead];
		++_indexBufRead;
		
		if (_indexBufRead >= _samplesBuffer)
		{
			_indexBufRead = 0;
			_err = Pa_ReadStream(_stream, _bufRead, _framesPerBuffer );
			//			if (_err != paNoError )
			//				this->portAudioError( "AudioIO::putSignal()" );
		}		
		
		_buffer[_indexBufWrite] = (PaFloat) _input_1;
		_buffer[++_indexBufWrite]= (PaFloat) _input_2;
		++_indexBufWrite;
		
		if (_indexBufWrite >= _samplesBuffer)
		{
			_indexBufWrite = 0;
			_err = Pa_WriteStream(_stream, _buffer, _framesPerBuffer );
			//			if (_err != paNoError )
			//				this->portAudioError( "AudioIO::putSignal()" );
		}				
		
		// Loop connection
		//		_err = Pa_WriteStream(_stream, _buffer, _framesPerBuffer );
		//		_err = Pa_ReadStream(_stream, _buffer, _framesPerBuffer );
	}
} //	end of namespace Aud
#endif
///EOF

