/***************************************************/
/*! \class AudioIO
 \brief
 
 The AudioDrome AudioIO class, is a wrapper
 for the PortAudio_v19 pa_stable_v19_20071207
 by Ross Bencina
 
 This class are not affected by _absoluteSR changing notification,
 in fact the hardware audio stream, or VST (RTAS o AUDIOUNIT) sampling rate in a host
 system, is considered like _absoluteSR value.
 
 $Id: portaudio.h 1247 2007-08-11 16:29:09Z rossb $
 PortAudio Portable Real-Time Audio Library
 PortAudio API Header File
 Latest version available at: http://www.portaudio.com/
 
 Copyright (c) 1999-2002 Ross Bencina and Phil Burk
 
 by Alessandro Petrolati 2006.
 www.alessandro-petrolati.it
 info@alessandro-petrolati.it
 
 This library is free software; you can redistribute it and/or
 modify it under the terms of the GNU Library General Public
 License as published by the Free Software Foundation; either
 version 2 of the License, or (at your option) any later version.
 
 This library is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 Library General Public License for more details.
 
 You should have received a copy of the GNU Library General Public
 License along with this library; if not, write to the Free
 Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
/***************************************************/

#ifdef __WITH_PORTAUDIO__
#include "AudioIO.h"

using namespace Aud; 

//  begin namespace
namespace Aud
{	
	short AudioIO::_isInitialized = 0;
	
	AudioIO::AudioIO(const int &chnls, 
					 const unsigned long &framesPerBuffer,
					 const PaSampleFormat &smpFormat,
					 const PaStreamFlags &flags)
	: _framesPerBuffer(framesPerBuffer), 
	_isOpened(false), 
	_isStarted(false), 
	_streamCallback(0), 
	_data(0), 
	_buffer(0),
	_channel(STEREO)
	{
#if __DEBUG__
		std::cout << "AudioIO Constructor: "<< this << std::endl;
#endif
		
		/* -- initialize PortAudio -- */
		if(_isInitialized == 0)
		{
			_err = Pa_Initialize();
			if( _err != paNoError )
				this->portAudioError( "AudioIO::AudioIO() Constructor");
			
			_isInitialized++;
		}
		
		_samplesBuffer = _framesPerBuffer * chnls;
		_streamFlags = flags;
		_framesPerBuffer = framesPerBuffer;
		
		_buffer = new PaFloat[_samplesBuffer];
		for(ul i = 0; i < _samplesBuffer; i++)
			_buffer[i] = 0.0;
		//		this->setParameters(device, chnls, smpFormat);
	}
	
	AudioIO::~AudioIO()
	{
#if __DEBUG__
		std::cout << "AudioIO Destructor: "<< this<<std::endl;
#endif
		if( !this->isStreamStarted() )
			this->stopStream();
		if( this->isStreamOpened() )
			this->closeStream();
		
		if(_buffer != NULL)
			delete _buffer;
		
		_isInitialized--;
		
		if(_isInitialized == 0)
			_err = Pa_Terminate();
		if( _err != paNoError )
			this->portAudioError( "AudioIO::~AudioIO() Destructor" );
	}
	
	void AudioIO::closeStream()	
	{
		if ( this->isStreamOpened() )
		{
			if ( this->isStreamStarted() )
				this->stopStream();
			
			_err = Pa_CloseStream( _stream );
			
			if (_err != paNoError )
				this->portAudioError( "AudioIO::closeStream()" );
			_isOpened = false;
			_isStarted = false;
		}
	}
	
	void AudioIO::startStream()
	{
		if ( this->isStreamOpened() )
		{
			if( !this->isStreamStarted() )
			{
				_err = Pa_StartStream( _stream );
				if (_err != paNoError )
					this->portAudioError( "AudioIO::startStream()"); 
				_isStarted = true;
			}
		}
	}
	
	void AudioIO::stopStream()
	{
		if ( this->isStreamOpened())
		{
			if(this->isStreamStarted())
			{
				_err = Pa_StopStream( _stream );
				if (_err != paNoError )
					this->portAudioError( "AudioIO::stopStream()");
				_isStarted = false;
			}
		}
	}
	
	void AudioIO::abortStream()
	{
		Pa_AbortStream( _stream );
		Pa_CloseStream( _stream );
		_isInitialized--;
		_isOpened = false;
		_isStarted = false;
	}
	
	void AudioIO::portAudioError(const std::string& myError)
	{
		fprintf(stderr, "An error occured while using the portaudio stream\n");
		fprintf(stderr, "Error number: %d\n", _err);
		std::cout <<  "PortAudio error message: " << Pa_GetErrorText( _err ) << std::endl;
		std::cout << "The error was caused by: " << myError << " Function!" << std::endl;
	}	
	
	void AudioIO::printAllHostDevices()
	{
		int devCount, apiCount;//, devDefault;
		const PaDeviceInfo *devInfo;
		const PaHostApiInfo *hostApiInfo;
		//    PaHostApiIndex apiIndx,apiDefault;
		//    PaDeviceIndex devIndx = -1;
		
		apiCount = Pa_GetHostApiCount (); ///number of avaibles hosts
		devCount = Pa_GetDeviceCount(); ///total numbers of devices
		//    devDefault = Pa_GetDefaultHostApi ();
		
		std::cout << "Driver Type"<< std::endl;
		std::cout << "==========="<< std::endl;
		for (int i = 0; i < apiCount; i++)
		{
			hostApiInfo = Pa_GetHostApiInfo (i);
			std::cout << "Host "<< i <<": "<< hostApiInfo->name<<std::endl;
			std::cout << "Devices "<< hostApiInfo->deviceCount<< std::endl;
			std::cout << "Default Input "<< hostApiInfo->defaultInputDevice<< std::endl;
			std::cout << "Default Output "<< hostApiInfo->defaultOutputDevice<< std::endl;
			std::cout << "----------------------------------------------"<< std::endl;
		}
		
		std::cout << std::endl << "Audio Devices"<< std::endl;
		std::cout << "============="<< std::endl;
		for (int i = 0; i < devCount; i++)
		{
			devInfo = Pa_GetDeviceInfo (i);
			std::cout << i << "   "<< devInfo->name<<std::endl;
			if (devInfo->maxInputChannels > 0)
				std::cout << "Input >>"<< std::endl;
			if (devInfo->maxOutputChannels > 0)
				std::cout << "Output <<"<< std::endl;
			std::cout << "Max Input Channels " << devInfo->maxInputChannels << std::endl;
			std::cout << "Max Output Channels " << devInfo->maxOutputChannels << std::endl;
			std::cout << "----------------------------------------------"<< std::endl;
		}
	}
	
	void AudioIO::setDevice(const PaDeviceIndex &devIndx)
	{
		bool start=false;
		bool open=false;
		
		if(isStreamStarted())
		{
			this->stopStream();
			start = true;
		}
		
		if(isStreamOpened())
		{
			this->closeStream();
			open = true;
		}
		
		PaDeviceIndex _devIndx;
		//		const PaStreamInfo* info;
		
		if (devIndx == -1)
		{
			std::cout << "Select a input/output audio device for the: " << std::endl 
			<< AudioDrome::getClassName() << " Object." << std::endl;
			std::cin >> _devIndx;
		}
		else
		{
			_devIndx = devIndx;
		}
		_parameters.device = _devIndx ;
		
		if(open)
			this->openStream();
		if(start)
			this->startStream();
	}
	
	void AudioIO::setFramesPerBuffer(const unsigned long &framesPerBuffer)
	{
		if(framesPerBuffer != _framesPerBuffer)
		{
			bool start = false;	
			bool open = false;
			
			if( this->isStreamStarted() )
			{
				this->stopStream();
				start = true;
			}
			if( this->isStreamOpened() )
			{
				this->closeStream();
				open = true;
			}
			_framesPerBuffer = framesPerBuffer;
			_samplesBuffer = _framesPerBuffer * _parameters.channelCount;
			
			if(_buffer != NULL)
			{
				delete _buffer;
				_buffer = NULL;
			}
			
			if(_buffer == NULL)
			{
				_buffer = new PaFloat[_samplesBuffer];
				for(ul i = 0; i < _samplesBuffer; i++)
					_buffer[i] = 0.0;
			}
			
			if(open)
				this->openStream();
			if(start)
				this->startStream();
		}
	}
} //	end of namespace Aud
#endif
///EOF
