/***************************************************/
/*! \class BP_Butterworth Filter
 \brief
 
 The AudioDrome BP_Butterworth class, implements a 
 band-pass Butterworth filter. 
 Coded by Paris Smaragdis 1994
 
 Csound's code butter.c
 
 by Alessandro Petrolati 2006.
 www.alessandro-petrolati.it
 info@alessandro-petrolati.it
 
 This library is free software; you can redistribute it and/or
 modify it under the terms of the GNU Library General Public
 License as published by the Free Software Foundation; either
 version 2 of the License, or (at your option) any later version.
 
 This library is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 Library General Public License for more details.
 
 You should have received a copy of the GNU Library General Public
 License along with this library; if not, write to the Free
 Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
/***************************************************/

#include "BP_Butterworth.h"
#include <iostream>

using namespace Aud; 

//  begin namespace
namespace Aud
{
	BP_Butterworth::BP_Butterworth(const AudFloat &centerFreq,
								   const AudFloat &bandWidth)
	{
#if __DEBUG__
		std::cout << "Butterworth BandPass Filter Constructor: "<< this <<std::endl;
#endif
		_bw = bandWidth;
		_cf = centerFreq;
		this->setCenterFrequency(centerFreq);
	}
	
	BP_Butterworth::~BP_Butterworth()
	{
#if __DEBUG__
		std::cout << "Butterworth BandPass Filter Destructor: "<< this <<std::endl;
#endif
	}
	
	void BP_Butterworth::setCenterFrequency(const AudFloat &centerFreq)
	{
		_cf = centerFreq;
		AudFloat c, d;
		c = 1.0 / (AudFloat)tan((double)(_pidsr * _bw));
		d = 2.0 * (AudFloat)cos( 2.0 * (double)(_pidsr * _cf));
		_coeff[1] = 1.0 / ( 1.0 + c);
		_coeff[2] = 0.0;
		_coeff[3] = -_coeff[1];
		_coeff[4] = -c * d * _coeff[1];
		_coeff[5] = (c - 1.0) * _coeff[1];
	}
	
	void BP_Butterworth::setBandWidth(const AudFloat &bandWidth)
	{
		_bw = bandWidth;
		this->setCenterFrequency(_cf);
	}
	
	void BP_Butterworth::setCutOff(const AudFloat &cutOff)
	{
		std::cout
		<< "In order to set up the center frequency band, it uses the function setCenterFrequency()."
		<< this <<std::endl;
	}
} //	end of namespace Aud
///EOF
