/***************************************************/
/*! \class BandPass Filter
 \brief
 
 The AudioDrome BandPass class, implements a second-order band-pass filter.
 
 Csound's code, based on Robin Whittle's
 ugrw2.c and ugrw2.h of 9 September 1995
 
 by Alessandro Petrolati 2006.
 www.alessandro-petrolati.it
 info@alessandro-petrolati.it
 
 This library is free software; you can redistribute it and/or
 modify it under the terms of the GNU Library General Public
 License as published by the Free Software Foundation; either
 version 2 of the License, or (at your option) any later version.
 
 This library is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 Library General Public License for more details.
 
 You should have received a copy of the GNU Library General Public
 License along with this library; if not, write to the Free
 Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
/***************************************************/

#include "BandPass.h"

using namespace Aud; 

//  begin namespace
namespace Aud
{
	BandPass::BandPass(const AudFloat &centerFreq, const AudFloat &bandWidth,
					   const SCALING &scale)
	{
#if __DEBUG__
		std::cout << "BandPass Filter Constructor: "<< this <<std::endl;
#endif
		_scale = scale;
		this->setCenterFrequency(centerFreq);
		this->setBandWidth(bandWidth);
		this->setScaleType(scale);
	}
	
	BandPass::~BandPass()
	{
#if __DEBUG__
		std::cout << "BandPass Filter Destructor: "<< this <<std::endl;
#endif
	}
	
	void BandPass::setCenterFrequency(const AudFloat &centerFreq) /*! Sampling Rate Change SLOT */
	{
		_centerFreq = centerFreq;
		/* Calculations for centre frequency if it changes.
		 * cosf = cos (2pi * freq / krate) */
		_cosf
		= cos((double)(2.0 * M_PI * _centerFreq
					   / _localSR));
		this->scaling(_scale);
	}
	
	void BandPass::setBandWidth(const AudFloat &bandWidth)
	{
		_bandWidth = bandWidth;
		/* Calculations for bandwidth if it changes.
		 * c3 = exp (-2pi * bwidth / krate) */
		_c3 = exp((double)(-2.0 * M_PI * _bandWidth / _localSR));
		this->scaling(_scale);
	}
	
	void BandPass::setScaleType(const SCALING &scale)
	{
		_scale = scale;
	}
	
	void BandPass::Processing()
	{
		/* Filter section, see c1, c2, c3 notes above.  Calculate output and
		 * the new values for the first and second delays.     */
		_output_1 = _c1 * _input_1 + _c2 * _yt1 - _c3 * _yt2;
		_yt2 = _yt1;
		_yt1 = _output_1;
	}
	
	void BandPass::scaling(const SCALING &scale)
	{
		/* Final calculations for the factors
		 * for the filter. Each multiplies
		 * something and sums it to be the
		 * output, and the input to the first
		 * delay.
		 * c1       Gain for input signal.
		 * c2       Gain for output of delay 1.
		 * c3       Gain for output of delay 2.
		 */
		
		_c3p1 = _c3 + 1.0;
		_c3t4 = _c3 * 4.0;
		_omc3 = 1 - _c3;
		
		/* c2= (c3 * 4 * cosf / (c3 + 1)      */
		_c2 = _c3t4 * _cosf / _c3p1; /* -B, so + below */
		_c2sqr = _c2 * _c2;
		
		switch (scale)
		{
			case (NOSCALE):
				/* iscl = 0. No scaling of the signal. Input gain c1 = 1 */
				_c1 = 1.0;
				break;
			case (CENTRE):
			{
				/* iscl = Make gain at centre = 1
				 * c1= (1 - c3) * sqrt( 1 - (c2 * c2 / (c3 * 4) )
				 */
				_c1 = _omc3 * (AudFloat)sqrt((double)1.0 - _c2sqr / _c3t4);
			}
				break;
			case (RMS):
			{
				/* iscl = Higher gain, so "RMS gain" = 1
				 * c1= sqrt((c3 + 1)*(c3 + 1) - cs*c2) * (1 - c3) / (c3 + 1)
				 * (I am not following the maths!)       */
				_c1 = (AudFloat)sqrt((double)((_c3p1*_c3p1-_c2sqr) * _omc3/_c3p1));
			}
				break;
			default:
				/* iscl = 0. No scaling of the signal. Input gain c1 = 1 */
				_c1 = 1.0;
		}
	}
} //	end of namespace Aud
///EOF
