/***************************************************/
/*! \class BandReject Filter (notch)
 \brief
 
 The AudioDrome BandReject class, implements a second-order band-reject (notch) filter.
 
 Csound's code, based on Robin Whittle's
 ugrw2.c and ugrw2.h of 9 September 1995
 
 by Alessandro Petrolati 2006.
 www.alessandro-petrolati.it
 info@alessandro-petrolati.it
 
 This library is free software; you can redistribute it and/or
 modify it under the terms of the GNU Library General Public
 License as published by the Free Software Foundation; either
 version 2 of the License, or (at your option) any later version.
 
 This library is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 Library General Public License for more details.
 
 You should have received a copy of the GNU Library General Public
 License along with this library; if not, write to the Free
 Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
/***************************************************/

#include "BandReject.h"

using namespace Aud; 

//  begin namespace
namespace Aud
{
	BandReject::BandReject(const AudFloat &centerFreq, const AudFloat &bandWidth,
						   const SCALING &scale) :
	BandPass(centerFreq, bandWidth, scale)
	{
#if __DEBUG__
		std::cout << "BandReject Filter Constructor: "<< this <<std::endl;
#endif
		_D = 2.0;
	}
	
	BandReject::~BandReject()
	{
#if __DEBUG__
		std::cout << "BandReject Filter Destructor: "<< this <<std::endl;
#endif
	}
	
	void BandReject::Processing()
	{
		_output_1 = _c1 * _input_1 + _c2 * _yt1 - _c3 * _yt2;
		_yt2 = _yt1;
		
		if (_scale == NOSCALE || _scale == CENTRE)
			_yt1 = _output_1 - _input_1; /* yt1 contains yt1-xt1 */
		
		else if (_scale == RMS)
			_yt1 = _output_1 - _D * _input_1; /* yt1 contains yt1-D*xt1 */
	}
	
	void BandReject::scaling(const SCALING &scale)
	{
		/* Final calculations for the factors
		 * for the filter. Each multiplies
		 * something and sums it to be the
		 * output, and the input to the first
		 * delay.
		 * c1       Gain for input signal.
		 * c2       Gain for output of delay 1.
		 * c3       Gain for output of delay 2.
		 */
		
		_c3p1 = _c3 + 1.0;
		_c3t4 = _c3 * 4.0;
		_omc3 = 1 - _c3;
		
		/* c2= (c3 * 4 * cosf / (c3 + 1)      */
		_c2 = _c3t4 * _cosf / _c3p1; /* -B, so + below */
		_c2sqr = _c2 * _c2;
		
		switch (scale)
		{
			case (NOSCALE):
			{
				/* iscl = 0. No scaling of the signal. Input gain c1 = 1 */
				_c1 = 0.0; /* cannot tell        */
			}
				break;
			case (CENTRE):
				/* iscl = Make gain at centre = 1 */
				_c1 = 1.0 - _omc3 * (AudFloat)sqrt((double)1.0 - _c2sqr / _c3t4);
				break;
			case (RMS):
				/* iscl = Higher gain, so "RMS gain" = 1.*/
				_c1 = _D - (AudFloat)sqrt((double)((_c3p1*_c3p1-_c2sqr)*_omc3/_c3p1));
				break;
			default:
				/* iscl = 0. No scaling of the signal. Input gain c1 = 1 */
				_c1 = 0.0; /* cannot tell */
		}
	}
} //	end of namespace Aud
///EOF
