/***************************************************/
/*! \class Buffer
 \brief
 
 The AudioDrome Buffer implements an abstraction of a buffer array. 
 The class interface is a collection of methods for data manipulation.
 
 by Alessandro Petrolati 2006.
 www.alessandro-petrolati.it
 info@alessandro-petrolati.it
 
 This library is free software; you can redistribute it and/or
 modify it under the terms of the GNU Library General Public
 License as published by the Free Software Foundation; either
 version 2 of the License, or (at your option) any later version.
 
 This library is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 Library General Public License for more details.
 
 You should have received a copy of the GNU Library General Public
 License along with this library; if not, write to the Free
 Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
/***************************************************/

#include "Buffer.h"

using namespace Aud; 

//  begin namespace
namespace Aud
{
	Buffer::Buffer(const ul &length)
	{
#if __DEBUG__
		std::cout << "Buffer Constructor: "<< this<< std::endl;
#endif
		_extendGuardPoint = 3;///<-- guard point = 2 (cubic interpolation).
		
		// OffSet normalizzato di extend guard point
		_realLimitNorm = 1.0 + (double(_extendGuardPoint)/_size); //(1.0 : _size = x : _extendGuardPoint)
		_size = (length > 0 ? length : 10);
		_table = new AudFloat[_size + _extendGuardPoint];
		for (ul i = 0; i < _size; i++)
			_table[i] = 0.0;
		writeGuardPoint();
	}
	
	Buffer::Buffer(const Buffer& x)
	{
#if __DEBUG__
		std::cout << "Buffer Copy Constructor: "<< this<<std::endl;
#endif
		_extendGuardPoint = x._extendGuardPoint;
		_realLimitNorm = x._realLimitNorm;
		_size = x._size;
		_table = new AudFloat[_size + _extendGuardPoint];
		for (ul i = 0; i < _size; i++)
			_table[i] = x._table[i];
		writeGuardPoint();
	}
	
	Buffer::~Buffer()
	{
#if __DEBUG__
		std::cout << "Buffer Destructor: "<< this<<std::endl;
#endif
		delete [] _table;
	}
	
	AudFloat* Buffer::getPtr() throw(AudError)
	{
#if __DEBUG__
		if (_table == NULL)
			throw AudError(NULL_POINTER);
#endif
		return _table;
	}
	
	ul Buffer::getSize() const
	{
		return _size;
	}
	
	void Buffer::writeRaw(const AudFloat &phi, const AudFloat &input) throw(AudError)
	{
#if __DEBUG__
		if (phi< 0.0 || phi >= _realLimitNorm)
		{
			std::cout << "Buffer::writeRaw() Warning!!! Index Out of range."<< std::endl;
			//			_table[_size] = input;
			AudError er(OUT_OF_RANGE);
			er.setMsgError("Warning!!! Index Out of range.");
			er.setCriticalValue(phi);
			throw er;
		}
#endif
		ul index = (ul) (phi * (AudFloat) _size);
		_table[index] = input;
	}
	
	AudFloat Buffer::readRaw(const AudFloat &phi) throw(AudError)
	{
		
#if __DEBUG__
		if (phi< 0.0 || phi >= _realLimitNorm)
		{
			std::cout << "Buffer::readRaw Warning!!! Index Out of range."<< std::endl;
			//			return _table[_size];
			AudError er(OUT_OF_RANGE);
			er.setMsgError("Warning!!! Index Out of range.");
			er.setCriticalValue(phi);
			throw er;
		}
#endif
		ul index = (ul) (phi * (AudFloat) _size); //NBNBNBNBNB _size + 0.5 only necessary for AudFloat = double ?!?!?!
		return _table[index];
	}
	
	AudFloat Buffer::readInterp(const AudFloat &phi) throw(AudError)
	{
#if __DEBUG__
		if (phi< 0.0 || phi >= _realLimitNorm)
		{
			std::cout << "Buffer::readInterp() Warning!!! Index Out of range."<< std::endl;
			AudError er(OUT_OF_RANGE);
			er.setMsgError("Warning!!! Index Out of range.");
			er.setCriticalValue(phi);
			throw er;
		}
#endif
		//Linear Interpolation (teo points)
		AudFloat tmpPhi = phi * (AudFloat) _size;
		ul index = (ul) (tmpPhi);
		AudFloat dif = tmpPhi - (AudFloat) index;
		AudFloat sample1 = _table[index];
		AudFloat sample2 = _table[index + 1];
		return sample1 + (sample2 - sample1) * dif;
		
		//Code optimized NB same results even worse!
		//		AudFloat *ptr = _table;
		//		AudFloat tmpPhi = phi * AudFloat(_size);
		//		ptr += ul(tmpPhi);
		//		return *ptr + (*(ptr+1) - *ptr) * (tmpPhi - AudFloat(ul(tmpPhi)));		
	}
	
	AudFloat Buffer::readInterp3(const AudFloat &phi) throw(AudError)
	{
#if __DEBUG__
		if (phi< 0.0 || phi >= _realLimitNorm)
		{
			std::cout << "Buffer::readInterp3() Warning!!! Index Out of range."<< std::endl;
			AudError er(OUT_OF_RANGE);
			er.setMsgError("Warning!!! Index Out of range.");
			er.setCriticalValue(phi);
			throw er;
		}
#endif
		
		///Cubic Interpolation (three points)
		AudFloat tmpPhi = phi * (AudFloat) _size;
		ul index = (ul) (tmpPhi);
		AudFloat fract = tmpPhi - (AudFloat) index;
		ul indx2 = (index == 0 ? _size-1 : index-1);
		AudFloat ym1 = _table[indx2];
		AudFloat y0 = _table[index];
		AudFloat y1 = _table[index+1];
		AudFloat y2 = _table[index+2];
		AudFloat t1 = y2 + y0+y0+y0;
		AudFloat frsq = fract*fract;
		AudFloat frcu = frsq*(AudFloat)ym1;
		return y0 + 0.5 * frcu + fract * (y1 - frcu/6.0- t1/6.0- ym1/3.0)+frsq
		* fract * (t1/6.0- 0.5 * y1)+ frsq * (0.5 * y1 - y0);
	}
	
	AudFloat Buffer::readInterp3Varga(const AudFloat &phi) throw(AudError)
	{
#if __DEBUG__
		if (phi< 0.0 || phi >= _realLimitNorm)
		{
			std::cout << "Buffer::readInterp3Varga() Warning!!! Index Out of range."<< std::endl;
			AudError er(OUT_OF_RANGE);
			er.setMsgError("Warning!!! Index Out of range.");
			er.setCriticalValue(phi);
			throw er;
		}
#endif
		
		/* Optimized Csound Code by Istvan Varga (Oct 2001) */
		AudFloat tmpPhi = phi * (AudFloat) _size;
		ul index = (ul) (tmpPhi);
		//AudFloat fract = tmpPhi - (AudFloat) index;
		AudFloat w, x, y, z;
		ul v0, v1, v2, v3;
		AudFloat fv1 = phi * _size;
		v1 = (ul) fv1;
		fv1 -=(ul)index;
		v2 = (v1 == _size-1 ? 0L : v1+1L);
		v0 = (v1 == 0 ? _size-1 : v1-1);
		v3 = (v2 == _size-1 ? 0 : v2+1);
		z = fv1 * fv1;
		z--;
		z *= 0.1666666667;
		y = fv1;
		y++;
		w = (y *= 0.5);
		w--;
		x = 3.0 * z;
		y -= x;
		w -= z;
		x -= fv1;
		return (w*_table[v0]+ x*_table[v1]+ y*_table[v2]+ z*_table[v3])* fv1
		+ _table[v1];
	}
	
	AudFloat Buffer::getMorphing(const AudFloat &phi, Buffer* b,
								 const AudFloat &point)
	{
		return point * (readInterp(phi) - b->readInterp(phi)) + b->readInterp(phi);
	}
	
	void Buffer::writeGuardPoint()
	{
		_table[_size] = _table[0];
		_table[_size+1] = _table[1];
		_table[_size+2] = _table[2];
	}
	
	void Buffer::clear()
	{
#if __DEBUG__
		std::cout << "void Buffer::clear()" << std::endl;
#endif								
		for (ul i = 0; i < _size; i++)
			*(_table+i) = 0.0;
		writeGuardPoint ();
	}
	
	void Buffer::copy(const Buffer& r) throw(AudError)
	{
#if __DEBUG__
		std::cout << "void Buffer::copy(const Buffer&)" << std::endl;
#endif								
		if (&r != this)
		{
			if (_size != r._size)
			{
#if __DEBUG__
				std::cout << "Buffer::copy() Resizing.... table!" << std::endl;
#endif
				delete [] _table;
				_size = r._size;
				_table = new AudFloat[_size + _extendGuardPoint];
				if(!_table)
					throw(NULL_POINTER);
			}
			for (ul i = 0; i < _size; i++)
				_table[i] = r._table[i];
			writeGuardPoint ();
		}
	}
	
	void Buffer::copy(const AudFloat& r)
	{
#if __DEBUG__
		std::cout << "void Buffer::copy(const AudFloat&)" << std::endl;
#endif								
		for (ul i = 0; i < _size; i++)
			_table[i] = r;
		writeGuardPoint ();
	}
	
	AudFloat Buffer::getPeak() throw(AudError)
	{
#if __DEBUG__
		std::cout << "AudFloat Buffer::getPeak()" << std::endl;
#endif								
		bool dominio;
		AudFloat peak = 0.f;
		AudFloat maxPos = 0.f;
		AudFloat maxNeg = 0.f;
		
		for (ul i = 0; i < _size; i++)
		{
			peak = _table[i];
			dominio = (peak > 0.0 ? true : false);
			
			if(dominio)
				maxPos = (peak > maxPos ? peak : maxPos);
			else
				maxNeg = (peak < maxNeg ? peak : maxNeg);					
		}
		return (maxPos > fabs(maxNeg) ? maxPos : maxNeg);
	}
	
	void Buffer::normalize()
	{
#if __DEBUG__
		std::cout << "void Buffer::normalize()" << std::endl;
#endif						
		AudFloat peak = fabs(this->getPeak());
		if (!peak) throw AudError(DIVIDE_BY_ZERO);
		AudFloat rescale = 1.0/peak;
		this->rescale(rescale);
	}
	
	void Buffer::rescale(const AudFloat& resc)
	{
#if __DEBUG__
		std::cout << "void Buffer::rescale(const AudFloat&)" << std::endl;
#endif				
		for (ul i = 0; i < _size; i++)
			_table[i] *= resc;
		writeGuardPoint ();
	}
	
	void Buffer::offSet(const AudFloat& offset)
	{
#if __DEBUG__
		std::cout << "void Buffer::offSet(const AudFloat&)" << std::endl;
#endif		
		if (offset != 0)
		{
			for (ul i = 0; i < _size; i++)
				_table[i] += offset;
			writeGuardPoint ();
		}
	}
	
	void Buffer::resize(const ul& newSize) throw(AudError)
	{
#if __DEBUG__
		std::cout << "void Buffer::resize(const ul&)" << std::endl;
#endif
		
#if __DEBUG__
		if (newSize < 10)
		{
			std::cout << "Buffer::resize() Warning! Dimensione minima > 10 punti." << std::endl;
			AudError er(INVALID_LENGTH);
			er.setMsgError("Warning! Dimensione minima > 10 punti.");
			throw er;
		}
#endif
		Buffer temp(_size);
		temp.copy(*this);
		
		AudFloat index = 0.0;
		AudFloat step = 0.0;
		ul lMax = (newSize > _size ? newSize : _size);
		ul lMin = (newSize < _size ? newSize : _size);
		
		if (newSize > _size)
			step = (AudFloat)lMin / (AudFloat)lMax; ///when newSize > _size
		
		else
			step = (AudFloat)lMax / (AudFloat)lMin; ///when newSize < _size
		
		_size = newSize;
		delete [] _table;
		_table = new AudFloat[_size + _extendGuardPoint];
		for (ul i = 0; i < _size; i++)
		{
			index += step;
			_table[i] = temp.readInterp(index/temp.getSize());
		}
		writeGuardPoint ();
	}
	
	bool Buffer::compare(const Buffer& x)
	{
#if __DEBUG__
		std::cout << "bool Buffer::compare(const Buffer&)" << std::endl;
#endif
		
		if(x.getSize() != this->getSize())
			return false;
		for (ul i = 0; i < _size; i++)
			if (_table[i] != x._table[i])
				return false;
		return true;
	}
	
	bool Buffer::operator== (const Buffer& x)
	{
		return compare(x);
	}
	
	Buffer& Buffer::operator= (const Buffer& r)
	{
#if __DEBUG__
		std::cout << "Buffer& operator= (const Buffer&)" << std::endl;
#endif
		this->copy(r);
		return *this;
	}
	
	Buffer& Buffer::operator= (const AudFloat& r)
	{
#if __DEBUG__
		std::cout << "Buffer& operator= (const double&)" << std::endl;
#endif
		this->copy(r);
		return *this;
	}
	
	AudFloat& Buffer::operator[] (const ul &i) throw(AudError)
	{
#if __DEBUG__
		if (i < 0 || i >= (_size + _extendGuardPoint))
		{
			std::cout << "Buffer::operator[] Warning!!! Index Out of range." << std::endl;
			//			return _table[_size];
			AudError er(OUT_OF_RANGE);
			er.setMsgError("Buffer::operator[] Warning!!! Index Out of range.");
			er.setCriticalValue(i);
			throw er;
			
		}
#endif
		return _table[i]; ///reference return
	}
	
	Buffer Buffer::operator+(const Buffer& b)
	{
#if __DEBUG__
		std::cout << "Buffer Buffer::operator+(const Buffer&)" << std::endl;
#endif		
		Buffer result;
		for (ul i = 0; i < _size; i++)
			result[i] = (_table[i] + b._table[i]);
		result.writeGuardPoint ();
		return result;
	}
	
	Buffer Buffer::operator-(const Buffer& b)
	{
#if __DEBUG__
		std::cout << "Buffer Buffer::operator-(const Buffer&)" << std::endl;
#endif
		Buffer result;
		for (ul i = 0; i < _size; i++)
			result[i] = (_table[i] - b._table[i]);
		result.writeGuardPoint ();
		return result;
	}
	
	Buffer Buffer::operator*(const Buffer& b)
	{
#if __DEBUG__
		std::cout << "Buffer Buffer::operator*(const Buffer&)" << std::endl;
#endif
		Buffer result;
		for (ul i = 0; i < _size; i++)
			result[i] = (_table[i] * b._table[i]);
		result.writeGuardPoint ();
		return result;
	}
	
	Buffer Buffer::operator/(const Buffer& b) throw(AudError)
	{
#if __DEBUG__
		std::cout << "Buffer Buffer::operator/(const Buffer&)" << std::endl;
#endif
		AudFloat val;
		Buffer result;
		for (ul i = 0; i < _size; i++)
		{
			val = b._table[i];
			if (!val) throw AudError(DIVIDE_BY_ZERO);
			result[i] = (_table[i] / val);
		}
		result.writeGuardPoint ();
		return result;
	}
	
	/*! Redefining operators with AudFloat parameter */
	Buffer Buffer::operator+(const AudFloat& b)
	{
#if __DEBUG__
		std::cout << "Buffer Buffer::operator+(const AudFloat&)" << std::endl;
#endif
		Buffer result;
		for (ul i = 0; i < _size; i++)
			result[i] = (_table[i] + b);
		result.writeGuardPoint ();
		return result;
	}
	
	Buffer Buffer::operator-(const AudFloat& b)
	{
#if __DEBUG__
		std::cout << "Buffer Buffer::operator-(const AudFloat&)" << std::endl;
#endif
		
		Buffer result;
		for (ul i = 0; i < _size; i++)
			result[i] = (_table[i] - b);
		result.writeGuardPoint ();
		return result;
	}
	
	Buffer Buffer::operator*(const AudFloat& b)
	{
#if __DEBUG__
		std::cout << "Buffer Buffer::operator*(const AudFloat&)" << std::endl;
#endif
		
		Buffer result;
		for (ul i = 0; i < _size; i++)
			result[i] = (_table[i] * b);
		result.writeGuardPoint ();
		return result;
	}
	
	Buffer Buffer::operator/(const AudFloat& b) throw(AudError)
	{
#if __DEBUG__
		std::cout << "Buffer Buffer::operator/(const AudFloat&)" << std::endl;
#endif
		
		if(!b) throw AudError(DIVIDE_BY_ZERO);
		Buffer result;
		for (ul i = 0; i < _size; i++)
			result[i] = (_table[i] / b);
		result.writeGuardPoint ();
		return result;
	}
	
	void Buffer::add(const Buffer& b) throw(AudError)
	{
#if __DEBUG__
		std::cout << "void Buffer::add(const Buffer&)" << std::endl;
#endif		
		
#if __DEBUG__
		if (this->_size != b.getSize() )
		{
			std::cout << "Buffer::add() The lengths of the array are different!" << std::endl;
			AudError er(INVALID_LENGTH);
			er.setMsgError("Buffer::add() The length of the array, is different from the current one!");
			throw er;
		}
#endif
		for (ul i = 0; i < _size; i++)
			_table[i] += b._table[i];
		writeGuardPoint ();
	}
	
	void Buffer::sub(const Buffer& b) throw(AudError)
	{
#if __DEBUG__
		std::cout << "void Buffer::sub(const Buffer&)" << std::endl;
#endif		
		
#if __DEBUG__
		if (this->_size != b.getSize() )
		{
			std::cout << "Buffer::sub() The lengths of the array are different!" << std::endl;
			AudError er(INVALID_LENGTH);
			er.setMsgError("Buffer::sub() The length of the array, is different from the current one!");
			throw er;
		}
#endif
		for (ul i = 0; i < _size; i++)
			_table[i] -= b._table[i];
		writeGuardPoint ();
	}
	
	void Buffer::mul(const Buffer& b) throw(AudError)
	{
#if __DEBUG__
		std::cout << "void Buffer::mul(const Buffer&)" << std::endl;
#endif		
		
#if __DEBUG__
		if (this->_size != b.getSize() )
		{
			std::cout << "Buffer::mul() The lengths of the array are different!" << std::endl;
			AudError er(INVALID_LENGTH);
			er.setMsgError("Buffer::mul() The length of the array, is different from the current one!");
			throw er;
		}
#endif
		for (ul i = 0; i < _size; i++)
			_table[i] *= b._table[i];
		writeGuardPoint ();
	}
	
	void Buffer::div(const Buffer& b) throw(AudError)
	{
#if __DEBUG__
		std::cout << "void Buffer::div(const Buffer&)" << std::endl;
#endif		
		
#if __DEBUG__
		if (this->_size != b.getSize() )
		{
			AudError er(INVALID_LENGTH);
			std::cout << "Buffer::div() The lengths of the array are different!" << std::endl;
			throw er;
		}
#endif
		AudFloat val;
		for (ul i = 0; i < _size; i++)
		{	
			val = b._table[i];
			if(!val)
				throw AudError(DIVIDE_BY_ZERO);
			_table[i] /= val;
		}
		writeGuardPoint ();
	}
	
	void Buffer::add(const AudFloat& b)
	{
#if __DEBUG__
		std::cout << "void Buffer::add(const AudFloat&)" << std::endl;
#endif		
		for (ul i = 0; i < _size; i++)
			_table[i] += b;
		writeGuardPoint ();
	}
	
	void Buffer::sub(const AudFloat& b)
	{
#if __DEBUG__
		std::cout << "void Buffer::sub(const AudFloat&)" << std::endl;
#endif		
		for (ul i = 0; i < _size; i++)
			_table[i] -= b;
		writeGuardPoint ();
	}						
	
	void Buffer::mul(const AudFloat& b)
	{
#if __DEBUG__
		std::cout << "void Buffer::mul(const AudFloat&)" << std::endl;
#endif
		for (ul i = 0; i < _size; i++)
			_table[i] *= b;
		writeGuardPoint ();
	}			
	
	
	void Buffer::div(const AudFloat& b) throw(AudError)
	{
#if __DEBUG__
		std::cout << "void Buffer::div(const AudFloat&)" << std::endl;
#endif
		if(!b)
			throw AudError(DIVIDE_BY_ZERO);
		
		for (ul i = 0; i < _size; i++)
			_table[i] /= b;
		writeGuardPoint ();
	}			
	
	void Buffer::morphing(const Buffer& b, const AudFloat &point) throw(AudError)
	{
#if __DEBUG__
		std::cout << "void Buffer::morphing(const Buffer&, const AudFloat&)" << std::endl;
#endif		
		Buffer *_b = const_cast<Buffer*>(&b);
		
#if __DEBUG__
		if (this->_size != b.getSize() )
		{
			std::cout << "Buffer::morphing() The lengths of the array are different!" << std::endl;
			AudError er(INVALID_LENGTH);
			
			er.setMsgError(" The length of the array, is different from the current one!");
			//			throw er;
		}
#endif
		for (ul i = 0; i < b.getSize(); i++)
			_table[i] = point * (_table[i] - *(_b->getPtr() + i))
			+ *(_b->getPtr() + i);
		writeGuardPoint ();
	}
} //	end of namespace Aud
///EOF
