/***************************************************/
/*! \class DecodeDTMF
 \brief
 
 The AudioDrome DecodeDTMF class, implements 
 a Dual Tone Multi Frequency decoder.
 http://zone.ni.com/cms/images/devzone/tut/a/75f1be30468.gif
 
 The pitch detector is based on the DFT (Discrete Fourier Trasform), 
 coefficients of the DFT are derived using the algorithm Goertzel.
 
 coef = 2.0 * cos( (2.0 * PI * k) / (float)_goertzel_n)) ;
 Where k = (int) (0.5 + ((float)_goertzel_n * target_freq) / SAMPLING_RATE));
 More simply: coef = 2.0 * cos( (2.0 * PI * target_freq) / SAMPLING_RATE );
 
 by Alessandro Petrolati 2008.
 www.alessandro-petrolati.com
 info@alessandro-petrolati.it
 
 This library is free software; you can redistribute it and/or
 modify it under the terms of the GNU Library General Public
 License as published by the Free Software Foundation; either
 version 2 of the License, or (at your option) any later version.
 
 This library is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 Library General Public License for more details.
 
 You should have received a copy of the GNU Library General Public
 License along with this library; if not, write to the Free
 Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
/***************************************************/

#include "DecodeDTMF.h"

using namespace Aud; 

//  begin namespace
namespace Aud
{	
	DecodeDTMF::DecodeDTMF(const ul& frame) : _sample_count(0), _lastDigit(' ')
	{
#if __DEBUG__
		std::cout << "DecodeDTMF Constructor: " << this <<std::endl;
		std::cout << DataType << std::endl;
#endif		
		this->setFrameLength(frame);
		_max_bins =  8;
		_q1 = (double*) new  AudFloat[ _max_bins ];
		_q2 = (double*) new AudFloat[ _max_bins ];
		_r = (double*) new AudFloat[ _max_bins ];
		_coefs = (double*) new AudFloat[ _max_bins ];
		_freqs = (double*) new AudFloat[ _max_bins];
		_freqs[0] = 697.0;
		_freqs[1] = 770.0;
		_freqs[2] = 852.0;
		_freqs[3] = 941.0;
		_freqs[4] = 1209.0;
		_freqs[5] = 1336.0;
		_freqs[6] = 1477.0;
		_freqs[7] = 1633.0;
		
		
		_row_col_ascii_codes = new char**[4];
		for (int i = 0; i < 4; i++)
			_row_col_ascii_codes[i] = new char*[4];
		
		//char*  (*_row_col_ascii_codes)[4] = new char*[4][4];
		
		_row_col_ascii_codes[0][0] = "1";
		_row_col_ascii_codes[0][1] = "2";
		_row_col_ascii_codes[0][2] = "3";
		_row_col_ascii_codes[0][3] = "A";
		
		_row_col_ascii_codes[1][0] = "4";
		_row_col_ascii_codes[1][1] = "5";
		_row_col_ascii_codes[1][2] = "6";
		_row_col_ascii_codes[1][3] = "B";
		
		_row_col_ascii_codes[2][0] = "7";
		_row_col_ascii_codes[2][1] = "8";
		_row_col_ascii_codes[2][2] = "9";
		_row_col_ascii_codes[2][3] = "C";
		
		_row_col_ascii_codes[3][0] = "*";
		_row_col_ascii_codes[3][1] = "0";
		_row_col_ascii_codes[3][2] = "#";
		_row_col_ascii_codes[3][3] = "D";
		
		/* Namely:
		 char* _row_col_ascii_codes[4][4] = {
		 {"1", "2", "3", "A"},
		 {"4", "5", "6", "B"},
		 {"7", "8", "9", "C"},
		 {"*", "0", "#", "D"}};
		 */
		
		this->calcCoeffs();
	}
	
	DecodeDTMF::~DecodeDTMF()
	{
#if __DEBUG__
		std::cout << "DecodeDTMF Destructor: " << this <<std::endl;
		std::cout << DataType << std::endl;
#endif
		delete[] _q1;
		delete[] _q2;
		delete[] _r;
		delete[] _freqs;
		delete[] _coefs;
		for (int i = 0; i < 4; i++)
			delete[] _row_col_ascii_codes[i];
		delete[] _row_col_ascii_codes;
		
	}
	
	void DecodeDTMF::setFrameLength(const ul& frame)
	{
		_goertzel_n = frame; 
	} 
	
	void DecodeDTMF::Processing()
	{
		_see_digit = false;
		_sample_count++;
		for ( _i=0; _i<_max_bins; _i++ )
		{
			_q0 = _coefs[_i] * _q1[_i] - _q2[_i] + (_input_1 * 32768.0);
			_q2[_i] = _q1[_i];
			_q1[_i] = _q0;
		}
		
		if (_sample_count >= _goertzel_n)
		{
			for ( _i=0; _i<_max_bins; _i++ )
			{
				_r[_i] = (_q1[_i] * _q1[_i]) + (_q2[_i] * _q2[_i]) - (_coefs[_i] * _q1[_i] * _q2[_i]);
				_q1[_i] = 0.0;
				_q2[_i] = 0.0;
			}
			_sample_count = 0;
			goertzel();
		}
	}
	
	char DecodeDTMF::getDigit() 
	{
		return _lastDigit;
	}
	
	void DecodeDTMF::printDigit() 
	{ 
		if(_see_digit)
			std::cout << _lastDigit << " ";
	}
	
	
	void DecodeDTMF::goertzel()
	{
		int row, col, peak_count, max_index, i;
		double maxval, t;
		_lastDigit = ' ';
		
		/* Find the largest in the row group. */
		row = 0;
		maxval = 0.0;
		for ( i=0; i<4; i++ )
		{
			if ( _r[i] > maxval )
			{
				maxval = _r[i];
				row = i;
			}
		}
		
		/* Find the largest in the column group. */
		col = 4;
		maxval = 0.0;
		for ( i=4; i<8; i++ )
		{
			if ( _r[i] > maxval )
			{
				maxval = _r[i];
				col = i;
			}
		}
		
		/* Check for minimum energy */
		
		if ( _r[row] < 4.0e5 )   /* 2.0e5 ... 1.0e8 no change */
		{
			/* energy not high enough */
		}
		else if ( _r[col] < 4.0e5 )
		{
			/* energy not high enough */
		}
		else
		{
			_see_digit = true;
			
			/* Twist check
			 * CEPT => twist < 6dB
			 * AT&T => forward twist < 4dB and reverse twist < 8dB
			 *  -ndB < 10 log10( v1 / v2 ), where v1 < v2
			 *  -4dB < 10 log10( v1 / v2 )
			 *  -0.4  < log10( v1 / v2 )
			 *  0.398 < v1 / v2
			 *  0.398 * v2 < v1
			 */
			if ( _r[col] > _r[row] )
			{
				/* Normal twist */
				max_index = col;
				if ( _r[row] < (_r[col] * 0.398) )    /* twist > 4dB, error */
					_see_digit = false;
			}
			else /* if ( _r[row] > _r[col] ) */
			{
				/* Reverse twist */
				max_index = row;
				if ( _r[col] < (_r[row] * 0.158) )    /* twist > 8db, error */
					_see_digit = false;
			}
			
			/* Signal to noise test
			 * AT&T states that the noise must be 16dB down from the signal.
			 * Here we count the number of signals above the threshold and
			 * there ought to be only two.
			 */
			if ( _r[max_index] > 1.0e9 )
				t = _r[max_index] * 0.158;
			else
				t = _r[max_index] * 0.010;
			
			peak_count = 0;
			for ( i=0; i<8; i++ )
			{
				if ( _r[i] > t )
					peak_count++;
			}
			if ( peak_count > 2 )
				_see_digit = false;
			
			if ( _see_digit )
			{
				_lastDigit = *(_row_col_ascii_codes[row][col-4]);
			}
		}
	}
	
	void DecodeDTMF::calcCoeffs() /*! Sampling Rate Change SLOT */
	{
		for(int n = 0; n < _max_bins; n++)
			_coefs[n] = 2.0 * cos(2.0 * M_PI * _freqs[n] / _localSR);
	}
	
} //	end of namespace Aud
///EOF

