/***************************************************/
/*! \class DelayLine
 \brief
 
 The AudioDrome DelayLine, implements a simple fast
 non interpolated delay line.
 
 Simple delay line:
 * ===================================================
 *  input -------[..... d.....] --------- Processing()
 * ===================================================
 
 by Alessandro Petrolati 2006.
 www.alessandro-petrolati.it
 info@alessandro-petrolati.it
 
 This library is free software; you can redistribute it and/or
 modify it under the terms of the GNU Library General Public
 License as published by the Free Software Foundation; either
 version 2 of the License, or (at your option) any later version.
 
 This library is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 Library General Public License for more details.
 
 You should have received a copy of the GNU Library General Public
 License along with this library; if not, write to the Free
 Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
/***************************************************/

#include "DelayLine.h"

using namespace Aud; 

//  begin namespace
namespace Aud
{
	DelayLine::DelayLine(const AudFloat &delayMax)
	{
#if __DEBUG__
		std::cout << "DelayTaps Constructor: "<< this <<std::endl;
#endif
		_buffer = new Buffer(delayMax * _localSR);
		_maxDelayTime = delayMax;
		_write = 1L;
		_read = 0L;
	}
	
	DelayLine::~DelayLine()
	{
#if __DEBUG__
		std::cout << "DelayLine Destructor: "<< this <<std::endl;
#endif
		delete _buffer;
	}
	
	void DelayLine::setMaxDelayTime(const AudFloat &delayMax) /*! Sampling Rate Change SLOT */
	{
		_maxDelayTime = delayMax;
		_buffer->resize(delayMax * _localSR);
	}
	
	void DelayLine::setDelayTime(const AudFloat &delay) /*! Sampling Rate Change SLOT */
	{
		_delayTime = fabs(delay);
		_delaySample = _delayTime * _localSR;
		
		if (_delaySample > _buffer->getSize())
		{
#if __DEBUG__
			std::cout
			<< "Warning! the delay Time exceeds regarding the maximum delay time;"
			<< std::endl;
			std::cout << "the delay will be set up like of the maximum delay time!"
			<< std::endl;
#endif
			_delaySample = _buffer->getSize() - 1;
			_delayTime = (AudFloat) _delaySample / _localSR;
		}
		
		if (_write >= _delaySample)
			_read = _write - _delaySample;
		else
			_read = _buffer->getSize() + _write - _delaySample;
	}
	
	void DelayLine::setDelaySample(const ul &sample) /*! Sampling Rate Change SLOT */
	{
		this->setDelayTime( (AudFloat) sample / _localSR );
	}
	
	AudFloat DelayLine::getDelayTime() const
	{
		return _delayTime;
	}
	
	AudFloat DelayLine::getDelaySample() const
	{
		return _delaySample;
	}
	
	AudFloat DelayLine::getMaxDelayTime() const
	{
		return _maxDelayTime;
	}
	
	void DelayLine::clearBuffer()
	{
		_buffer->clear();
	}
	
	void DelayLine::Processing()
	{
		*(_buffer->getPtr()+_write++) = _input_1;
		
		if (_write == _buffer->getSize())
			_write = 0L;
		
		if (_read ==  _buffer->getSize())
			_read = 0L;
		
		_output_1 = *(_buffer->getPtr()+_read++);
	}
	
} //	end of namespace Aud
///EOF
