/***************************************************/
/*! \class DelayTaps
 \brief
 
 The AudioDrome DelayTaps, implements a 
 interpolated delay line with multitaps outs.
 
 Simple delay line:
 * ===================================================
 *  input -------[..... d.....] --------- Processing()
 * ===================================================
 
 Multitaps delay line:
 * ===================================================
 *                 _____________________ getSignalTap()
 *                |      _______________ getSignalTap()
 * 				  |     |   ____________ getSignalTap()
 * 		          |     |  |
 *  input -------[..... d.....] --------- Processing()
 * ===================================================
 
 by Alessandro Petrolati 2006.
 www.alessandro-petrolati.it
 info@alessandro-petrolati.it
 
 This library is free software; you can redistribute it and/or
 modify it under the terms of the GNU Library General Public
 License as published by the Free Software Foundation; either
 version 2 of the License, or (at your option) any later version.
 
 This library is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 Library General Public License for more details.
 
 You should have received a copy of the GNU Library General Public
 License along with this library; if not, write to the Free
 Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
/***************************************************/

#include "DelayTaps.h"

using namespace Aud; 

//  begin namespace
namespace Aud
{
	DelayTaps::DelayTaps(const AudFloat &delayMax, const INTERPOLATION& type) : DelayLine(delayMax)
	{
#if __DEBUG__
		std::cout << "DelayTaps Constructor: "<< this <<std::endl;
#endif
		
		_interpType = type;
		this->setInterpolation(_interpType);
	}
	
	DelayTaps::~DelayTaps()
	{
#if __DEBUG__
		std::cout << "DelayTaps Destructor: "<< this <<std::endl;
#endif
	}
	
	void DelayTaps::setInterpolation(const INTERPOLATION &type)
	{
		switch (type)
		{
			case (DISABLED):
			{
				_interpPtr = &Buffer::readRaw;
			}
				break;
			case (LINEAR):
			{
				_interpPtr = &Buffer::readInterp;
			}
				break;
			case (CUBIC):
			{
				_interpPtr = &Buffer::readInterp3;
			}
				break;
			case (VARGA):
			{
				_interpPtr = &Buffer::readInterp3Varga;
			}
				break;
			default:
			{
				_interpPtr = &Buffer::readInterp;
			}
		}
	}
	
	void DelayTaps::Processing()
	{
		/* WRITE */
		*(_buffer->getPtr()+_write++) = _input_1;
		
		if (_write == _buffer->getSize())
			_write = 0L;
		
		if (_write < 3L)
			_buffer->writeGuardPoint();
		
		/* READ */
		_output_1 = this->OutputTap(_delayTime);
	}
	
	AudFloat DelayTaps::OutputTap(const AudFloat &delTapTime) /*Non observed because change every sample!!*/
	{
		AudFloat _readPhase = AudFloat(_write) - fabs(delTapTime) * _localSR; 
		
		if (_readPhase < 0.0)
			_readPhase += AudFloat(_buffer->getSize());
		return (_buffer->*_interpPtr)(_readPhase / AudFloat(_buffer->getSize()));
	}
	
} //	end of namespace Aud
///EOF