/***************************************************/
/*! \class Distortion
 \brief
 
 The AudioDrome Distortion class, implements a 
 DNL (Distortion Not Linear) algorithm.
 Distort an audio signal via waveshaping and optional clipping
 
 The waveshaping is implemented by an object AudioDrome::Buffer.
 
 The distortion opcode uses modified hyperbolic tangent distortion.
 Coded by Hans Mikelson November 1998 -> biquad.c e compress.c
 
 by Alessandro Petrolati 2006.
 www.alessandro-petrolati.it
 info@alessandro-petrolati.it
 
 This library is free software; you can redistribute it and/or
 modify it under the terms of the GNU Library General Public
 License as published by the Free Software Foundation; either
 version 2 of the License, or (at your option) any later version.
 
 This library is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 Library General Public License for more details.
 
 You should have received a copy of the GNU Library General Public
 License along with this library; if not, write to the Free
 Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
/***************************************************/

#include "Distortion.h"
using namespace Aud; 

namespace Aud
{
	Distortion::Distortion(Buffer* shape) :
	_flag(false)
	{
#if __DEBUG__
		std::cout << "Distrortion Constructor: " << this <<std::endl;
#endif
		if (shape == NULL)
		{
			_flag = true;
			_shape = new Buffer();
			Wave* tmp = new Wave();
			tmp->genSigmoid();
			_shape->copy(*tmp);
			delete tmp;
		}
		else
			_shape = shape;
		
		unsigned long size = _shape->getSize();
		
		AudFloat val = _shape->operator[](size - 1);
		_shape->operator[](size) = val;
		_shape->operator[](size + 1) = val;
		_shape->operator[](size + 2) = val;
		
		this->setAmountDistort(0);
		_hp = 10;
		_stor = 0;
		_min_rms = 1;
		
		this->calcCoefs();
		
		if (!_stor)
		{
			_prvq = 0.0;
			_prvd = 1000.0 * _min_rms;
		}
	}
	
	Distortion::~Distortion()
	{
#if __DEBUG__
		std::cout << "Distrortion Destructor: " << this <<std::endl;
#endif
		if (_flag)
			delete _shape;
	}
	
	void Distortion::setAmountDistort(const AudFloat &dist)
	{
		_dist = dist;
	}
	
	void Distortion::setShape(Buffer* shape)
	{
		if (_flag)
			delete _shape;
		_shape = shape;
		_flag = false;
	}
	
	void Distortion::Processing()
	{
		q = _prvq;
		q = _c1 * _input_1 * _input_1 + _c2 * q;
		_prvq = q;
		
		/* get running rms */
		rms = (AudFloat) sqrt((double) q);
		if (rms < _min_rms)
			rms = _min_rms;
		if ((dist = _dist) < 0.001)
			dist = 0.001;
		
		/* & compress factor */
		dnew = rms / dist;
		dcur = _prvd;
		dinc = (dnew - dcur);
		
		/* compress the sample */
		sig = _input_1 / dcur;
		
		/* as index into table */
		phs = 0.5 * (1.0 + sig);
		if (phs <= 0.0)
			val = _shape->readInterp(0.0);
		
		/* check sticky bits */
		else if (phs >= 1.0)
			val = _shape->readInterp(1.0);
		else
			val = _shape->readInterp(phs);
		
		/* and restor the amp   */
		_output_1 = val * dcur; 		/* output sample distort   */
		dcur += dinc;
		_prvd = dcur;
	}
} //	end of namespace Aud

///EOF
