/***************************************************/
/*! \class EUVerb
 \brief
 
 The AudioDrome EUVerb class, implements 
 a reverb according to the model of 
 Eugenio Giordani.
 www.eugenio-giordani.it
 L.E.M.S.
 Pesaro 
 
 by Alessandro Petrolati 2008.
 www.alessandro-petrolati.it
 info@alessandro-petrolati.it
 
 This library is free software; you can redistribute it and/or
 modify it under the terms of the GNU Library General Public
 License as published by the Free Software Foundation; either
 version 2 of the License, or (at your option) any later version.
 
 This library is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 Library General Public License for more details.
 
 You should have received a copy of the GNU Library General Public
 License along with this library; if not, write to the Free
 Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
/***************************************************/

#include "EUVerb.h"

using namespace Aud;

namespace Aud
{
	
	EUVerb::EUVerb(const INTERPOLATION& type)
	{
#if __DEBUG__
		std::cout << "EUVerb Constructor: "<< this<<std::endl;
#endif
		MILLISECONDS = 0.001;
		
		/* PREDEFINED DEFAULTS */
		
		//Equalizer values
		_loshEqFreq = 20;
		_loshEqGain = 0;
		_hishEqFreq = 1000;
		_hishEqGain = 0;
		_peakEqFreq = 20;
		_peakEqGain = 0;
		
		//Early Reflection values
		_erTapL1 = 0.007;
		_erTapL2 = 0.027;				
		_erTapL3 = 0.033;
		_erTapL4 = 0.055;				
		_erTapL5 = 0.059;
		_erTapL6 = 0.061;
		
		//Early reflection second tap values
		_erTapR1 = 0.008;
		_erTapR2 = 0.039;
		_erTapR3 = 0.051;
		_erTapR4 = 0.063;
		_erTapR5 = 0.065;
		_erTapR6 = 0.071;
		
		//Er first sent gain
		_egL1 = 0.5;
		_egL2 = 0.8;
		_egL3 = 0.9;
		_egL4 = 0.7;
		_egL5 = 0.5;
		_egL6 = 0.3;
		
		//Er second sent gain
		_egR1 = 0.5;
		_egR2 = 0.8;
		_egR3 = 0.9;
		_egR4 = 0.7;
		_egR5 = 0.5;
		_egR6 = 0.3;
		
		//AllPass delay times
		_delAllPass1 = 17.65;
		_delAllPass2 = 13.91;
		_delAllPass3 = 10.97;
		_delAllPass4 = 8.61;
		_delAllPass5 = 6.83;
		_delAllPass6 = 5.31;
		
		//AllPass gains
		_gainApScale1 = 1;
		_gainApScale2 = 1;
		_gainApScale3 = 1;
		_gainApScale4 = 1;
		_gainApScale5 = 1;
		_gainApScale6 = 1;
		
		//Combs delay times
		_delComb1 = 54.5501;
		_delComb2 = 58.3473;
		_delComb3 = 62.4089;
		_delComb4 = 66.7532;
		_delComb5 = 71.4001;
		_delComb6 = 77.3317;
		
		//Random values from the seconds tap of comb's
		_delCombTap1 = _delComb1 * (1.0 - 0.213);
		_delCombTap2 = _delComb2 * (1.0 - 0.317);
		_delCombTap3 = _delComb3 * (1.0 - 0.271);
		_delCombTap4 = _delComb4 * (1.0 - 0.177);
		_delCombTap5 = _delComb5 * (1.0 - 0.263);
		_delCombTap6 = _delComb6 * (1.0 - 0.299);
		
		//Send signals
		_masterLevel = 0.7;
		_dirLevel = 0.81;
		_earlyLevel = 0.0;
		_revLevel = 0.15;
		_gainAllPass = 0.5;
		
		/* DSP INITIALIZE */
		_eq1.setMode(LOWSHELVING);
		_eq2.setMode(HIGHSHELVING);
		_eq3.setMode(PEAKING);
		_preDelay.setInterpolation(DISABLED);
		_directDelay.setInterpolation(DISABLED);
		_multitap.setInterpolation(DISABLED);
		
		_maxdelayTime = 0.33;
		_preDelay.setMaxDelayTime(_maxdelayTime);
		_directDelay.setMaxDelayTime(_maxdelayTime);
		_multitap.setMaxDelayTime(_maxdelayTime);
		_cmb1.setMaxDelayTime(_maxdelayTime);
		_cmb2.setMaxDelayTime(_maxdelayTime);
		_cmb3.setMaxDelayTime(_maxdelayTime);
		_cmb4.setMaxDelayTime(_maxdelayTime);
		_cmb5.setMaxDelayTime(_maxdelayTime);
		_cmb6.setMaxDelayTime(_maxdelayTime);		
		_ap1.setMaxDelayTime(_maxdelayTime);		
		_ap2.setMaxDelayTime(_maxdelayTime);		
		_ap3.setMaxDelayTime(_maxdelayTime);		
		_ap4.setMaxDelayTime(_maxdelayTime);		
		_ap5.setMaxDelayTime(_maxdelayTime);		
		_ap6.setMaxDelayTime(_maxdelayTime);		
		
		this->setInperpolation(type);
		this->setNoiseInject(0.00000000001);
		
		this->setMasterLevel(_masterLevel);	
		this->setDirectLevel(_dirLevel);
		this->setReverberatedLevel(_revLevel);
		this->setEarlyReflectionLevel(_earlyLevel);
		
		this->setEarlyTimeScale(1.0); //da 0.35 a 4.0
		
		this->setLowShelfEqParam(_loshEqFreq, _loshEqGain);
		this->setHighShelfEqParam(_hishEqFreq, _hishEqGain);
		this->setPeakEqParam(_peakEqFreq,  _peakEqGain);
		
		this->setCombCutOff(12000);
		
		this->setCombTimeScale(1.5);
		this->setComb1Time(_delComb1);
		this->setComb2Time(_delComb2);
		this->setComb3Time(_delComb3);
		this->setComb4Time(_delComb4);
		this->setComb5Time(_delComb5);
		this->setComb6Time(_delComb6);
		
		this->setAllPassTimeScale(1.5); //from 0.2 to 4
		
		this->setAllPassGain(_gainAllPass);
		this->setAllPass1GainScale(_gainApScale1);
		this->setAllPass2GainScale(_gainApScale2);
		this->setAllPass3GainScale(_gainApScale3);
		this->setAllPass4GainScale(_gainApScale4);
		this->setAllPass5GainScale(_gainApScale5);
		this->setAllPass6GainScale(_gainApScale6);
		this->setPreDelay(0.00001);
		this->setDirectDelay(0.00001);
		this->setT60(1.2);
	}
	
	EUVerb::~EUVerb()
	{
#if __DEBUG__
		std::cout << "EUVerb Destructor: "<< this<<std::endl;
#endif
	}
	
	
	void EUVerb::setInperpolation(const INTERPOLATION& type)
	{
		//_multitap.setInterp(type);
		//_preDelay.setInterp(type);
		//_directDelay.setInterp(type);
		_ap1.setInterpolation(type);
		_ap2.setInterpolation(type);
		_ap3.setInterpolation(type);
		_ap4.setInterpolation(type);
		_ap5.setInterpolation(type);
		_ap6.setInterpolation(type);
		_cmb1.setInterpolation(type);
		_cmb2.setInterpolation(type);
		_cmb3.setInterpolation(type);
		_cmb4.setInterpolation(type);
		_cmb5.setInterpolation(type);
		_cmb6.setInterpolation(type);
	}
	
	void EUVerb::setNoiseInject(const AudFloat& amount)
	{
		_sputo.setAmplitude(amount);
	}
	
	void EUVerb::setMasterLevel(const AudFloat& level)
	{
		_masterLevel = level;
	}
	
	void EUVerb::setDirectLevel(const AudFloat& dirlevel)
	{
		_dirLevel = dirlevel;
	}
	
	void EUVerb::setReverberatedLevel(const AudFloat& revlevel)
	{
		_revLevel = revlevel;
	}
	
	void EUVerb::setEarlyReflectionLevel(const AudFloat& earlevel)
	{
		_earlyLevel = earlevel;
	}
	
	void EUVerb::setCombTimeScale(const AudFloat& scale)
	{
		_combDelayScale = scale;
		this->setComb1Time(_delComb1);
		this->setComb2Time(_delComb2);
		this->setComb3Time(_delComb3);
		this->setComb4Time(_delComb4);
		this->setComb5Time(_delComb5);
		this->setComb6Time(_delComb6);
	}
	
	
	void EUVerb::setComb1Time(const AudFloat& time1)
	{
		_delComb1 = time1;
		_cmb1.setDelayTime(_delComb1 * _combDelayScale * MILLISECONDS);		
		_delCombTap1 = _cmb1.getDelayTime() * (1.0 - 0.213);
	}
	
	void EUVerb::setComb2Time(const AudFloat& time2)
	{
		_delComb2 = time2;
		_cmb2.setDelayTime(_delComb2 * _combDelayScale * MILLISECONDS);
		_delCombTap2 = _cmb2.getDelayTime() * (1.0 - 0.317);		
	}
	
	void EUVerb::setComb3Time(const AudFloat& time3)
	{
		_delComb3 = time3;
		_cmb3.setDelayTime(_delComb3 * _combDelayScale * MILLISECONDS);
		_delCombTap3 = _cmb3.getDelayTime() * (1.0 - 0.271);		
	}
	
	void EUVerb::setComb4Time(const AudFloat& time4)
	{
		_delComb4 = time4;
		_cmb4.setDelayTime(_delComb4 * _combDelayScale * MILLISECONDS);
		_delCombTap4 = _cmb4.getDelayTime() * (1.0 - 0.177);
	}
	
	void EUVerb::setComb5Time(const AudFloat& time5)
	{
		_delComb5 = time5;
		_cmb5.setDelayTime(_delComb5 * _combDelayScale * MILLISECONDS);
		_delCombTap5 = _cmb5.getDelayTime() * (1.0 - 0.263);
	}
	
	void EUVerb::setComb6Time(const AudFloat& time6)
	{
		_delComb6 = time6;
		_cmb6.setDelayTime(_delComb6 * _combDelayScale * MILLISECONDS);
		_delCombTap6 = _cmb6.getDelayTime() * (1.0 - 0.299);		
	}
	
	void EUVerb::setT60(const AudFloat& t60)
	{
		_cmb1.setT60(t60);
		_cmb2.setT60(t60);
		_cmb3.setT60(t60);
		_cmb4.setT60(t60);
		_cmb5.setT60(t60);
		_cmb6.setT60(t60);
	}
	
	void EUVerb::setCombCutOff(const AudFloat& cuttOff)
	{				
		_cmb1.setCutOff(cuttOff);
		_cmb2.setCutOff(cuttOff);
		_cmb3.setCutOff(cuttOff);
		_cmb4.setCutOff(cuttOff);
		_cmb5.setCutOff(cuttOff);
		_cmb6.setCutOff(cuttOff);
	}
	
	void EUVerb::setAllPassTimeScale(const AudFloat& scale)
	{
		_ap1.setDelayTime(_delAllPass1 * scale * MILLISECONDS);
		_ap2.setDelayTime(_delAllPass2 * scale * MILLISECONDS);
		_ap3.setDelayTime(_delAllPass3 * scale * MILLISECONDS);
		_ap4.setDelayTime(_delAllPass4 * scale * MILLISECONDS);
		_ap5.setDelayTime(_delAllPass5 * scale * MILLISECONDS);
		_ap6.setDelayTime(_delAllPass6 * scale * MILLISECONDS);
	}
	
	void EUVerb::setAllPassGain(const AudFloat& gain)
	{
		_gainAllPass = gain;
		this->setAllPass1GainScale(_gainApScale1);
		this->setAllPass2GainScale(_gainApScale2);
		this->setAllPass3GainScale(_gainApScale3);
		this->setAllPass4GainScale(_gainApScale4);
		this->setAllPass5GainScale(_gainApScale5);
		this->setAllPass6GainScale(_gainApScale6);
	}
	
	void EUVerb::setAllPass1GainScale(const AudFloat& scale1)
	{
		_gainApScale1 = scale1;
		_ap1.setGain(_gainAllPass * _gainApScale1);
	}
	
	void EUVerb::setAllPass2GainScale(const AudFloat& scale2)
	{
		_gainApScale2 = scale2;
		_ap2.setGain(_gainAllPass * _gainApScale2);
	}
	
	void EUVerb::setAllPass3GainScale(const AudFloat& scale3)
	{
		_gainApScale3 = scale3;
		_ap3.setGain(_gainAllPass * _gainApScale3);
	}
	
	void EUVerb::setAllPass4GainScale(const AudFloat& scale4)
	{
		_gainApScale4 = scale4;
		_ap4.setGain(_gainAllPass * _gainApScale4);
	}
	
	void EUVerb::setAllPass5GainScale(const AudFloat& scale5)
	{
		_gainApScale5 = scale5;
		_ap5.setGain(_gainAllPass * _gainApScale5);
	}
	
	void EUVerb::setAllPass6GainScale(const AudFloat& scale6)
	{
		_gainApScale6 = scale6;
		_ap6.setGain(_gainAllPass * _gainApScale6);
	}
	
	void EUVerb::setLowShelfEqParam(const AudFloat& fc, const AudFloat& ab, const AudFloat& q)
	{
		_eq1.setFc(fc);
		_eq1.setAmount( AudioDrome::rawDb(ab) );
		_eq1.setQuality(q);
	}
	
	void EUVerb::setHighShelfEqParam(const AudFloat& fc, const AudFloat& ab, const AudFloat& q)
	{
		_eq2.setFc(fc);
		_eq2.setAmount( AudioDrome::rawDb(ab) );
		_eq2.setQuality(q);
	}
	
	void EUVerb::setPeakEqParam(const AudFloat& fc, const AudFloat& ab, const AudFloat& q)
	{
		_eq3.setFc(fc);
		_eq3.setAmount( AudioDrome::rawDb(ab) );
		_eq3.setQuality(q);
	}
	
	void EUVerb::setEarlyTimeScale(const AudFloat& scale)
	{
		//Valori per l'Early Reflection
		_erTapL1 = 0.007 * scale;
		_erTapL2 = 0.027 * scale;				
		_erTapL3 = 0.033 * scale;
		_erTapL4 = 0.055 * scale;				
		_erTapL5 = 0.059 * scale;
		_erTapL6 = 0.061 * scale;
		_erTapR1 = 0.008 * scale;
		_erTapR2 = 0.039 * scale;
		_erTapR3 = 0.051 * scale;
		_erTapR4 = 0.063 * scale;
		_erTapR5 = 0.065 * scale;
		_erTapR6 = 0.071 * scale;
	}
	
	void EUVerb::setPreDelay(const AudFloat& time)
	{
		_preDelay.setDelayTime(time * MILLISECONDS);
	}
	
	void EUVerb::Processing()
	{
		//Inject a spit of noise
		_sputo.PROCESSING();
		_input = _input_1 + _sputo.Output();
		
		//Equalizing
		_eq1.Input(_input);
		_eq1.PROCESSING();
		_eq2.Input(_eq1.Output());
		_eq2.PROCESSING();
		
		
		_eq3.Input( _eq2.Output() );
		_eq3.PROCESSING();
		_outEqPeaking = _eq3.Output();
		
		//Early Reflection	Left
		_multitap.Input(_outEqPeaking);
		_multitap.PROCESSING();
		
		//Left Early rescale
		AudFloat erDummyL = 
		_multitap.OutputTap(_erTapL1) * _egL1 
		+ _multitap.OutputTap(_erTapL2) * _egL2 
		+ _multitap.OutputTap(_erTapL3) * _egL3 
		+ _multitap.OutputTap(_erTapL4) * _egL4 
		+ _multitap.OutputTap(_erTapL5) * _egL5;
		
		//AllpAss selection
		_preDelay.Input( _eq2.Output() );
		_preDelay.PROCESSING();
		_ap1.Input( _preDelay.Output() );
		_ap1.PROCESSING();
		_ap2.Input( _ap1.Output() );
		_ap2.PROCESSING();
		_ap3.Input( _ap2.Output() );
		_ap3.PROCESSING();
		_ap4.Input( _ap3.Output() );
		_ap4.PROCESSING();
		_ap5.Input( _ap4.Output() );
		_ap5.PROCESSING();
		_ap6.Input( _ap5.Output() );
		_ap6.PROCESSING();
		
		//Comb selection
		_cmb1.Input( _ap6.Output() );
		_cmb2.Input( _ap6.Output() );
		_cmb3.Input( _ap6.Output() );
		_cmb4.Input( _ap6.Output() );
		_cmb5.Input( _ap6.Output() );
		_cmb6.Input( _ap6.Output() );
		_cmb1.PROCESSING();
		_cmb2.PROCESSING();
		_cmb3.PROCESSING();
		_cmb4.PROCESSING();
		_cmb5.PROCESSING();
		_cmb6.PROCESSING();
		
		AudFloat comb_L = 
		_cmb1.Output() 
		+ _cmb2.Output() 
		+ _cmb3.Output() 
		+ _cmb4.Output() 
		+ _cmb5.Output() 
		+ _cmb6.Output();
		
		_directDelay.Input(_input);
		_directDelay.PROCESSING();
		_input = _directDelay.Output();
		_output_1 = ((_earlyLevel * erDummyL) + (_dirLevel * _input) + (_revLevel * comb_L)) * _masterLevel;
	}
	
	AudFloat EUVerb::Output_2()
	{
		//Early Reflection	Rigth			
		//_multitapsR.writeInput(_outEqPeaking);
		
		//Rigth Early rescale
		//AudFloat erDummyR = _multitapsR.outTap(_erTapR1) * _egR1 
		//		+ _multitapsR.outTap(_erTapR2) * _egR2 + _multitapsR.outTap(_erTapR3) * _egR3 
		//		+ _multitapsR.outTap(_erTapR4) * _egR4 + _multitapsR.outTap(_erTapR5) * _egR5;
		AudFloat erDummyR = _multitap.OutputTap(_erTapR1) * _egR1 
		+ _multitap.OutputTap(_erTapR2) * _egR2 + _multitap.OutputTap(_erTapR3) * _egR3 
		+ _multitap.OutputTap(_erTapR4) * _egR4 + _multitap.OutputTap(_erTapR5) * _egR5;
		
		AudFloat comb_R = _cmb1.OutputTap(_delCombTap1) + _cmb2.OutputTap(_delCombTap2) 
		+ _cmb3.OutputTap(_delCombTap3) +_cmb4.OutputTap(_delCombTap4) 
		+ _cmb5.OutputTap(_delCombTap5) +_cmb6.OutputTap(_delCombTap6);
		
		return ((_earlyLevel * erDummyR) + (_dirLevel * _input) + (_revLevel * comb_R)) * _masterLevel;
	}
} //	end of namespace Aud
///EOF

