/***************************************************/
/*! \class Envelops
 \brief
 
 The AudioDrome Envelops class, implements 
 a envelope generator, based on the control 
 of the attack and decay.
 The values of attack and decay,
 are expressed as a percentage of total duration.
 therefore, the "sustain" value, is implicit:
 
 _sus = _dur - (_attack + _decay);
 
 by Alessandro Petrolati 2006.
 www.alessandro-petrolati.it
 info@alessandro-petrolati.it
 
 This library is free software; you can redistribute it and/or
 modify it under the terms of the GNU Library General Public
 License as published by the Free Software Foundation; either
 version 2 of the License, or (at your option) any later version.
 
 This library is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 Library General Public License for more details.
 
 You should have received a copy of the GNU Library General Public
 License along with this library; if not, write to the Free
 Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
/***************************************************/

#include "Envelops.h"

using namespace Aud; 

//  begin namespace
namespace Aud
{
	Envelops::Envelops(Buffer* shape, const INTERPOLATION& type) : _flag(false)
	{
#if __DEBUG__
		std::cout << "Envelops Constructor: " << this <<std::endl;
#endif
		if (shape == NULL)
		{
			_flag = true;
			_shape = new Buffer();
			Window* tmp = new Window();
			tmp->genGaussian();
			_shape->copy(*tmp);
			delete tmp;
		}
		else
			_shape = shape;
		
		this->setInterpolation(type);
		this->setDuration(0.5);
		this->setAttack(0.5);
		this->setDecay(0.5);
	}
	
	Envelops::~Envelops()
	{
#if __DEBUG__
		std::cout << "Envelops Destructor: " << this <<std::endl;
#endif
		if (_flag)
			delete _shape;
	}
	
	void Envelops::setShape(Buffer* shape)
	{
		if (_flag)
			delete _shape;
		_shape = shape;
		_flag = false;
	}
	
	void Envelops::setInterpolation(const INTERPOLATION &type)
	{
		switch (type)
		{
			case (DISABLED):
			{
				_interpPtr = &Buffer::readRaw;
			}
				break;
			case (LINEAR):
			{
				_interpPtr = &Buffer::readInterp;
			}
				break;
			case (CUBIC):
			{
				_interpPtr = &Buffer::readInterp3;
			}
				break;
			case (VARGA):
			{
				_interpPtr = &Buffer::readInterp3Varga;
			}
				break;
			default:
			{
				_interpPtr = &Buffer::readInterp;
			}
		}
	}
	
	void Envelops::Processing()
	{
		if (_trig)
		{
			Phasor::Processing();
			if(Phasor::_trigger)
			{
				_trig = false;
				_output_1 = 0.0;
				return;
			}
			if (_phi < _attack)
				_output_1 = (_shape->*_interpPtr)(_phi/_attack * 0.5);
			else if (_phi < _attack + _sus)
				_output_1 = (_shape->*_interpPtr)(0.5);
			else
				_output_1 = (_shape->*_interpPtr)(1.0 - (1.0 - _phi) / _decay * 0.5);
		}
		else
			_output_1 = (_shape->*_interpPtr)(1.0);
		
	}
} //	end of namespace Aud
///EOF

