/***************************************************/
/*! \class Flanger
 \brief
 
 The AudioDrome Flanger class, implements a flanger effect
 based on the delay line interpolated.
 This unit is useful for generating choruses and flangers. 
 
 Lehman, Scott (1996). Effects Explained. Harmony Central. Retrieved 6/04 from
 http://www.harmony-central.com/Effects/effects-explained.html
 
 Mikelson, Hans (2000). Modeling a multieffects processor in Csound. 
 In Boulanger, Richard (2000), The Csound book (pp 575-594). Cambridge, MA: MIT Press.
 Schindler, Allan. (1998). Eastman Csound tutorial.  Eastman School of Music. Retrieved 6/04 from
 http://www.esm.rochester.edu/onlinedocs/allan.cs/
 Vercoe, Barry. (1992). The public Csound reference manual, version 4.16. MIT Press.  
 Retrieved 6/04 from http://www.lakewoodsound.com/csound/hypertext/manual.htm
 
 by Alessandro Petrolati 2006.
 www.alessandro-petrolati.it
 info@alessandro-petrolati.it
 
 This library is free software; you can redistribute it and/or
 modify it under the terms of the GNU Library General Public
 License as published by the Free Software Foundation; either
 version 2 of the License, or (at your option) any later version.
 
 This library is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 Library General Public License for more details.
 
 You should have received a copy of the GNU Library General Public
 License along with this library; if not, write to the Free
 Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
/***************************************************/

#include "Flanger.h"

using namespace Aud; 

//  begin namespace
namespace Aud
{
	Flanger::Flanger(const AudFloat &delayMax) :
	DelayTaps(delayMax, CUBIC)
	{
#if __DEBUG__
		std::cout << "Flanger Constructor: "<< this <<std::endl;
#endif
		_wave = new Wave(4096);
		_wave->genSine();
		_lfo = new Oscillator(_wave);
		
		this->setDelayTime(0.008);
		this->setDepthTime(0.003);
		this->setFeedGain(0.5);
		this->setLfoRatio(0.5);
		this->setModShape(SINE);
		this->setMixing(0.5);
	}
	
	Flanger::~Flanger()
	{
#if __DEBUG__
		std::cout << "Flanger Destructor: "<< this <<std::endl;
#endif
		delete _lfo;
		delete _wave;
	}
	
	void Flanger::setDelayTime(const AudFloat &delay)
	{
		if (delay > DelayTaps::getMaxDelayTime())
			_delay = DelayTaps::getMaxDelayTime();
		else
			_delay = delay;
	}
	
	void Flanger::setDepthTime(const AudFloat &depth)
	{
		if (depth > _delay)
		{
#if __DEBUG__
			std::cout << "Warning! the Depth delay exceeds regarding the delay time;"
			<< std::endl;
#endif
			_lfo->setAmplitude(_delay);
		}
		else
			_lfo->setAmplitude(depth);
	}
	
	void Flanger::setFeedGain(const AudFloat &feedGain)
	{
		_feedGain = feedGain;
	}
	
	void Flanger::setLfoRatio(const AudFloat &ratio)
	{
		_lfo->setFrequency(ratio);
	}
	
	void Flanger::setModShape(const WAVESHAPE &wave)
	{
		switch (wave)
		{
			case (SINE):
			{
				_wave->genSine();
			}
				break;
			case (SAWTOOTH):
			{
				_wave->genSaw();
			}
				break;
			case (SQUARE):
			{
				_wave->genSquare();
			}
				break;
			case (TRIANGLE):
			{
				_wave->genTriangle();
			}
				break;
			default:
			{
				_wave->genSine();
			}
		}
	}
	
	void Flanger::setMixing(const AudFloat& mixing)
	{
		_flanging = (mixing > .99 ? .99 : mixing);
		_flanging = (mixing < 0.0 ? 0.0 : mixing);
		_direct = 1.0 - _flanging; //percent unreverberated
	}
	
	void Flanger::Processing()
	{
		/*
		 y[n] = x[n] + x[n-m] + g * y[n-m]
		 y[n] = output signal
		 x[n] = input signal
		 sr = sampling rate (Hz)
		 maxDelay = maximum delay time (sec)
		 minDelay = minimum delay time (sec)
		 f = oscillator frequency (Hz)
		 t = time (sec)
		 g = feedback level
		 m = delay time in samples
		 m = sr * ([maxDelay-minDelay]*[1/2 sin(ft)+1/2] + minDelay)
		 */
		
		_lfo->PROCESSING();
		DelayTaps::setDelayTime(_delay + _lfo->Output() );
		_input_1 = _input_1 + (_feedGain * _last);
		DelayTaps::Processing();
		_last = _output_1;
		_output_1 = (_direct * _input_1) + (_last * _flanging);
	}
} //	end of namespace Aud
///EOF
