/***************************************************/
/*! \class GenerateDTMF
 \brief
 
 The AudioDrome GenerateDTMF class,
 implements a Dual Tone Multi Frequency generator.
 http://en.wikipedia.org/wiki/DTMF
 
 by Alessandro Petrolati 2008.
 www.alessandro-petrolati.com
 info@alessandro-petrolati.it
 
 This library is free software; you can redistribute it and/or
 modify it under the terms of the GNU Library General Public
 License as published by the Free Software Foundation; either
 version 2 of the License, or (at your option) any later version.
 
 This library is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 Library General Public License for more details.
 
 You should have received a copy of the GNU Library General Public
 License along with this library; if not, write to the Free
 Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
/***************************************************/

#include "GenerateDTMF.h"

using namespace Aud; 

//  begin namespace
namespace Aud
{
	GenerateDTMF::GenerateDTMF(const AudFloat ampDb) : _breakCount(0), _retrig(false), _mode(false)
	{
#if __DEBUG__
		std::cout << "GenerateDTMF Constructor: " << this <<std::endl;
		std::cout << DataType << std::endl;
#endif
		//DTMF
		_freqs = new AudFloat[8];
		_freqs[0] = 697.0;
		_freqs[1] = 770.0;
		_freqs[2] = 852.0;
		_freqs[3] = 941.0;
		_freqs[4] = 1209.0;
		_freqs[5] = 1336.0;
		_freqs[6] = 1477.0;
		_freqs[7] = 1633.0;
		
		//MF (CCITT R1)
		//		delete _freqs;
		//		_freqs = new AudFloat[10];
		//		_freqs[0] = 700.0;
		//		_freqs[1] = 900.0;
		//		_freqs[2] = 1100.0;
		//		_freqs[3] = 1300.0;
		//		_freqs[4] = 1500.0;
		//		_freqs[5] = 900.0;
		//		_freqs[6] = 1100.0;
		//		_freqs[7] = 1300.0;
		//		_freqs[8] = 1500.0;
		//		_freqs[9] = 1700.0;
		
		this->setToneLevel(ampDb);
		this->setNoiseLevel(-120);
		this->setPauseChar(',');
		this->setToneTime(100);
		this->setPauseTime(0);
		this->setToneInterrupt(40);
		
		Sinusoide::setFrequency(0.0);
		_oscColonna.setFrequency(0.0);
		_env.enableWrapAround(true);
	}
	
	GenerateDTMF::~GenerateDTMF()
	{
#if __DEBUG__
		std::cout << "GenerateDTMF Destructor: " << this <<std::endl;
#endif
		delete[] _freqs;
	};
	
	void GenerateDTMF::setToneLevel(const AudFloat& ampDb)
	{
		AudFloat raw = AudioDrome::rawDb(ampDb);
		raw /= 2;				
		Sinusoide::setAmplitude(raw);
		_oscColonna.setAmplitude(raw);
	}
	
	void GenerateDTMF::setNoiseLevel(const AudFloat& ampDb)
	{
		_noise.setAmplitude(AudioDrome::rawDb(ampDb));
	}
	
	void GenerateDTMF::setToneTime(const AudFloat& ms) 
	{ 
		_toneTime = ms;
		this->setEnvelope();
	}
	void GenerateDTMF::setToneInterrupt(const AudFloat& ms) 
	{ 
		_interrupt = ms; 
		this->setEnvelope();
	}
	void GenerateDTMF::setPauseTime(const AudFloat& ms) 
	{
		_pauseTime = ms; 
		this->setEnvelope();		
	}
	
	void GenerateDTMF::setPauseChar(const char& chr)
	{ 
		if( isgraph(chr) )
			_pauseChar = chr; 
	}
	
	void GenerateDTMF::setStringTone(const std::string& s)
	{
		_stringa = s;
	}
	
	void GenerateDTMF::computeToneFreqs(const char& n)
	{
		/*=================================================
		 1      2	    3	    A	    697 Hz
		 4	    5	    6	    B	    770 Hz
		 7	    8	    9	    C	    852 Hz
		 *	    0	    #	    D	    941 Hz
		 1209 Hz1336 Hz	1477 Hz	1633 Hz
		 ==================================================*/			
		switch (n) 
		{
			case ('1'):
				Sinusoide::setFrequency(_freqs[0]);
				_oscColonna.setFrequency(_freqs[4]);
				break;
			case ('2'):
				Sinusoide::setFrequency(_freqs[0]);
				_oscColonna.setFrequency(_freqs[5]);
				break;
			case ('3'):
				Sinusoide::setFrequency(_freqs[0]);
				_oscColonna.setFrequency(_freqs[6]);
				break;
			case ('A'):
				Sinusoide::setFrequency(_freqs[0]);
				_oscColonna.setFrequency(_freqs[7]);
				break;
			case ('4'):
				Sinusoide::setFrequency(_freqs[1]);
				_oscColonna.setFrequency(_freqs[4]);
				break;
			case ('5'):
				Sinusoide::setFrequency(_freqs[1]);
				_oscColonna.setFrequency(_freqs[5]);
				break;
			case ('6'):
				Sinusoide::setFrequency(_freqs[1]);
				_oscColonna.setFrequency(_freqs[6]);
				break;
			case ('B'):
				Sinusoide::setFrequency(_freqs[1]);
				_oscColonna.setFrequency(_freqs[7]);
				break;							
			case ('7'):
				Sinusoide::setFrequency(_freqs[2]);
				_oscColonna.setFrequency(_freqs[4]);
				break;
			case ('8'):
				Sinusoide::setFrequency(_freqs[2]);
				_oscColonna.setFrequency(_freqs[5]);
				break;
			case ('9'):
				Sinusoide::setFrequency(_freqs[2]);
				_oscColonna.setFrequency(_freqs[6]);
				break;
			case ('C'):
				Sinusoide::setFrequency(_freqs[2]);
				_oscColonna.setFrequency(_freqs[7]);
				break;					
			case ('*'):
				Sinusoide::setFrequency(_freqs[3]);
				_oscColonna.setFrequency(_freqs[4]);
				break;
			case ('0'):
				Sinusoide::setFrequency(_freqs[3]);
				_oscColonna.setFrequency(_freqs[5]);
				break;
			case ('#'):
				Sinusoide::setFrequency(_freqs[3]);
				_oscColonna.setFrequency(_freqs[6]);
				break;
			case ('D'):
				Sinusoide::setFrequency(_freqs[3]);
				_oscColonna.setFrequency(_freqs[7]);	
				break;
			default:
				Sinusoide::setFrequency(0.0);
				_oscColonna.setFrequency(0.0);			
				
				if(n == _pauseChar && _pauseTime > 0)//pause char
				{
					_env.clearAllBreakPoints();
					_env.addBreakPoint(_pauseTime*0.001, 0);
					_retrig = false;
				}
				else//The other characters that are not allowed
				{
					_retrig = true;
				}
				break;
		}
	}
	
	void GenerateDTMF::Processing()
	{	
		if(_env.getTrigger() || _retrig)
		{	
			if(_breakCount < _stringa.length())
			{
				this->setEnvelope();
				_retrig = false;
				this->computeToneFreqs( _stringa[_breakCount] );
				_breakCount++;
			}
			else
			{
				if(_mode)
				{
					_breakCount = 0;
					_env.enableWrapAround(true);
					_env.trigger();
					_retrig = true;
					Sinusoide::setFrequency(0.0);
					_oscColonna.setFrequency(0.0);			
					
				}
				else
				{
					_breakCount = 0;
					_env.enableWrapAround(false);
					Sinusoide::setFrequency(0.0);
					_oscColonna.setFrequency(0.0);			
					
				}
			}					
		}
		
		Sinusoide::Processing();
		_oscColonna.PROCESSING();
		_noise.PROCESSING();
		_env.PROCESSING();
		
		_output_1 = ( ( Sinusoide::Output() + _oscColonna.Output() ) + _noise.Output() ) * _env.Output();
	}
	
	void GenerateDTMF::setEnvelope()
	{
		_env.clearAllBreakPoints();
		AudFloat attDec = (_toneTime*0.001)*.1; //10 % of duration
		_env.addBreakPoint(attDec, 1.0);
		_env.addBreakPoint((_toneTime*0.001) - (2.0 * attDec), 1.0);
		_env.addBreakPoint(attDec, 0.0);
		_env.addBreakPoint((_interrupt*0.001), 0.0);
	}
	
} //	end of namespace Aud
///EOF
