/***************************************************/
/*! \class Granulator
 \brief
 
 The AudioDrome Granulator class, implements 
 the  AGS (Asynchronous Granular Synthesis). 
 This DSP (Digital Signal Processing) manages
 all the classics parameters of the AGS
 
 by Alessandro Petrolati 2006.
 www.alessandro-petrolati.it
 info@alessandro-petrolati.it
 
 This library is free software; you can redistribute it and/or
 modify it under the terms of the GNU Library General Public
 License as published by the Free Software Foundation; either
 version 2 of the License, or (at your option) any later version.
 
 This library is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 Library General Public License for more details.
 
 You should have received a copy of the GNU Library General Public
 License along with this library; if not, write to the Free
 Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
/***************************************************/

#include "Granulator.h"
#include <iostream>

using namespace Aud; 

//  begin namespace
namespace Aud
{
	Granulator::Granulator(Wave* wave, Window* window, const unsigned int &maxOverlap) : 
	_overlap(maxOverlap), _defaultWave(false), _defaultWin(false)
	{
#if __DEBUG__
		std::cout << "Granulator Constructor: "<< this<<std::endl;
#endif		
		if (wave == NULL)
		{
			_defaultWave = true;
			_Wave = new Wave();
			static_cast<Wave*>(_Wave)->genSine();
		}
		else
			_Wave = wave;
		
		if (window == NULL)
		{
			_defaultWin = true;
			_Win = new Buffer();
			static_cast<Window*>(_Win)->genGaussian();
		}
		else
			_Win = window;
		
		_trig = new Metronome(false);
		_trig->setFrequency( _localSR ); /*! Sampling Rate Change SLOT */
		
		_osc = new OscilTimout* [_overlap];
		_env = new Envelops* [_overlap];
		
		_Left = new AudFloat [_overlap];
		_Rigth = new AudFloat [_overlap];
		_oscAct = new unsigned int [_overlap];
		
		for (unsigned int i = 0; i < _overlap; i++)
		{
			_osc[i] = new OscilTimout (_Wave, LINEAR, false);
			_env[i] = new Envelops (_Win);
		}
		
		this->setDefaults();
	}
	
	Granulator::~Granulator()
	{
#if __DEBUG__
		std::cout << "Granulatore Destructor: "<< this<<std::endl;
#endif
		if (_defaultWave)
			delete _Wave;
		if (_defaultWin)
			delete _Win;
		delete _trig;
		delete [] _Left;
		delete [] _Rigth;
		delete [] _oscAct;
		
		///Namely:
		for (unsigned int i = 0; i < _overlap; i++)
		{
			delete _osc[i];
			delete _env[i];
		}
	}
	
	void Granulator::setOverlap(const unsigned int &maxOverlap)
	{
		if (this->_overlap != maxOverlap)
		{
			for (unsigned int i = 0; i < _overlap; i++)
			{
				delete _osc[i];
				delete _env[i];
			}
			
			delete _Left;
			delete _Rigth;
			
			_overlap = maxOverlap;
			
			_osc = new OscilTimout* [_overlap];
			_env = new Envelops* [_overlap];
			_Left = new AudFloat [_overlap];
			_Rigth = new AudFloat [_overlap];
			
			for (unsigned int i = 0; i < _overlap; i++)
			{
				_osc[i] = new OscilTimout (_Wave);
				_env[i] = new Envelops (_Win);	
				
			}
		}
	}
	
	void Granulator::setWave(Wave* wave)
	{
		if (_defaultWave)
			delete _Wave;
		_Wave = wave;
		_defaultWave = false;
		for (unsigned int i = 0; i < _overlap; i++)
			_osc[i]->setWave(_Wave);
	}
	
	void Granulator::setWindow(Window* window)
	{
		if (_defaultWin)
			delete _Win;
		_Win = window;
		_defaultWin = false;
		for (unsigned int i = 0; i < _overlap; i++)
			_env[i]->setShape(_Win);
		///Use this, or setting up inside the Processing().
	}
	
	void Granulator::setDefaults()
	{
		setLevel(0.5);
		setWidth(0.25);
		setAttack(0.25);
		setDecay(0.25);
		setLength(85.0);
		setDensity(10.0);
		setFrequency(440.0);
		setPhase(0.0);
		setRndLength(0.5);
		setRndDensity(0.5);
		setRndFrequency(0.0);
		setRndPhase(0.0);
	}
	
	void Granulator::setLevel(const AudFloat &volume)
	{
		_Volume = volume; //normalized
	}
	
	void Granulator::setWidth(const AudFloat &width)
	{
		_Width = (fabs(width) > 1.0 ? 1.0 : fabs(width)); //normalized
	}
	
	void Granulator::setAttack(const AudFloat &attack)
	{
		_Attack = (fabs(attack) > 0.5 ? 0.5 : fabs(attack)); //% (0 - .5 (_Length/2))
	}
	
	void Granulator::setDecay(const AudFloat &decay)
	{
		_Decay = (fabs(decay) > 0.5 ? 0.5 : fabs(decay)); //% (0 - .5 (_Length/2))
	}
	
	void Granulator::setLength(const AudFloat &length)
	{
		_Length = fabs(length) * 0.001; //milliseconds
	}
	
	void Granulator::setDensity(const AudFloat &density)
	{
		_Density = fabs(density); //gps
	}
	
	void Granulator::setFrequency(const AudFloat &frequency)
	{
		_Frequency = frequency; //cps
	}
	
	void Granulator::setPhase(const AudFloat &phase)
	{
		_Phase = (fabs(phase) > 1.0 ? 1.0 : fabs(phase)); //from 0 to 1
	}
	
	void Granulator::setRndLength(const AudFloat &rndLength)
	{
		_RndLength = rndLength * 0.0005; // Namely: /200 //% of length
		//Namely: _RndLength = (rndLength / 1000.0) / 2.0;
	}
	
	void Granulator::setRndDensity(const AudFloat &rndDensity)
	{
		_RndDensity = rndDensity * 0.0005; // Namely: /200  //% of density
		//Namely: _RndDensity = (rndDensity / 1000.0) / 2.0;
	}
	
	void Granulator::setRndFrequency(const AudFloat &rndFrequency)
	{
		_RndFrequency = rndFrequency;
	}
	
	void Granulator::setRndPhase(const AudFloat &rndPhase)
	{
		_RndPhase = (fabs(rndPhase) > 1.0 ? 1.0 : fabs(rndPhase)); //from 0 to 1
	}
	
	void Granulator::Processing()
	{
		_trig->PROCESSING();
		if (_trig->Output())
		{
			AudFloat RndWidth = Random::unipolar(_Width);
			AudFloat Length = _Length + Random::bipolar(_RndLength * _Length);
			AudFloat Density = _Density + Random::bipolar(_RndDensity * _Density);
			AudFloat Frequency = _Frequency
			* pow(2.0, Random::bipolar(_RndFrequency));
			AudFloat Phase = _Phase + Random::bipolar(_RndPhase); //fabs??
			
			_trig->setFrequency(Density);
			
			for (unsigned int j = 0; j < _overlap; j++)
			{
				if (!_osc[j]->getStatus())
				{
					_osc[j]->setTime(Length);
					_osc[j]->start();
					_oscAct[j] = OscilTimout::getNumOscAtive();
					_osc[j]->setParameters(_Volume, Frequency, Phase);
					_env[j]->setParameters(Length, _Attack, _Decay);
					_env[j]->trigger();
					
					this->_Left[j] = RndWidth + (1 - _Width);
					this->_Rigth[j] = (1 - RndWidth);
					break;
				}
			}
		}
		
		AudFloat outSample = 0.0;
		_output_1 = 0.0;
		_output_2 = 0.0;
		
		for (unsigned int i = 0; i < _oscAct[i]; i++)
		{
			if (_osc[i]->getStatus() )
			{
				_osc[i]->PROCESSING();
				_env[i]->PROCESSING();
				outSample = _osc[i]->Output() * _env[i]->Output();
				_output_1 += outSample * _Left[i];
				_output_2 += outSample * _Rigth[i];
			}
		}
	}
} //	end of namespace Aud
///EOF
