/***************************************************/
/*! \class GranulatorSF
 \brief
 
 The AudioDrome GranulatorSF class, implements 
 the  AGS (Asynchronous Granular Synthesis). 
 This DSP (Digital Signal Processing) manages
 all the classics parameters of the AGS.
 This DSP, working on audio-files, are 
 also implemented control functions: pointers 
 to sound-file and jittering.
 
 by Alessandro Petrolati 2006.
 www.alessandro-petrolati.it
 info@alessandro-petrolati.it
 
 This library is free software; you can redistribute it and/or
 modify it under the terms of the GNU Library General Public
 License as published by the Free Software Foundation; either
 version 2 of the License, or (at your option) any later version.
 
 This library is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 Library General Public License for more details.
 
 You should have received a copy of the GNU Library General Public
 License along with this library; if not, write to the Free
 Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
/***************************************************/

#include "GranulatorSF.h"

using namespace Aud; 

//  begin namespace
namespace Aud
{
	GranulatorSF::GranulatorSF(Sample* sndFile, Window* window, const unsigned int &maxOverlap) 
	: Granulator((Wave*)sndFile, window, maxOverlap)
	{
#if __DEBUG__
		std::cout << "GranulatorSF Constructor: "<< this<<std::endl;
#endif
		if (sndFile != NULL)
			this->setSample(sndFile);
		
		this->setSpeed(1.0);
		this->setDefaults();			
	}
	
	GranulatorSF::~GranulatorSF()
	{
#if __DEBUG__
		std::cout << "GranulatorSF Destructor: "<< this<<std::endl;
#endif
	}
	
	void GranulatorSF::setSample(Sample* sndFile)
	{
		if(sndFile != NULL)
		{
			Granulator::setWave((Wave*)sndFile);
			Granulator::setFrequency(static_cast<Sample*>(_Wave)->getNaturalFrequency());
		}
		
	}
	
	void GranulatorSF::setDefaults()
	{
		Granulator::setDensity(50);
		
		this->setPitch(0.0);
		this->setSpeed(_speed);
		this->setStart(0.0);
		this->setEnd(1.0);
		this->setRndPitch(0.0);
		this->setJittering(5.0);
	}
	
	void GranulatorSF::setStart(const AudFloat &start)
	{
		_start = start; //normalized
		_rangeScan = _end - _start;
		setSpeed(_speed);
	}
	
	void GranulatorSF::setEnd(const AudFloat &end)
	{
		_end = end; //normalized
		
		_rangeScan = fabs(_end - _start);
		if (_rangeScan == 0.0)
			_rangeScan = 0.00001;
		
		setSpeed(_speed);
	}
	
	void GranulatorSF::setSpeed(const AudFloat &speed)
	{
		_speed = speed;
		if(!_defaultWave)
			_scanning.setFrequency(_speed / ((static_cast<Sample*>(_Wave)->getDurationTime() * _rangeScan)));
		else
			_scanning.setFrequency( 0.00001 );
	}
	
	void GranulatorSF::setPitch(const AudFloat &pitch)
	{
		if(!_defaultWave)
			Granulator::setFrequency( pow(2.0, (double)(pitch/12.0)) 
									 * static_cast<Sample*>(_Wave)->getNaturalFrequency() );
	}
	
	void GranulatorSF::setRndPitch(const AudFloat &rndPitch)
	{
		Granulator::setRndFrequency(rndPitch / 24.0); //semitones
	}
	
	void GranulatorSF::setJittering(const AudFloat &jittering)
	{
		if(!_defaultWave)
			Granulator::setRndPhase( ( (jittering*0.001) / static_cast<Sample*>(_Wave)->getDurationTime()) );
	}
	
	void GranulatorSF::Processing()
	{
		_scanning.PROCESSING();
		Granulator::setPhase( _scanning.Output() * _rangeScan );
		Granulator::Processing();
	}
} //	end of namespace Aud
///EOF
