/***************************************************/
/*! \class LowPass Filter
 \brief
 
 The AudioDrome LowPass class, implements a first-order filter.
 
 Csound's code, based on Robin Whittle's
 ugrw2.c and ugrw2.h of 9 September 1995
 
 by Alessandro Petrolati 2006.
 www.alessandro-petrolati.it
 info@alessandro-petrolati.it
 
 This library is free software; you can redistribute it and/or
 modify it under the terms of the GNU Library General Public
 License as published by the Free Software Foundation; either
 version 2 of the License, or (at your option) any later version.
 
 This library is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 Library General Public License for more details.
 
 You should have received a copy of the GNU Library General Public
 License along with this library; if not, write to the Free
 Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
/***************************************************/

#include "LowPass.h"

using namespace Aud; 

//  begin namespace
namespace Aud
{
	LowPass::LowPass(const AudFloat &cutOff)
	{
#if __DEBUG__
		std::cout << "LowPass Filter Constructor: "<< this <<std::endl;
#endif
		_c1 = 0.0;
		_c2 = 1.0;
		_yt1 = 0.0;
		this->setCutOff(cutOff);
	}
	
	LowPass::~LowPass()
	{
#if __DEBUG__
		std::cout << "LowPass Filter Destructor: "<< this <<std::endl;
#endif
	}
	
	void LowPass::setCutOff(const AudFloat &cutOff) /*! Sampling Rate Change SLOT */
	{
		_cutOff = cutOff;
		/* 
		 b = cos ( 2 * pi * freq / krate)
		 c2 = b - sqrt ( b squared - 1)
		 c1 = 1 - c2
		 tpidsr = 2 * pi / a sample rate
		 so tpidsr * ksmps = 2 * pi / k rate.
		 We need this since we are filtering at k rate, not a rate.
		 */
		AudFloat b = 2.0 - (AudFloat) cos((double)(_cutOff * ( 2 * M_PI / _localSR)));
		_c2 = b - (AudFloat)sqrt((double)(b * b - 1.0));
		_c1 = 1.0 - _c2;
	}
	
	void LowPass::Processing()
	{
		/* 
		 Filter calculation on each a rate sample:
		 New state =   (c2 * old state) + (c1 * input value)               
		 */
		_output_1 = _yt1 = _c1 * _input_1 + _c2 * _yt1;
	}
} //	end of namespace Aud
///EOF
