/***************************************************/
/*! \class MoogLadder LowPass Filter
 \brief
 
 The AudioDrome MoogLadder class, implements
 a Moog ladder lowpass filter.
 Moogladder is an new digital implementation of the 
 Moog ladder filter based on the work of Antti Huovilainen, 
 described in the paper "Non-Linear Digital Implementation
 of the Moog Ladder Filter" (Proceedings of DaFX04, Univ of Napoli). 
 This implementation is probably a more accurate digital 
 representation of the original analogue filter.
 
 Csound's code newfils.c
 Author: Victor Lazzarini;
 January 2005
 
 by Alessandro Petrolati 2008.
 www.alessandro-petrolati.it
 info@alessandro-petrolati.it
 
 This library is free software; you can redistribute it and/or
 modify it under the terms of the GNU Library General Public
 License as published by the Free Software Foundation; either
 version 2 of the License, or (at your option) any later version.
 
 This library is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 Library General Public License for more details.
 
 You should have received a copy of the GNU Library General Public
 License along with this library; if not, write to the Free
 Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
/***************************************************/

#include "MoogLadder.h"

using namespace Aud; 

//  begin namespace
namespace Aud
{
	
	MoogLadder::MoogLadder(const AudFloat& freq, const AudFloat& res, const AudFloat& V)
	{
#if __DEBUG__
		std::cout << "MoogLadder Constructor: "<< this <<std::endl;
#endif
		_res4 = _input = _acr = _tune = 0.0;
		_thermal = (1.0 / V);//40000.0);  /* transistor thermal voltage  */		
		
		_delay = new AudFloat[6];
		_tanhstg = new AudFloat[3];
		_stg = new double[4];
		
		for (int i = 0; i < 6; i++)
			_delay[i] = 0.0;
		for (int i = 0; i < 3; i++)
			_tanhstg[i] = 0.0;
		for (int i = 0; i < 4; i++)
			_stg[i] = 0.0;
		
		this->setCutOff(freq);
		this->setResonance(res);
	}
	
	MoogLadder::~MoogLadder()
	{
#if __DEBUG__
		std::cout << "MoogLadder Destructor: "<< this <<std::endl;
#endif
		delete[] _delay;
		delete[] _tanhstg;
		delete[] _stg;
	}
	
	void MoogLadder::setResonance(const AudFloat& res)
	{
		if (res < 0) 
			_res = 0;
		else 
			_res = res;
		_res4 = 4.0 * (double) _res * _acr;		
		
	}
	
	void MoogLadder::setCutOff(const AudFloat& hz) /*! Sampling Rate Change SLOT */
	{
		_freq = hz;
		/* sr is half the actual filter sampling rate  */
		double fc =  (double)(_freq/_localSR);
		double f  =  fc/2;
		double fc2 = fc*fc;
		double fc3 = fc2*fc;
		/* frequency & amplitude correction  */
		double fcr = 1.8730*fc3 + 0.4955*fc2 - 0.6490*fc + 0.9988;
		_acr = -3.9364*fc2 + 1.8409*fc + 0.9968;
		_tune = (1.0 - exp(-(M_TWO_PI*f*fcr))) / _thermal;   /* filter tuning  */
		_res4 = 4.0*(double)_res*_acr;
		
	}
	
	void MoogLadder::setTransistorVoltage(const AudFloat& V)
	{
		//base-emitter voltages of the transistors
		_thermal = (1.0 / V);  /* transistor thermal voltage  */
		this->setCutOff(_freq);
		this->setResonance(_res);
	}
	
	void MoogLadder::Processing()
	{						
		for (int j = 0; j < 2; j++) {
			/* filter stages  */
			for (int k = 0; k < 4; k++) {
				if (k) {
					_input = _stg[k-1];
					_stg[k] = _delay[k]
					+ _tune*((_tanhstg[k-1] = tanh(_input*_thermal))
							 - (k != 3 ? _tanhstg[k] : tanh(_delay[k]*_thermal)));
				}
				else {
					_input = _input_1 - _res4 /*4.0*_res*_acr*/ * _delay[5];
					_stg[k] = _delay[k] + _tune*(tanh(_input*_thermal) - _tanhstg[k]);
				}
				_delay[k] = _stg[k];
			}
			/* 1/2-sample delay for phase compensation  */
			_delay[5] = (_stg[3] + _delay[4])*0.5;
			_delay[4] = _stg[3];
		}
		_output_1 = _delay[5];
	}	
} //	end of namespace Aud
///EOF


