/***************************************************/
/*! \class Oscillator
 \brief
 
 The AudioDrome Oscillator class, implements a 
 oscillator with interpolation.
 
 by Alessandro Petrolati 2006.
 www.alessandro-petrolati.it
 info@alessandro-petrolati.it
 
 This library is free software; you can redistribute it and/or
 modify it under the terms of the GNU Library General Public
 License as published by the Free Software Foundation; either
 version 2 of the License, or (at your option) any later version.
 
 This library is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 Library General Public License for more details.
 
 You should have received a copy of the GNU Library General Public
 License along with this library; if not, write to the Free
 Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
/***************************************************/

#include "Oscillator.h"

using namespace Aud; 

//  begin namespace
namespace Aud
{
	Oscillator::Oscillator(Buffer* wave,
						   const INTERPOLATION& type) : _flag(false)
	{
#if __DEBUG__
		std::cout << "Oscillator Construct: " << this <<std::endl;
#endif
		if (wave == NULL)
		{
			_flag = true;
			Wave* tmp = new Wave();
			tmp->genSine();
			_Wave = new Buffer();
			_Wave->copy(*tmp);
			delete tmp;
		}
		else
			_Wave = wave;
		
		this->setParameters();
		this->setInterpolation(type);
	}
	
	Oscillator::~Oscillator()
	{
#if __DEBUG__
		std::cout << "Oscillator Destruct: " << this <<std::endl;
#endif
		
		if (_flag)
			delete _Wave;
	}
	
	void Oscillator::setWave(Buffer* wave)
	{
		if (_flag)
			delete _Wave;
		_Wave = wave;
		_flag = false;
	}
	
	void Oscillator::setParameters(const AudFloat &amp, const AudFloat &freq,
								   const AudFloat &phi)
	{
		this->setAmplitude(amp);
		Phasor::setFrequency(freq);
		Phasor::setPhase(phi);
	}
	
	void Oscillator::setAmplitude(const AudFloat &amp)
	{
		_amp = amp;
	}
	
	void Oscillator::setInterpolation(const INTERPOLATION &type)
	{
		switch (type)
		{
			case (DISABLED):
			{
				_interpPtr = &Buffer::readRaw;
			}
				break;
			case (LINEAR):
			{
				_interpPtr = &Buffer::readInterp;
			}
				break;
			case (CUBIC):
			{
				_interpPtr = &Buffer::readInterp3;
			}
				break;
			case (VARGA):
			{
				_interpPtr = &Buffer::readInterp3Varga;
			}
				break;
			default:
			{
				_interpPtr = &Buffer::readInterp;
			}
		}
	}
	
	void Oscillator::Processing()
	{
		Phasor::Processing();
		_output_1 = (_Wave->*_interpPtr)(Phasor::Output()) * _amp;
	}
} //	end of namespace Aud
///EOF
