/***************************************************/
/*! \class ParametricEQ Filters
 \brief
 
 The AudioDrome ParametricEQ class, implements
 a parametric equalizer.
 This implementation of Zoelzer's parametric equalizer filters,
 with some modifications by the author.
 
 Csound's code: biquad.c
 
 by Alessandro Petrolati 2006.
 www.alessandro-petrolati.it
 info@alessandro-petrolati.it
 
 This library is free software; you can redistribute it and/or
 modify it under the terms of the GNU Library General Public
 License as published by the Free Software Foundation; either
 version 2 of the License, or (at your option) any later version.
 
 This library is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 Library General Public License for more details.
 
 You should have received a copy of the GNU Library General Public
 License along with this library; if not, write to the Free
 Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
/***************************************************/

#include "ParametricEQ.h"

using namespace Aud; 

namespace Aud
{
	
	ParametricEQ::ParametricEQ(const AudFloat &fc, const AudFloat &amount,
							   const AudFloat &q, const MODE &mode)
	{
#if __DEBUG__
		std::cout << "ParametricEQ Filter Constructor: "<< this <<std::endl;
#endif
		this->calcCoefs();
		this->setFc(fc);
		this->setAmount(amount);
		this->setQuality(q);
	}
	
	ParametricEQ::~ParametricEQ()
	{
#if __DEBUG__
		std::cout << "ParametricEQ Filter Destructor: "<< this <<std::endl;
#endif
	}
	
	void ParametricEQ::Processing()
	{
		AudFloat xn, yn;
		double a1 = _a1, a2 = _a2;
		double b0 = _b0, b1 = _b1, b2 = _b2;
		double xnm1 = _xnm1, xnm2 = _xnm2, ynm1 = _ynm1, ynm2 = _ynm2;
		xn = (double)_input_1;
		yn = b0 * xn + b1 * xnm1 + b2 * xnm2 - a1 * ynm1 - a2 * ynm2;
		xnm2 = xnm1;
		xnm1 = xn;
		ynm2 = ynm1;
		ynm1 = yn;
		_output_1 = yn;
		_xnm1 = xnm1;
		_xnm2 = xnm2;
		_ynm1 = ynm1;
		_ynm2 = ynm2;
	}
	
	void ParametricEQ::mode(const MODE &mode)
	{
		_mode = mode;
		double omega = (double)(_tpidsr * _fc), k, kk, vkk, vk, vkdq, a0;
		
		switch (mode)
		{
			case PEAKING:
			{
				/* Peaking EQ */
				k = tan(omega * 0.5);
				kk = k * k;
				vk = (double)_v * k;
				vkdq = vk / (double)_q;
				_b0 = 1.0 + vkdq + kk;
				_b1 = 2.0 * (kk - 1.0);
				_b2 = 1.0 - vkdq + kk;
				a0 = 1.0 + k / (double)_q + kk;
				_a1 = 2.0 * (kk - 1.0);
				_a2 = 1.0 - k / (double)_q + kk;
			}
				break;
				/* Low Shelf */
			case LOWSHELVING:
			{
				double sq = sqrt(2.0 * (double)_v);
				k = tan(omega * 0.5);
				kk = k * k;
				vkk = (double)_v * kk;
				_b0 = 1.0 + sq * k + vkk;
				_b1 = 2.0 * (vkk - 1.0);
				_b2 = 1.0 - sq * k + vkk;
				a0 = 1.0 + k / (double)_q + kk;
				_a1 = 2.0 * (kk - 1.0);
				_a2 = 1.0 - k / (double)_q + kk;
			}
				break;
				/* High Shelf */
			case HIGHSHELVING:
			{
				double sq = sqrt(2.0 * (double)_v);
				k = tan((M_PI - omega) * 0.5);
				kk = k * k;
				vkk = (double)_v * kk;
				_b0 = 1.0 + sq * k + vkk;
				_b1 = -2.0 * (vkk - 1.0);
				_b2 = 1.0 - sq * k + vkk;
				a0 = 1.0 + k / (double)_q + kk;
				_a1 = -2.0 * (kk - 1.0);
				_a2 = 1.0 - k / (double)_q + kk;
			}
				break;
				/* Peaking EQ */
			default:
			{
				k = tan(omega * 0.5);
				kk = k * k;
				vk = (double)_v * k;
				vkdq = vk / (double)_q;
				_b0 = 1.0 + vkdq + kk;
				_b1 = 2.0 * (kk - 1.0);
				_b2 = 1.0 - vkdq + kk;
				a0 = 1.0 + k / (double)_q + kk;
				_a1 = 2.0 * (kk - 1.0);
				_a2 = 1.0 - k / (double)_q + kk;
			}
		}
		a0 = 1.0 / a0;
		_a1 *= a0;
		_a2 *= a0;
		_b0 *= a0;
		_b1 *= a0;
		_b2 *= a0;
	}
}
///EOF

