/***************************************************/
/*! \class Phasor
 \brief
 
 The AudioDrome Phasor class, implements
 a normalized moving phase value.
 An internal phase is successively accumulated 
 in accordance with the setFrequency() to produce
 a moving phase value, normalized in the range 0 <= phs < 1.
 When used as the index to a Buffer object, this phase 
 (multiplied by the desired function table length) 
 will cause it to behave like an oscillator.
 
 Note that phasor is a special kind of integrator, accumulating phase increments that represent frequency settings.
 
 by Alessandro Petrolati 2006.
 www.alessandro-petrolati.it
 info@alessandro-petrolati.it
 
 This library is free software; you can redistribute it and/or
 modify it under the terms of the GNU Library General Public
 License as published by the Free Software Foundation; either
 version 2 of the License, or (at your option) any later version.
 
 This library is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 Library General Public License for more details.
 
 You should have received a copy of the GNU Library General Public
 License along with this library; if not, write to the Free
 Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
/***************************************************/

#include "Phasor.h"

using namespace Aud; 

//  begin namespace
namespace Aud
{
	Phasor::Phasor()
	{
#if __DEBUG__
		std::cout << "Phasor Constructor: " << this <<std::endl;
#endif
		_phi = 0.0;
		_incr = 0.0;
		_trigger = false;
	}
	
	Phasor::~Phasor()
	{
#if __DEBUG__
		std::cout << "Phasor Destructor: " << this <<std::endl;
#endif
	}
	
	void Phasor::setFrequency(const AudFloat &freq) /*! Sampling Rate Change SLOT */
	{
		_freq = freq;
		_incr = _freq / _localSR;
	}
	
	void Phasor::setPhase(const AudFloat &phi)
	{
		_phi = (fabs (phi) > 1.0 ? 1.0 : fabs(phi));
	}
	
	bool Phasor::getTrigger() const
	{
		return _trigger;
	}
	
	void Phasor::Processing()
	{
		_output_1 = _phi += _incr;
		_trigger = false;
		if (_phi >= 1.0)
		{
			_phi -= 1.0;
			_trigger = true;
		}
		else if (_phi < 0.0)
		{
			_phi += 1.0;
			_trigger = true;
		}
		_output_1 -= _incr;
	}
} //	end of namespace Aud
///EOF
