/***************************************************/
/*! \class RmsBalance
 \brief
 
 The AudioDrome RmsBalance, adjust 
 one audio signal according to the values of another.
 Determines the root-mean-square amplitude of an audio signal
 and adjusts the amplitude audio signal according to a root-mean-square value.
 
 Csound sources ugens5.c
 by Alessandro Petrolati 2008.
 www.alessandro-petrolati.it
 info@alessandro-petrolati.it
 
 This library is free software; you can redistribute it and/or
 modify it under the terms of the GNU Library General Public
 License as published by the Free Software Foundation; either
 version 2 of the License, or (at your option) any later version.
 
 This library is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 Library General Public License for more details.
 
 You should have received a copy of the GNU Library General Public
 License along with this library; if not, write to the Free
 Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
/***************************************************/

#include "RmsBalance.h"

using namespace Aud; 
using namespace Aud;

RmsBalance::RmsBalance(const ALGORITHM& type) : _prvq(0.0), _prvr(0.0), _prva(0.0)
{	
#if __DEBUG__
	std::cout << "RmsBalance Constructor: "<< this <<std::endl;
#endif
	
	this->setAlgorithm(type);
	this->setHalfPowerPoint(10);
}


RmsBalance::~RmsBalance()
{
#if __DEBUG__
	std::cout << "RmsBalance Destructor: "<< this <<std::endl;
#endif
}

void RmsBalance::setAlgorithm(const ALGORITHM& type)
{
	switch (type)
	{
		case (SIMPLE):
			_balancePtr = &RmsBalance::balanceSimple;
			break;
		case (BEST):
			_balancePtr = &RmsBalance::balanceBest;
			break;
		default:
			_balancePtr = &RmsBalance::balanceBest;
	}
}

void RmsBalance::setHalfPowerPoint(const AudFloat& cutoff) /*! Sampling Rate Change SLOT */
{
	_cutOff = cutoff;
	//Rms SIMPLE
	_costh = 2.0 - cos(M_TWO_PI * _cutOff / _localSR); 
	_coef = sqrt(_costh*_costh - 1.0) - _costh; 
	
	//Csound BEST
	double   b;
	b = 2.0 - cos((double)(_cutOff / _localSR));
	_c2 = b - sqrt(b*b - 1.0);
	_c1 = 1.0 - _c2;
}

void RmsBalance::Processing()
{
	_output_1 = (this->*_balancePtr)(_input_1, _compareSig);
}


AudFloat RmsBalance::balanceBest (const AudFloat& input, const AudFloat& compareSig)
{
	_q = _prvq;
	_r = _prvr;				
	
	double as = (double) input;
	double cs = (double) compareSig;
	
	_q = _c1 * as * as + _c2 * _q;
	_r = _c1 * cs * cs + _c2 * _r;
	
	_prvq = _q;
	_prvr = _r;
	if (_q != 0.0)
		_rms = sqrt(_r/_q);
	else
		_rms = sqrt(_r);
	////Linear interpolation, when change the value for the ksmps samples
	////ugens5.c riga 977-985
	//if ((diff = a - _prva) != 0.0) 
	//{
	//_rms = _m = _prva;
	//_inc = _diff; //* (double) csound->onedksmps; <==N.B.
	//_ar = input * _m;
	//_m += _inc;
	//_prva = _rms;
	//}
	//else 
	_ar = input * _rms;
	return _ar;
}

AudFloat  RmsBalance::balanceSimple (const AudFloat& input, const AudFloat& compareSig)
{ 
	_asig = input;				
	_del[0] = (AudFloat)
	((input < 0 ? -input : input) * (1+_coef)  
	 - _del[0]*_coef); 
	
	_del[1] = (AudFloat)
	((compareSig < 0 ? -compareSig : compareSig)*(1+_coef)  
	 - _del[1] * _coef); 
	
	_asig *= (AudFloat)(_del[0] ? _del[1]/_del[0] : _del[1]); 
	return _asig;
} //	end of namespace Aud
///EOF
