/***************************************************/
/*! \class Sample
 \brief
 
 This class inherits from Buffer
 The left channel of audio-file, 
 are copied in the Buffer::_table. 
 
 If the file is multichannels, the samples
 are subsequently writed.
 In this moment, only raw and wav files, 
 are supported, no aiff are supported.
 
 Thanks to by Dr Fred DePiero - CalPoly State University
 http://people.msoe.edu/~taylor/examples/wav.htm
 
 by Alessandro Petrolati 2006.
 www.alessandro-petrolati.it
 info@alessandro-petrolati.it
 
 This library is free software; you can redistribute it and/or
 modify it under the terms of the GNU Library General Public
 License as published by the Free Software Foundation; either
 version 2 of the License, or (at your option) any later version.
 
 This library is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 Library General Public License for more details.
 
 You should have received a copy of the GNU Library General Public
 License along with this library; if not, write to the Free
 Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
/***************************************************/

#include "Sample.h"
#include <fstream>

using namespace Aud; 

//  begin namespace
namespace Aud
{
	Sample::Sample() : Buffer(0), _filePtr(NULL)
	{
#if __DEBUG__
		std::cout << "Sample Constructor: "<< this<<std::endl;
#endif
		// allocate _wav header 
		_wav = new WAV_HDR;
		_chk = new CHUNK_HDR;
		if(_wav==NULL)
			throw(AudError(NULL_POINTER));
		if(_chk==NULL)
			throw(AudError(NULL_POINTER));
		
		// setup defaultd
		_wav->nSamplesPerSec = 44100;
		_wav->nBitsPerSample = 16;
		_wav->nChannels = 2;
		
		// open and inport file
		this->setPath("/");
	}
	
	Sample::~Sample()
	{
#if __DEBUG__
		std::cout << "Sample Destructor: "<< this <<std::endl;
#endif
		//if is open, close file and clear string path
		this->closeSample();
		
		// deleting header structs
		if(_wav!=NULL) 
			delete _wav;
		if(_chk!=NULL) 
			delete _chk;		
	}
	
	void Sample::setPath(const std::string &path)
	{
		_path.clear();
		_path = path;
		
		// Make sure the path includes a "/"
		if (_path[_path.length()-1] != '/')
			_path += "/";
		
	}
	
	void Sample::importWav(const std::string &filename) throw(AudError)
	{
		_fileName.clear();
		_fileName = filename;
		
		this->openSample();
		this->readRiff();		
		this->readChunks();			
		this->converts();
		this->closeSample();
	}
	
	void Sample::importRaw(const std::string &filename, const AudFloat &sr, 
						   const short &bit, const short &chnl)  throw(AudError)
	{	
		_fileName.clear();
		_fileName = filename;
		
		this->openSample();
		
		// obtain file size:
		if(_filePtr)
		{
			fseek (_filePtr , 0 , SEEK_END);
			_chk->dLen = ftell (_filePtr);
			rewind (_filePtr);
		}
		
		// set default value
		_wav->nBitsPerSample = bit;
		_wav->nChannels = chnl;
		_wav->nSamplesPerSec = sr;
		
		this->converts();
		this->closeSample();		
	}
	
	void Sample::printInfo() const
	{
		std::cout << std::endl << "∞∞∞∞∞∞∞∞∞∞ FILE INFORMATION ∞∞∞∞∞∞∞∞∞∞"<< std::endl <<std::endl;
		std::cout << "Sample Name: "<< getName() << std::endl;
		std::cout << "Path: "<< getPath() << std::endl;
		std::cout << "Number of Samples: " << getNumberOfSamples() << std::endl;
		std::cout << "Sample Sr: "<< getFileSR() << std::endl;
		std::cout << "Sample Channels: "<< getChannels() << std::endl;
		std::cout << "Sample Time Duration: "<< getDurationTime() << std::endl;
		std::cout << "Bits Per Sample = " << getBitPerSamples() << std::endl;
		std::cout << "Number of Channels = " << _wav->nChannels << std::endl;
		
		std::cout << std::endl << "∞∞∞∞∞∞∞∞∞∞∞∞∞∞∞∞∞∞∞∞∞∞∞∞∞∞∞∞∞∞∞∞∞∞∞∞∞∞"<< std::endl;
	}
	
	/*!  private */
	
	void Sample::readRiff()
	{
		/*!
		 * An example oh Wav header file, is the following:
		 *
		 * RIFF|dim|WAVE|fmt |16|1|2|8000|32000|4|16|data|1920000|
		 *
		 * The wav-file header, is a string with length of 44 byte. 
		 * Every 4 byte has one information specifies.
		 */
		
		unsigned int wstat;
		long int rmore;
		/* read riff/_wav header */ 
		wstat = fread((void*)_wav, sizeof(WAV_HDR), (size_t) 1, _filePtr);
		if(wstat!= 1)
		{ 
			AudError e;
			e.setMsgError("cant read _wav\n"); 
			throw(e);
		}
		
		// check format of header 
		for(int i = 0; i < 4; i++) 
			_obuff[i] = _wav->rID[i];
		_obuff[4] = 0;
		
		if(strcmp(_obuff,"RIFF") != 0)
		{ 
			AudError e;
			e.setMsgError("bad RIFF format\n"); 
			throw(e);
		}
		
		for(int i = 0; i < 4; i++) 
			_obuff[i] = _wav->wID[i];
		_obuff[4] = 0;
		
		if(strcmp(_obuff,"WAVE")!=0)
		{ 
			AudError e;
			e.setMsgError("bad WAVE format\n"); 
			throw(e); 
		}
		
		for(int i = 0; i < 3; i++) 
			_obuff[i] = _wav->fId[i];
		_obuff[3] = 0;
		
		if(strcmp(_obuff,"fmt")!=0)
		{ 
			AudError e;
			e.setMsgError("bad fmt format\n"); 
			throw(e); 
		}
		
		// skip over any remaining portion of _wav header 
		rmore = _wav->pcm_header_len - (sizeof(WAV_HDR) - 20);
		wstat = fseek(_filePtr, rmore, SEEK_CUR);
		if(wstat!=0)
		{ 
			AudError e;
			e.setMsgError("cant seek\n"); 
			throw(e); 
		}
	}
	
	void Sample::readChunks()
	{
		unsigned int wstat;
		int sflag = 1;
		
		while(sflag!=0){
			
			// check attempts 
			if(sflag>10)
			{ 
				AudError e;
				e.setMsgError("too many chunks\n"); 
				throw(e); 
			}
			
			// read chunk header 
			wstat = fread((void *)_chk,sizeof(CHUNK_HDR),(size_t)1,_filePtr);
			if(wstat!=1)
			{ 
				AudError e;
				e.setMsgError("cant read chunk\n"); 
				throw(e); 
			}
			
			// check chunk type 
			for(int i = 0; i < 4; i++) 
				_obuff[i] = _chk->dId[i];
			_obuff[4] = 0;
			if(strcmp(_obuff,"data")==0) break;
			
			// skip over chunk 
			sflag++; 
			wstat = fseek(_filePtr, _chk->dLen, SEEK_CUR); //current position
			if(wstat!=0)
			{ 
				AudError e;
				e.setMsgError("cant seek\n"); 
				throw(e); 
			}
		}
		
	}
	
	void Sample::converts()
	{
		/*
		 *                   Specific integral type limits
		 *
		 * Specifier   Common Equivalent	Signing      Bits    	    Bytes
		 *
		 * int8_t	   signed char	        Signed	     8	            1
		 * uint8_t	   unsigned char	    Unsigned	 8	            1
		 * int16_t	   short	            Signed	     16	            2
		 * uint16_t	   unsigned short	    Unsigned	 16	            2
		 * int32_t	   long	                Signed	     32	            4
		 * uint32_t	   unsigned long 	    Unsigned	 32	            4
		 * int64_t	   long long	        Signed       64	            8
		 * uint64_t	   unsigned long long	Unsigned     64	            8
		 */
		/*! 8 bit
		 * -------
		 * unsigned char = 1 byte
		 * double = 4 byte 
		 *
		 * |____|____|____|____|____|____| byte
		 * 1    2    3    4    5    6    7
		 * |____|____|____|____|____|____| samples
		 * 1    2    3    4    5    6    7
		 *
		 * unsigned char* cptr = reinterpret_cast<unsigned char*>(wbuff);
		 *
		 *
		 * 16 bit
		 * -------
		 * char = 1 byte
		 * short = 2 byte
		 *
		 * |____|____|____|____|____|____| byte
		 * 1    2    3    4    5    6    7
		 * |_________|_________|_________| samples
		 * 1         2         3         4
		 *
		 * short int* uptr = reinterpret_cast<short*>(wbuff);
		 *
		 * 24 bit
		 * -------
		 *
		 * |____|____|____|____|____|____|____|____|____| byte
		 * 1    2    3    4    5    6    7    8    9    10
		 * |______________|______________|______________| samples
		 * 1              2              3              4
		 *
		 * ???
		 *
		 * 32 bit
		 * -------
		 * unsigned char = 1 byte
		 * float = 4 byte
		 *
		 * |____|____|____|____|____|____|____|____|____|____|____|____| byte
		 * 1    2    3    4    5    6    7    8    9    10   11   12   13
		 * |___________________|___________________|___________________| samples
		 * 1                   2                   3                   4   
		 *
		 * float* fptr = reinterpret_cast<float*>(wbuff);
		 */
		
		unsigned int wstat;
		
		/* allocate new buffers */ 
		char* wbuff = new char [_chk->dLen];
		
		if(wbuff==NULL)
		{ 
			AudError e;
			e.setMsgError("cant alloc\n"); 
			throw(e); 
		}
		
		/* read signal data */ 
		wstat = fread((void *)wbuff,_chk->dLen,(size_t)1,_filePtr);
		
		if(wstat!=1)
		{ 
			AudError e;
			e.setMsgError("cant read wbuff\n"); 
			throw(e); 
		}
		
		//AudioDrome_Buffer memory allocation
		this->resize(getNumberOfSamples());
		
		// convert data 			
		if(_wav->nBitsPerSample == 8)
		{
			unsigned char* cptr = reinterpret_cast<unsigned char*>(wbuff);
			for(int i = 0; i < getNumberOfSamples(); i++) 
				_table[i] = (AudFloat) (cptr[i]);
		}
		
		else if(_wav->nBitsPerSample == 16)
		{
			short int* uptr = reinterpret_cast<short*>(wbuff);
			for(int i = 0; i < getNumberOfSamples(); i++) 
				_table[i] = (AudFloat) (uptr[i]);
		}
		
		else if(_wav->nBitsPerSample == 24)
		{
			AudError e;
			e.setMsgError("Not implemented!"); 
			throw(e); 
		}
		
		else if(_wav->nBitsPerSample == 32)
		{
			float* fptr = reinterpret_cast<float*>(wbuff);
			for(int i = 0; i < getNumberOfSamples(); i++) 
				_table[i] = (AudFloat) (fptr[i]);
		}
		
		this->Buffer::normalize();
		
		// be polite - clean up 
		if(wbuff!=NULL) 
			delete wbuff;
	}			
	
} //	end of namespace Aud
///EOF
