/***************************************************/
/*! \class Schroeder
 \brief
 
 The AudioDrome Schroeder, implements 
 a classic reverb.
 A standard reverb unit is composed 
 of four Comb filters in parallel followed
 by two Alpass units in series. 
 Loop times are set for optimal “natural room response.”
 
 Definition for reverberation time: 
 Time until the sound has decreased 
 to 0.1% of the original amplitude.
 The original signal is piped through 
 4 parallel comb filters then through 
 2 serial all-pass filters.
 Comb delay times are relatively prime to 
 one another (0.031, 0.037, 0.041, 0.043)
 The all-pass filters have delays between 
 0.005 and 0.02 seconds. The PROCESSING the echo density. 
 The first all-pass turns each comb echo into a series.
 
 by Alessandro Petrolati 2006.
 www.alessandro-petrolati.it
 info@alessandro-petrolati.it
 
 This library is free software; you can redistribute it and/or
 modify it under the terms of the GNU Library General Public
 License as published by the Free Software Foundation; either
 version 2 of the License, or (at your option) any later version.
 
 This library is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 Library General Public License for more details.
 
 You should have received a copy of the GNU Library General Public
 License along with this library; if not, write to the Free
 Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
/***************************************************/

#include "Schroeder.h"

using namespace Aud; 

namespace Aud
{
	
	Schroeder::Schroeder(const AudFloat& t60, const AudFloat &maxTimeRev, const INTERPOLATION& type)
	{
#if __DEBUG__
		std::cout << "SchroederRev Constructor: "<< this<<std::endl;
#endif
		_norm = 0.91999;
		
		_cmb1 = new Comb(maxTimeRev);
		_cmb2 = new Comb(maxTimeRev);
		_cmb3 = new Comb(maxTimeRev);
		_cmb4 = new Comb(maxTimeRev);
		_allPass1 = new AllPass(1.0);
		_allPass2 = new AllPass(1.0);
		
		_cmb1->setDelayTime(.0297);
		_cmb2->setDelayTime(.0371);
		_cmb3->setDelayTime(.0411);
		_cmb4->setDelayTime(.0437);
		_allPass1->setDelayTime(.005);
		_allPass1->setT60(.096835);
		_allPass2->setDelayTime(.0017);
		_allPass2->setT60(.032924);
		
		_cmb1->setInterpolation(type);
		_cmb2->setInterpolation(type);
		_cmb3->setInterpolation(type);
		_cmb4->setInterpolation(type);
		
		_allPass1->setInterpolation(type);
		_allPass2->setInterpolation(type);
		
		this->setMaxT60(t60 * 2.0);
		this->setT60(t60);
		this->setAmpGain();
		this->setMixing(0.5);
	}
	
	Schroeder::~Schroeder()
	{
#if __DEBUG__
		std::cout << "SchroederRev Destructor: "<< this <<std::endl;
#endif
		delete _cmb1;
		delete _cmb2;
		delete _cmb3;
		delete _cmb4;
		delete _allPass1;
		delete _allPass2;
	}
	void Schroeder::setMaxT60(const AudFloat &maxTimeRev)
	{
		_cmb1->setMaxDelayTime(maxTimeRev);
		_cmb2->setMaxDelayTime(maxTimeRev);
		_cmb3->setMaxDelayTime(maxTimeRev);
		_cmb4->setMaxDelayTime(maxTimeRev);
	}
	
	//	 gain to control amplitude, 0 <= igain <= 1
	void Schroeder::setAmpGain(const AudFloat& ampGain)
	{
		_gain = (ampGain <= 0 ? .01 : ampGain);
		_gain = (ampGain > 1.0 ? 1.0 : ampGain);
	}
	
	//	percent reverberated signal, 0 <= ireverb <= 1
	void Schroeder::setMixing(const AudFloat& mixing)
	{
		_reverb = (mixing > .99 ? .99 : mixing);
		_reverb = (mixing < 0 ? 0 : mixing);
		_acoustic = 1 - _reverb; //percent unreverberated
	}
	
	void Schroeder::setT60(const AudFloat& t60)
	{
		_cmb1->setT60(t60);
		_cmb2->setT60(t60);
		_cmb3->setT60(t60);
		_cmb4->setT60(t60);
	}
	
	void Schroeder::Processing()
	{
		_cmb1->Input(_input_1);
		_cmb2->Input(_input_1);
		_cmb3->Input(_input_1);
		_cmb4->Input(_input_1);
		_cmb1->PROCESSING();
		_cmb2->PROCESSING();
		_cmb3->PROCESSING();
		_cmb4->PROCESSING();
		
		AudFloat comb = _cmb1->Output() + _cmb2->Output()
		+ _cmb3->Output() + _cmb4->Output();

		_allPass1->Input(comb);
		_allPass1->PROCESSING();
		_allPass2->Input( _allPass1->Output() );
		_allPass2->PROCESSING();
		
		_output_1 = (_acoustic * _input_1) + (_reverb * _allPass2->Output());
		_output_1 = _output_1 * _gain / _norm;
		
			}
} //	end of namespace Aud
///EOF
