/***************************************************/
/*! \class Sinusoide
 \brief
 
 The AudioDrome Sinusoide class,
 implements a simple sinusoidal 
 generator based on the two methods:
 No recursive and Goertzel.
 
 by Alessandro Petrolati 2008.
 www.alessandro-petrolati.com
 info@alessandro-petrolati.it
 
 This library is free software; you can redistribute it and/or
 modify it under the terms of the GNU Library General Public
 License as published by the Free Software Foundation; either
 version 2 of the License, or (at your option) any later version.
 
 This library is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 Library General Public License for more details.
 
 You should have received a copy of the GNU Library General Public
 License along with this library; if not, write to the Free
 Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
/***************************************************/

#include "Sinusoide.h"

using namespace Aud; 
using namespace Aud;

Sinusoide::Sinusoide(const AudFloat& amp, const AudFloat& freq,  const AudFloat& phi, const ALGORITHM& type)
{
#if __DEBUG__
	std::cout << "Sinusoide Construct: "<< this <<std::endl;
#endif
	
	this->setAmplitude(amp);
	this->setFrequency(freq);
	this->setAlgorithm(type);
	this->setPhase(phi);
	_q0 = 0.0;
	for(int i = 0; i < 2; i++)
		_q1[i] = _q2[i] = 0.0;
	
	_q1[0] = 0;
	_q2[1] = sin (_coeff);
}


Sinusoide::~Sinusoide()
{
#if __DEBUG__
	std::cout << "Sinusoide Destruct: "<< this <<std::endl;
#endif
	
}

void Sinusoide::setAlgorithm(const ALGORITHM& type)
{
	switch (type)
	{
		case (NONRECURSIVE):
			_oscPtr = &Sinusoide::nonRecursive;
			break;
		case (GOERTZEL):
			_oscPtr = &Sinusoide::Goertzel;
			break;
		default:
			_oscPtr = &Sinusoide::nonRecursive;
	}
}

void Sinusoide::setFrequency(const AudFloat& freq) /*! Sampling Rate Change SLOT */
{
	_freq = freq;
	_phaseInc = _freq / _localSR;
	_coeff = (M_TWO_PI * _freq) / _localSR;
}

void Sinusoide::Processing()
{
	_output_1 = (this->*_oscPtr)() * _amp;
}

AudFloat Sinusoide::nonRecursive ()
{
	_phase += _phaseInc;
	if( _phase >= 1.0 ) 
		_phase -= 1.0;
	return  sin( _phase * M_TWO_PI);				
}

AudFloat  Sinusoide::Goertzel ()
{
	for (int n = 0; n <  2; n++ )
	{
		_q0 = 2.0 * cos(_coeff) * _q1[n] - _q2[n];
		_q2[n] = _q1[n];
		_q1[n] = _q0;
	}
	return _q0;
} //	end of namespace Aud
///EOF
