/***************************************************/
/*! \class Trapezoid
 \brief
 
 The AudioDrome Trapezoid class, implements 
 a trapezoidal envelope generator, based on the control 
 of the attack and decay.
 The values of attack and decay,
 are expressed as a percentage of total duration.
 therefore, the "sustain" value, is implicit:
 
 _sus = _dur - (_attack + _decay);
 
 by Alessandro Petrolati 2006.
 www.alessandro-petrolati.it
 info@alessandro-petrolati.it
 
 This library is free software; you can redistribute it and/or
 modify it under the terms of the GNU Library General Public
 License as published by the Free Software Foundation; either
 version 2 of the License, or (at your option) any later version.
 
 This library is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 Library General Public License for more details.
 
 You should have received a copy of the GNU Library General Public
 License along with this library; if not, write to the Free
 Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
/***************************************************/

#include "Trapezoid.h"

using namespace Aud; 

//  begin namespace
namespace Aud
{
	Trapezoid::Trapezoid() : _trig(true)
	{
#if __DEBUG__
		std::cout << "Trapezoid Constructor: " << this <<std::endl;
#endif
		//    this->setDuration (.1);
		this->setAttack(.25);
		this->setDecay(.25);
		
	}
	
	Trapezoid::~Trapezoid()
	{
#if __DEBUG__
		std::cout << "Trapezoid Destructor: " << this <<std::endl;
#endif
	}
	
	void Trapezoid::setParameters(const AudFloat &dur, const AudFloat &attack,
								  const AudFloat &decay)
	{
		this->setDuration(dur);
		this->setAttack(attack);
		this->setDecay(decay);
	}
	
	void Trapezoid::setDuration(const AudFloat &dur)
	{
		Phasor::setFrequency (1.0/(dur <= 0 ? .0001 : dur));
		this->setAttack(_attack);
		this->setDecay(_decay);
	}
	
	void Trapezoid::setAttack(const AudFloat &attack)
	{
		///da 0 a .5 (50 % of envelope Duration)
		_attack = (attack < .50 ? (attack < 0.0 ? 0.0 : attack) : .50);
		_sus = 1.0 - (_attack + _decay);
		
	}
	
	void Trapezoid::setDecay(const AudFloat &decay)
	{
		///da 0 a .5 (50 % of envelope Duration)
		_decay = (decay < .50 ? (decay < 0.0 ? 0.0 : decay) : .50);
		_sus = 1.0 - (_attack + _decay);
		
	}
	
	void Trapezoid::trigger()
	{
		_trig = true;
		Phasor::trigger();
	}
	
	void Trapezoid::Processing()
	{
		if (_trig)
		{
			Phasor::Processing();
			if(Phasor::_trigger)
			{
				_trig = false;
				_output_1 = 0.0;
				return;
			}
			if (_phi < _attack)
				_output_1 = _phi/_attack;
			else if (_phi < _attack + _sus)
				_output_1 = 1.0;
			else
			{ 
				_output_1 = (1.0 - _phi) / _decay; /// (1 - _attack - _sus);
			}
		}
		else
			_output_1 = 0.0;
	}
} //	end of namespace Aud
///EOF
