/***************************************************/
/*! \class Wave
 \brief
 
 The AudioDrome Wave is a specialization of Buffer.
 Mainly methods of this class, are used to generate 
 prototypal Waveform
 
 by Alessandro Petrolati 2006.
 www.alessandro-petrolati.it
 info@alessandro-petrolati.it
 
 This library is free software; you can redistribute it and/or
 modify it under the terms of the GNU Library General Public
 License as published by the Free Software Foundation; either
 version 2 of the License, or (at your option) any later version.
 
 This library is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 Library General Public License for more details.
 
 You should have received a copy of the GNU Library General Public
 License along with this library; if not, write to the Free
 Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
/***************************************************/

#include "Wave.h"

using namespace Aud; 

//  begin namespace
namespace Aud
{
	Wave::Wave(const unsigned long &length, const WAVESHAPE& type) :
	Buffer(length)
	{
#if __DEBUG__
		std::cout << "Wave Constructor: "<< this<<std::endl;
#endif
		
		switch (type)
		{
			case (SINE):
				this->genSine();
				break;
			case (COSINE):
				this->genCosine();
				break;
			case (SAWTOOTH):
				this->genSaw();
				break;
			case (SQUARE):
				this->genSquare();
				break;
			case (TRIANGLE):
				this->genTriangle();
				break;
			case (SIGMOID):
				this->genSigmoid();
				break;
			case (DOUBLESIGMOID):
				this->genDoubleSigmoid();
				break;
			default:
				this->genSine();
		}
	}
	
	Wave::~Wave()
	{
#if __DEBUG__
		std::cout << "Wave Destructor: "<< this<<std::endl;
#endif
	}
	
	void Wave::genArm(const unsigned int &numArm, const AudFloat &ampArm,
					  const AudFloat &phArm, const AudFloat &offSet)
	{
		unsigned long ph = (unsigned long) (phArm * _size);
		for (unsigned long i = 0; i < _size; i++)
			*(_table+i) += offSet + (ampArm * sin((double)numArm * (2.0 * M_PI)
												  * (double)(ph + i) / _size));
		
		writeGuardPoint();
	}
	
	void Wave::genSine()
	{
		for (unsigned long i = 0; i < _size; i++)
			*(_table+i) = (AudFloat) sin(2.0 * M_PI * (double) i / _size);
		writeGuardPoint();
	}
	
	void Wave::genCosine()
	{
		for (unsigned long i = 0; i < _size; i++)
			*(_table+i) = (AudFloat) cos(2.0 * M_PI* (double) i / _size);
		writeGuardPoint();
	}
	
	void Wave::genSquare()
	{
		for (unsigned int i = 0; i < _size; i++)
		{
			if (i < _size * .5)
				*(_table+i) = 1.0;
			else
				*(_table+i) = -1.0;
		}
		writeGuardPoint();
	}
	
	void Wave::genSaw()
	{
		for (unsigned long i = 0; i < _size; i++)
			*(_table+(int)i) = (AudFloat) 2.0 * (i/ (AudFloat)_size) - 1.0;
		writeGuardPoint();
	}
	
	void Wave::genTriangle()
	{
		for (unsigned long i = 0; i < _size; i++)
		{
			if (i < _size * .5)
				*(_table+(int)i) = (AudFloat) 2.0 * (i / (AudFloat)(_size * .5))
				- 1.0;
			else
				*(_table+(int)i) = (AudFloat) 2.0 * (2.0 - (i / (AudFloat)(_size
																		   * .5)))- 1.0;
		}
		writeGuardPoint();
	}
	
	void Wave::genSigmoid()
	{
		for (unsigned long i = 0; i < _size; i++)
		{
			double x = (double) (_size - i) / _size;
			*(_table+i) = (AudFloat) cos(M_PI * x);
		}
		writeGuardPoint();
	}
	
	void Wave::genDoubleSigmoid()
	{
		unsigned long size = _size / 2;
		for (unsigned long i = 0; i < size; i++)
		{
			double x = (double) (size - i) / size;
			*(_table+i) = (AudFloat) (cos(M_PI * x) * 0.5) - 0.5;
			
			unsigned long indx = i + size;
			*(_table+indx) = (AudFloat) (cos(M_PI * x) * 0.5) + 0.5;
		}
		writeGuardPoint();
	}
} //	end of namespace Aud
///EOF
