/***************************************************/
/*! \class Window
 \brief
 
 The AudioDrome Window is a specialization of Buffer.
 Mainly methods of this class, are used to generates
 functions of different windows.
 These windows are usually used for spectrum analysis or 
 for grain envelope.
 
 by Alessandro Petrolati 2006.
 www.alessandro-petrolati.it
 info@alessandro-petrolati.it
 
 This library is free software; you can redistribute it and/or
 modify it under the terms of the GNU Library General Public
 License as published by the Free Software Foundation; either
 version 2 of the License, or (at your option) any later version.
 
 This library is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 Library General Public License for more details.
 
 You should have received a copy of the GNU Library General Public
 License along with this library; if not, write to the Free
 Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
/***************************************************/

#include "Window.h"

using namespace Aud; 

//  begin namespace
namespace Aud
{
	Window::Window(unsigned int length, const WINDOWS& type) :
	Buffer(length)
	{
#if __DEBUG__
		std::cout << "Window Constructor: "<< this<<std::endl;
#endif
		switch (type)
		{
			case (BARLETT):
				this->genBarlett();
				break;
			case (GAUSSIAN):
				this->genGaussian();
				break;
			case (HANNING):
				this->genHanning();
				break;
			case (HAMMING):
				this->genHamming();
				break;
			case (BLACKMANN):
				this->genBlackman();
				break;
			case (BLACKMANNHARRIS):
				this->genBlackmanHarris();
				break;
			case (BARLETTHANNING):
				this->genBarlettHanning();
				break;
			case (NUTTALL):
				this->genNuttall();
				break;
			case (SINC):
				this->genSync();
				break;
			default:
				this->genGaussian();				
		}
	}
	
	Window::~Window()
	{
#if __DEBUG__
		std::cout << "Window Destructor: "<< this<<std::endl;
#endif
	}
	
	void Window::genRectangular()
	{
		for (unsigned long i = 0; i < _size; i++)
			*(_table+i) = 1.0;
		writeGuardPoint();
	}
	
	void Window::genBarlett()
	{
		unsigned long i;
		AudFloat incr = 2.0/_size;
		for (i = 0; i < (_size >> 1); i++)
			*(_table+i) = (AudFloat) (i * incr);
		for (; i < _size; i++, i++)
			*(_table+i) = (AudFloat) (2.0 - i * incr) ;
		writeGuardPoint();
	}
	
	void Window::genGaussian()
	{
		unsigned long i;
		AudFloat x;
		AudFloat incr = 12.0 / _size;
		for (i = 0, x = -6.0; i < (_size >> 1); i++, x += incr)
			*(_table+i) = (AudFloat) ((pow(2.718281828459, (double)-(x*x) / 2.0)));
		for (x = 0.0; i <= _size; i++, x += incr)
			*(_table+i) = (AudFloat) ((pow(2.718281828459, (double)-(x*x) / 2.0)));
		writeGuardPoint();
	}
	
	void Window::genHanning()
	{
		AudFloat A0 = .5;
		AudFloat A1 = .5;
		for (unsigned int i = 0; i < _size; i++)
		{
			*(_table+i) = (AudFloat) A0 - A1 * cos(2.0 * M_PI * (double) i / _size);
		}
		writeGuardPoint();
	}
	
	void Window::genHamming()
	{
		AudFloat A0 = .54;
		AudFloat A1 = .46;
		for (unsigned int i = 0; i < _size; i++)
		{
			*(_table+i) = (AudFloat) A0 - A1 * cos(2.0 * M_PI * (double) i / _size);
		}
		writeGuardPoint();
	}
	
	void Window::genBlackman()
	{
		AudFloat A0 = .42;
		AudFloat A1 = .5;
		AudFloat A2 = .08;
		
		for (unsigned int i = 0; i < _size; i++)
		{
			*(_table+i) = (AudFloat) A0 - A1 * cos(2.0 * M_PI* i / _size) + A2
			* cos(4 * M_PI * (double) i / _size);
		}
		writeGuardPoint();
	}
	
	void Window::genBlackmanHarris()
	{
		AudFloat A0 = 0.35875;
		AudFloat A1 = 0.48829;
		AudFloat A2 = 0.14128;
		AudFloat A3 = 0.01168;
		for (unsigned int i = 0; i < _size; i++)
		{
			*(_table+i) = (AudFloat) A0 - A1 * cos((2.0 * M_PI) * i / _size) + A2
			* cos((4.0 * M_PI) * (double)i / _size) -A3 * cos((6.0 * M_PI)
															  * (double)i / _size);
		}
		writeGuardPoint();
	}
	
	void Window::genBarlettHanning()
	{
		AudFloat A0 = 0.62;
		AudFloat A1 = 0.48;
		AudFloat A2 = 0.38;
		for (unsigned int i = 0; i < _size; i++)
		{
			*(_table+i) = (AudFloat) A0 - A1 * ((double)(i / _size) - 0.5) + A2
			* cos((2.0 * M_PI) * ((i / _size) - 0.5));
		}
		writeGuardPoint();
	}
	
	void Window::genNuttall()
	{
		AudFloat A0 = 0.3635819;
		AudFloat A1 = 0.4891775;
		AudFloat A2 = 0.1365995;
		AudFloat A3 = 0.0106411;
		for (unsigned int i = 0; i < _size; i++)
		{
			*(_table+i) = (AudFloat) A0 - A1
			* cos((2.0 * M_PI) * (double)i / _size) + A2 * cos((4.0 * M_PI)
															   * (double) i / _size) -A3 * cos((6.0 * M_PI) * (double)i
																							   / _size);
		}
		writeGuardPoint();
	}
	
	void Window::genSync(const AudFloat &value)
	{
		unsigned long i;
		AudFloat x;
		AudFloat arg = (2.0 * M_PI) / _size;
		for (i = 0, x = -M_PI; i < (_size >> 1); i++, x += arg)
			*(_table+i) = (AudFloat) (value * sin(x) / x);
		*(_table+i++) = 1.0;
		for (x = arg; i <= _size; i++, x += arg)
			*(_table+i) = (AudFloat) (value * sin(x) / x);
		writeGuardPoint();
	}
} //	end of namespace Aud
///EOF
